/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"


/* furniture types */
const char *furn_types[NUM_FURN_TYPES] =
{
  "Throne",
  "Chair",
  "Table",
  "Bookcase",
  "Bed",
  "Counter",
  "Desk",
  "\n"
};

/* types of doors */
const char *door_flags[NUM_DOOR_FLAGS + 1] =
{
  "Is a door",
  "Closed (R)",
  "Lockable",
  "Pickproof",
  "Secret",
  "Blocked",
  "Walled (R)",
  "Extra Wide",
  "Staircase",
  "No Walls",
  "Not Shown",
  "Ladder",
  "Made of Stone",
  "Made of Iron",
  "Smashed Open",
  "No Mobs Pass",
  "Spiked",
  "Thick Wood",
  "Thick Stone",
  "Thick Metal",
  "Bars/Gate",
  "\n"
};

/* cardinal directions */
const char *dirs[] =
{
  "north",
  "east",
  "south",
  "west",
  "up",
  "down",
  "northeast",
  "northwest",
  "southeast",
  "southwest",
  "\n"
};


/* cardinal directions */
const char *dirs_look[] =
{
  "to your north",
  "to your east",
  "to your south",
  "to your west",
  "above you",
  "below you",
  "to your northeast",
  "to your northwest",
  "to your southeast",
  "to your southwest",
  "\n"
};

/* cardinal directions */
const char *distances_look[] =
{
  "here",
  "just",
  "close by",
  "a span",
  "a couple spans",
  "numerous spans",
  "in the distance",
  "almost out of sight",
  "\n"
};

/* ROOM_MAP_SPECIALS_xx */
const char *room_map_specials[] =
{
  "NONE",
  "COLUMNS",
  "RIVER",
  "\n"
};

/* ROOM_SHAPE_x */
const char *room_shapes[] =
{
  "Square (olc only)",
  "Circle (olc only)",
  "Octagon (olc only)",
  "Rough/Cavern (olc only)",
  "Wilderness Map",
  "Room Map",
  "\n"
};

/* ROOM_x */
const char *room_bits[] =
{
  "DARK",
  "DEATH",
  "!MOB",
  "INDOORS",
  "PEACEFUL",
  "SOUNDPROOF",
  "!TRACK",
  "!MAGIC",
  "TUNNEL",
  "PRIVATE",
  "GODROOM",
  "HOUSE",
  "HCRSH",
  "ATRIUM",
  "OLC",
  "*",				/* BFS MARK */
  "BIT_16",
  "BIT_17",
  "BIT_18",
  "BIT_19",
  "BIT_20",
  "BIT_21",
  "BIT_22",
  "BIT_23",
  "BIT_24",
  "BIT_25",
  "BIT_26",
  "BIT_27",
  "\n"
};


/* SECT_ */
const char *sector_types[] =
{
  "Inside",
  "City",
  "Field",
  "Forest",
  "Hills",
  "Mountains",
  "Water (Swim)",
  "Water (No Swim)",
  "Underwater",
  "In Flight",
  "Roadway",
  "Swamp",
  "Cavern/UnderGround",
  "\n"
};


/* SEX_x */
const char *genders[] =
{
  "Neutral",
  "Male",
  "Female"
};

/* POS_x */
const char *position_types[] =
{
  "prone",
  "kneeling",
  "sitting",
  "standing",
  "dead",
  "mortally wounded",
  "incapacitated",
  "sleeping",
  "resting",
  "normal",
  "\n"
};

/* PLR_x */
const char *player_bits[] =
{
  "UNUSED",
  "UNUSED",
  "FROZEN",
  "DONTSET",
  "WRITING",
  "MAILING",
  "CSH",
  "SITEOK",
  "NOSHOUT",
  "NOTITLE",
  "DELETED",
  "LOADRM",
  "!WIZL",
  "!DEL",
  "INVST",
  "CRYO",
  "\n"
};


/* MOB_x */
const char *action_bits[] =
{
  "SPEC",
  "SENTINEL",
  "SCAVENGER",
  "ISNPC",
  "AWARE",
  "AGGR",
  "STAY-ZONE",
  "WIMPY",
  "AGGR_EVIL",
  "AGGR_GOOD",
  "AGGR_NEUTRAL",
  "MEMORY",
  "HELPER",
  "!CHARM",
  "!SUMMN",
  "!SLEEP",
  "!BASH",
  "!BLIND",
  "MOUNTABLE"
  "\n"
};


/* PRF_x */
const char *preference_bits[NUM_PREF + 1] =
{
  "COMPACT",
  "DEAF",
  "!TELL",
  "!HASS",
  "!REP",
  "LIGHT",
  "!WIZ",
  "L1",
  "L2",
  "RMFLG",
  "GLDTLK",
  "\n"
};

/* AFF_x */
const char *affected_bits[] =
{
  "BLIND",
  "INVISIBLE",
  "DETECT_ALIGN",
  "DETECT_INVIS",
  "HASTE",
  "SENSE_LIFE",
  "MINOR_GLOBE",
  "TAMED",
  "SANCTUARY",
  "GROUP",
  "SCRIBING",
  "WATERBREATH",
  "CURSE",
  "PROTECT_EVIL",
  "POISON",
  "DETECT_MAGIC",
  "PROTECT_GOOD",
  "SLEEP",
  "WATERWALK",
  "SNEAK",
  "HIDE",
  "FEAR",
  "CHARM",
  "MEDITATE",
  "BARKSKIN",
  "INFRAVISION",
  "LEVITATE",
  "FLY",
  "NOTRACK",
  "PROT_FIRE",
  "CAMPING",
  "MEMORIZING",
  "\n"
};

/* affected_by2 */
const char *affected2_bits[] =
{
  "FIRESHIELD",
  "ULTRAVISION",
  "DETECT_EVIL",
  "DETECT_GOOD",
  "SLOW_POISON",
  "CHANGE SELF",
  "PROT_COLD",
  "PROT_LIGHTNING",
  "MINOR_PARALYSIS",
  "MAJOR_PARALYSIS",
  "SLOW",
  "GLOBE",
  "PROT_GAS",
  "PROT_ACID",
  "POISONED",
  "COMPREHEND LANG",
  "SILENCED",
  "MINOR_INVIS",
  "GAZE REFLECTION",
  "STUNNED",
  "DROPPED_PRIM",
  "DROPPED_SECOND",
  "FUMBLING_PRIM",
  "FUMBLING_SECOND",
  "HOLDING_BREATH",
  "SHIELD",
  "IS_DROWNING",
  "PASSDOOR",
  "DRAINING",
  "CASTING",
  "SPIDER_CLIMB",
  "HUNTER",
  "\n"
};

/* affected_by 3 */
const char *affected3_bits[] =
{

  "TENSORS_DISC",
  "TRACKING",
  "SINGING",
  "ALTER SELF",
  "BLUR",
  "BLINK",
  "HOLD",
  "POLYMORPH",
  "NON_DETECTION",
  "SILVER",
  "PLUSONE",
  "PLUSTWO",
  "PLUSTHREE",
  "PLUSFOUR",
  "PLUSFIVE",
  "ENLARGE",
  "REDUCE",
  "COVER",
  "FOUR_ARMS",
  "ANTI-MAGIC SHELL",
  "TRUE SIGHT",
  "COLDSHIELD",
  "MIND BLANK",
  "SWIMMING",
  "SHAPE CHANGE",
  "UNDERWATER",
  "TIME STOP",
  "NECKBITING",
  "ELEMENTAL_FORM",
  "PASS_WITHOUT_TRACE",
  "DETECT POISON",
  "FREE ACTION",
  "\n"
};

const char *affected4_bits[] =
{
  "LOOTER",
  "CARRY_DISEASE",
  "SACKING",
  "SENSE_FOLLOWER",
  "WIND WALK",
  "",
  "VAMPIRE_FORM",
  "NO_UNMORPH",
  "FAERIE FIRE",
  "FEINT",
  "GALLOP",
  "",
  "THROAT_CRUSH",
  "REGENERATION",
  "BEARHUG",
  "GRAPPLE",
  "GRAPPLED",
  "",
  "WRAITHFORM",
  "",
  "",
  "",
  "KNOCKED_OUT",
  "BOUND",
  "",
  "STONE_SKIN",
  "",
  "",
  "",
  "",
  "",
  "",
  "\n"
};

/* CON_x */
const char *connected_types[NUM_CON + 1] =
{
  "Playing",
  "Disconnecting",
  "Get name",
  "Confirm name",
  "Get password",
  "Get new PW",
  "Confirm new PW",
  "Reading MOTD",
  "Main Menu",
  "Get descript.",
  "Changing PW 1",
  "Changing PW 2",
  "Changing PW 3",
  "Self-Delete 1",
  "Self-Delete 2",
  "Object edit",
  "Room edit",
  "Zone edit",
  "Mobile edit",
  "Shop edit",
  "Clan edit",
  "Quest edit",
  "Help edit",
  "Player Edit",
  "Action Edit",
  "Terminal",
  "Trigger Edit",
  "\n"
};


/* WEAR_x - for eq list */
const char *where[NUM_WEARS + 5] =
{
  "<worn as badge>      ",
  "<worn on head>       ",
  "<worn on face>       ",
  "<worn on eyes>       ",
  "<worn on ear>        ",
  "<worn on ear>        ",
  "<worn around neck>   ",
  "<worn on body>       ",
  "<worn about body>    ",
  "<worn on back>       ",
  "<worn on arms>       ",
  "<worn around wrist>  ",
  "<worn around wrist>  ",
  "<worn on finger>     ",
  "<worn on finger>     ",
  "<worn on hands>      ",
  "<worn about waist>   ",
  "<attach to belt>     ",
  "<attach to belt>     ",
  "<attach to belt>     ",
  "<worn as shield>     ",
  "<wielded>            ",
  "<wielded>            ",
  "<held>               ",
  "<held>               ",
  "<worn on legs>       ",
  "<worn on feet>       ",
  // These three MUST be at the end, at positions NUM_WEAR 
  // and NUM_WEAR+1
  "<right hand>         ",
  "<left hand>          ",
  "<wielding two-handed>",
  "<holding two-handed> ",
  "\n"
};


/* WEAR_x - for stat */
const char *equipment_types[NUM_WEARS + 1] =
{
  "Worn as badge",
  "Worn on head",
  "Worn on face",
  "Worn on eyes",
  "Worn on right ear",
  "Worn on left ear",
  "Worn around neck",
  "Worn on body",
  "Worn about body",
  "Worn on back",
  "Worn on arms",
  "Worn around right wrist",
  "Worn around left wrist",
  "Worn on right finger",
  "Worn on left finger",
  "Worn on hands",
  "Worn around waist",
  "Attach to belt",
  "Attach to belt",
  "Attach to belt",
  "Worn as shield",
  "Wielded",
  "Wielded",
  "Held",
  "Held",
  "Worn on legs",
  "Worn on feet",
  "\n"
};


/* ITEM_x (ordinal object types) */
const char *item_types[] =
{
  "UNDEFINED",
  "LIGHT",
  "SCROLL",
  "WAND",
  "STAFF",
  "WEAPON",
  "FIRE WEAPON",
  "MISSILE",
  "TREASURE",
  "ARMOR",
  "POTION",
  "WORN",
  "OTHER",
  "TRASH",
  "TRAP",
  "CONTAINER",
  "NOTE",
  "LIQ CONTAINER",
  "KEY",
  "FOOD",
  "MONEY",
  "PEN",
  "BOAT",
  "FOUNTAIN",
  "BOARD",
  "BOOK",
  "WALL",
  "CORPSE",
  "TELEPORT",
  "TIMER",
  "VEHICLE",
  "SHIP",
  "SWITCH",
  "QUIVER",
  "PICK",
  "INSTRUMENT",
  "FURNITURE",
  "STORAGE",
  "SCABBARD",
  "SHIELD",
  "TROOP",
  "HOLY SYMBOL",
  "PIT/TRAPDOOR",
  "VIRTUAL DOOR",
  "\n"
};



/* ITEM_WEAR_ (wear bitvector) */
const char *wear_bits[] =
{
  "TAKE",
  "FINGER",
  "NECK",
  "BODY",
  "HEAD",
  "LEGS",
  "FEET",
  "HANDS",
  "ARMS",
  "SHIELD",
  "ABOUT",
  "WAIST",
  "WRIST",
  "WIELD",
  "HOLD",
  "LIGHT",
  "EYES",
  "FACE",
  "EARRING",
  "BADGE",
  "BACK",
  "ATTACH",
  "\n"
};

const char *item_size_types[] =
{
  "none",
  "tiny",
  "small",
  "small-medium",
  "medium",
  "medium-large",
  "large",
  "huge",
  "giant",
  "gargantuan",
  "magical",
  "\n"
};

const char *size_types[] =
{
  "none",
  "tiny",
  "small",
  "small-medium",
  "medium",
  "medium-large",
  "large",
  "huge",
  "giant",
  "gargantuan",
  "\n"
};

const char *extra_bits[] =
{
  "GLOW",
  "NOSHOW",
  "BURIED",
  "NOSELL",
  "EVIL",
  "INVISIBLE",
  "MAGIC",
  "NODROP",
  "BLESS",
  "ANTI_GOOD",
  "ANTI_EVIL",
  "ANTI_NEUTRAL",
  "SECRET",
  "FLOAT",
  "NOBURN",
  "NOLOCATE",
  "NOIDENTIFY",
  "NOSUMMON",
  "LIT",
  "TRANSIENT",
  "NOSLEEP",
  "NOCHARM",
  "TWOHANDS",
  "NORENT",
  "GOOD",
  "HUM",
  "LEVITATES",
  "ONEORTWOHANDS",
  "",
  "WHOLE_BODY",
  "WHOLE_HEAD",
  "WAS_DISARMED (R)",
  "\n"
};

const char *extra2_bits[] =
{
  "SILVER",
  "PLUSONE",
  "PLUSTWO",
  "PLUSTHREE",
  "PLUSFOUR",
  "PLUSFIVE",
  "RETURNING",
  "CAN_THROW1",
  "CAN_THROW2",
  "REMOVE 2HAND (R)",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "ENLARGE",
  "REDUCE",
  "\n"
};

/* Bitvector for 'anti_flags' */
const char *anti_bits[] =
{
  "ALLOW_ALL",
  "ALLOW_CLERIC",
  "ALLOW_DRUID",
  "ALLOW_FIGHTER",
  "ALLOW_PALADIN",
  "ALLOW_RANGER",
  "ALLOW_MAGIC_USER",
  "ALLOW_ILLUSIONIST",
  "ALLOW_THIEF",
  "ALLOW_BARD",
  "\n"
};


/* APPLY_x */
const char *apply_types[] =
{
  "NONE",
  "STR",
  "DEX",
  "INT",
  "WIS",
  "CON",
  "CHA",
  "CLASS",
  "LEVEL",
  "AGE",
  "CHAR_WEIGHT",
  "CHAR_HEIGHT",
  "HIT",
  "MAGIC_RESISTANCE",
  "EXP",
  "ARMOR",
  "HITROLL",
  "DAMROLL",
  "SAVING_PARA",
  "SAVING_ROD",
  "SAVING_PETRI",
  "SAVING_BREATH",
  "SAVING_SPELL",
  "SAVING_ALL",
  "HEARING",
  "VISION",
  "\n"
};


/* CONT_x */
const char *container_bits[] =
{
  "CLOSEABLE",
  "PICKPROOF",
  "CLOSED",
  "LOCKED",
  "\n",
};


/* LIQ_x */
const char *drinks[] =
{
  "water",
  "beer",
  "wine",
  "ale",
  "dark ale",
  "whisky",
  "lemonade",
  "firebreather",
  "local speciality",
  "slime mold juice",
  "milk",
  "tea",
  "coffee",
  "blood",
  "salt water",
  "clear water",
  "\n"
};


/* other constants for liquids ***************************************** */


/* one-word alias for each drink */
const char *drinknames[] =
{
  "water",
  "beer",
  "wine",
  "ale",
  "ale",
  "whisky",
  "lemonade",
  "firebreather",
  "local",
  "juice",
  "milk",
  "tea",
  "coffee",
  "blood",
  "salt",
  "water",
  "\n"
};


/* effect of drinks on hunger, thirst, and drunkenness -- see values.doc */
const int drink_aff[][3] =
{
  {0, 1, 10},
  {3, 2, 5},
  {5, 2, 5},
  {2, 2, 5},
  {1, 2, 5},
  {6, 1, 4},
  {0, 1, 8},
  {10, 0, 0},
  {3, 3, 3},
  {0, 4, -8},
  {0, 3, 6},
  {0, 1, 6},
  {0, 1, 6},
  {0, 2, -1},
  {0, 1, -2},
  {0, 0, 13}
};


/* color of the various drinks */
const char *color_liquid[] =
{
  "clear",
  "brown",
  "clear",
  "brown",
  "dark",
  "golden",
  "red",
  "green",
  "clear",
  "light green",
  "white",
  "brown",
  "black",
  "red",
  "clear",
  "crystal clear"
};


/* level of fullness for drink containers */
const char *fullness[] =
{
  "less than half ",
  "about half ",
  "more than half ",
  ""
};

const int shield_pr[] =
{
  1, 2, 3, 4
};

//weapon applies
const struct weapon_app_type weapon_app[] =
{
  //{mod,type,swing,thrust,range,parry,max dam}
  {0,0,-20,-20,1,0,0},  //undefined, hit, punch, animal attacks
  {-5,STK_CUT,3,-20,2,0,0}, //battle axe
  {0,STK_IMPALE,-20,-20,10,0,0}, //bow
  {-5,STK_CRUSH,1,-20,2,0,0}, //club
  {0,STK_IMPALE,-20,-20,10,0,0}, //crossbow
  {-4,STK_IMPALE,-20,-1,1,-2,4}, //dagger
  {0,STK_IMPALE, -20,-20,10,0,0}, //dart
  {-6,STK_CRUSH,4,-20,2,-20,0},  //flail
  {-5,STK_CRUSH,3,-20,2,0,0}, //mace
  {-5,STK_IMPALE,1,-20,2,0,0},  //pick
  {-5,STK_CRUSH,3,-20,2,0,0}, //hammer
  {-5,STK_CUT,2,-20,2,0,0},  //hand axe
  {-5,STK_IMPALE,-20,3,3,0,0},  //harpoon
  {-5,STK_IMPALE,-20,1,3,0,0}, //javelin
  {-4,STK_CUT_IMP,-2,0,1,-2,5}, //knife
  {0,0,0,0,0,0,0}, //lance
  {0,0,0,0,0,0,0}, //mancatcher
  {-6,STK_CRUSH,3,-20,2,-20,0},  //morning star
  {-5,STK_CUT,4,-20,4,0,0},  //polearm
  {-5,STK_CRUSH,2,2,2,2,0},  //staff
  {0,0,0,0,0,0,0},  //scrouge
  {0,0,0,0,0,0,0}, //sickle
  {0,0,0,0,0,0,0},  //sling
  {-5,STK_IMPALE,-20,2,3,0,0}, //spear
  {0,0,0,0,0,0,0}, //staff sling
  {-5,STK_CUT_IMP,1,2,2,0,0},  //bastard sword
  {-5,STK_CUT_IMP,1,1,2,0,0}, //broadsword
  {-5,STK_CUT_IMP,-1,2,2,1,0},  //Khopesh
  {-5,STK_CUT_IMP,1,0,2,1,0}, //long sword
  {-5,STK_CUT_IMP,0,0,2,0,0}, //short sword
  {-5,STK_CUT_IMP,2,3,2,0,0},  //two handed sword
  {0,0,0,0,0,0,0},  //trident
  {0,0,0,0,0,0,0}, //whip
  {-5,STK_IMPALE,-20,1,2,2,4}, //rapier
  {-5,STK_CUT_IMP,0,1,2,2,5} //saber
};
/* str, int, wis, dex, con applies ************************************* */


/* [ch] strength apply (all) */
const struct str_app_type str_app[] =
{
  //{Thrust, Swing, weight carried, max weapon weight}
  {-5, -4, 0, 0},		/* str = 0 */
  {-5, -4, 3, 1},		/* str = 1 */
  {-3, -2, 3, 2},
  {-3, -1, 10, 3},
  {-5, -5, 25, 4},
  {-5, -5, 55, 5},		/* str = 5 */
  {-4, -4, 80, 6},
  {-3, -3, 90, 7},
  {-3, -2, 100, 8},
  {-2, -1, 100, 9},
  {-2, 0, 115, 10},		/* str = 10 */
  {-1, 1, 115, 11},
  {-1, 2, 140, 12},
  {0, 3, 140, 13},
  {0, 4, 170, 14},
  {1, 5, 170, 15},		/* str = 15 */
  {1, 6, 195, 16},
  {2, 7, 220, 18},
  {2, 8, 255, 20},		/* str = 18 */
  {5, 12, 640, 40},
  {5, 13, 700, 40},		/* str = 20 */
  {6, 14, 810, 40},
  {6, 15, 970, 40},
  {7, 16, 1130, 40},
  {8, 18, 1440, 40},
  {9, 20, 1750, 40},		/* str = 25 */
  {3, 9, 280, 22},		/* str = 18/0 - 18-50 */
  {3, 9, 305, 24},		/* str = 18/51 - 18-75 */
  {3, 10, 330, 26},		/* str = 18/76 - 18-90 */
  {4, 10, 380, 28},		/* str = 18/91 - 18-99 */
  {4, 11, 480, 30}		/* str = 18/100 */
};



/* [dex] skill apply (thieves only) */
const struct dex_skill_type dex_app_skill[] =
{
  {-99, -99, -90, -99, -60},	/* dex = 0 */
  {-90, -90, -60, -90, -50},	/* dex = 1 */
  {-80, -80, -40, -80, -45},
  {-70, -70, -30, -70, -40},
  {-60, -60, -30, -60, -35},
  {-50, -50, -20, -50, -30},	/* dex = 5 */
  {-40, -40, -20, -40, -25},
  {-30, -30, -15, -30, -20},
  {-20, -20, -15, -20, -15},
  {-15, -10, -10, -20, -10},
  {-10, -5, -10, -15, -5},	/* dex = 10 */
  {-5, 0, -5, -10, 0},
  {0, 0, 0, -5, 0},
  {0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0},		/* dex = 15 */
  {0, 5, 0, 0, 0},
  {5, 10, 0, 5, 5},
  {10, 15, 5, 10, 10},		/* dex = 18 */
  {15, 20, 10, 15, 15},
  {15, 20, 10, 15, 15},		/* dex = 20 */
  {20, 25, 10, 15, 20},
  {20, 25, 15, 20, 20},
  {25, 25, 15, 20, 20},
  {25, 30, 15, 25, 25},
  {25, 30, 15, 25, 25}		/* dex = 25 */
};



/* [dex] apply (all) */
struct dex_app_type dex_app[] =
{
  {-7, -7, 6},			/* dex = 0 */
  {-6, -6, 5},			/* dex = 1 */
  {-4, -4, 5},
  {-3, -3, 4},
  {-2, -2, 3},
  {-1, -1, 2},			/* dex = 5 */
  {0, 0, 1},
  {0, 0, 0},
  {0, 0, 0},
  {0, 0, 0},
  {0, 0, 0},			/* dex = 10 */
  {0, 0, 0},
  {0, 0, 0},
  {0, 0, 0},
  {0, 0, 0},
  {0, 0, -1},			/* dex = 15 */
  {1, 1, -2},
  {2, 2, -3},
  {2, 2, -4},			/* dex = 18 */
  {3, 3, -4},
  {3, 3, -4},			/* dex = 20 */
  {4, 4, -5},
  {4, 4, -5},
  {4, 4, -5},
  {5, 5, -6},
  {5, 5, -6}			/* dex = 25 */
};



/* [con] apply (all) */
struct con_app_type con_app[] =
{
  {-4, 20, 0},			/* con = 0 */
  {-3, 25, 0},			/* con = 1 */
  {-2, 30, 0},
  {-2, 35, 0},
  {-1, 40, 1},
  {-1, 45, 1},			/* con = 5 */
  {-1, 50, 1},
  {0, 55, 2},
  {0, 60, 2},
  {0, 65, 2},
  {0, 70, 2},			/* con = 10 */
  {0, 75, 3},
  {0, 80, 3},
  {0, 85, 3},
  {0, 88, 4},
  {1, 90, 4},			/* con = 15 */
  {2, 95, 4},
  {3, 97, 4},
  {4, 99, 5},			/* con = 18 */
  {5, 99, 5},
  {5, 99, 5},			/* con = 20 */
  {6, 99, 6},
  {6, 99, 6},
  {6, 99, 6},
  {7, 99, 6},
  {7, 99, 7}			/* con = 25 */
};



/* [int] apply (all) */
struct int_app_type int_app[] =
{
  {0, 0, 0, 0},			/* int = 0 */
  {0, 0, 0, 0},			/* int = 1 */
  {1, 0, 0, 0},
  {1, 0, 0, 0},
  {1, 0, 0, 0},
  {1, 0, 0, 0},
  {1, 0, 0, 0},
  {1, 0, 0, 0},
  {1, 0, 0, 0},			/* int = 8 */
  {2, 4, 35, 6},
  {2, 5, 40, 7},
  {2, 5, 45, 7},
  {3, 6, 50, 7},
  {3, 6, 55, 8},
  {4, 7, 60, 8},
  {4, 7, 65, 11},		/* int = 15 */
  {5, 8, 70, 11},
  {6, 8, 75, 14},
  {7, 9, 85, 18},
  {8, 9, 95, 99},
  {9, 9, 96, 99},		/* int = 20 */
  {10, 9, 97, 99},
  {11, 9, 98, 99},
  {12, 9, 99, 99},
  {15, 9, 100, 99},
  {20, 9, 100, 99}		/* int = 25 */
};


/* [wis] apply (all) */
struct wis_app_type wis_app[] =
{
  {-6, 80},			/* wis = 0 */
  {-6, 80},
  {-4, 60},
  {-3, 50},
  {-2, 45},
  {-1, 40},
  {-1, 35},
  {-1, 30},
  {0, 25},
  {0, 20},
  {0, 15},
  {0, 10},
  {0, 5},
  {0, 0},
  {0, 0},
  {1, 0},
  {2, 0},
  {3, 0},
  {4, 0},
  {4, 0},
  {4, 0},
  {4, 0},
  {4, 0},
  {4, 0},
  {4, 0},
  {4, 0}			/* wis = 25 */
};



const char *spell_wear_off_msg[] =
{
  "RESERVED DB.C",		/* 0 */
  "You feel less protected.",	/* 1 */
  "!Teleport!",
  "You feel less righteous.",
  "You feel a cloak of blindness disolve.",
  "!Burning Hands!",		/* 5 */
  "!Call Lightning",
  "You feel more self-confident.",
  "You feel your strength return.",
  "!Clone!",
  "!Color Spray!",		/* 10 */
  "!Control Weather!",
  "!Create Food!",
  "!Create Water!",
  "!Cure Blind!",
  "!Cure Critic!",		/* 15 */
  "!Cure Light!",
  "You feel more optimistic.",
  "You feel less aware.",
  "Your eyes stop tingling.",
  "The detect magic wears off.",	/* 20 */
  "The detect poison wears off.",
  "!Dispel Evil!",
  "!Earthquake!",
  "!Enchant Weapon!",
  "!Energy Drain!",		/* 25 */
  "!Fireball!",
  "!Harm!",
  "!Heal!",
  "You feel yourself exposed.",
  "!Lightning Bolt!",		/* 30 */
  "!Locate object!",
  "!Magic Missile!",
  "You feel less sick.",
  "You feel less protected.",
  "!Remove Curse!",		/* 35 */
  "The white aura around your body fades.",
  "!Shocking Grasp!",
  "You feel less tired.",
  "You feel weaker.",
  "!Summon!",			/* 40 */
  "!Ventriloquate!",
  "!Word of Recall!",
  "!Remove Poison!",
  "You feel less aware of your suroundings.",
  "!Animate Dead!",		/* 45 */
  "!Dispel Good!",
  "!Group Armor!",
  "!Group Heal!",
  "!Group Recall!",
  "Your night vision seems to fade.",	/* 50 */
  "Your feet seem less boyant.",
  "!UNUSED!"
};

const int rev_dir[] =
{
  2,
  3,
  0,
  1,
  5,
  4,
  9,
  8,
  7,
  6
};

/*
 * Definitions necessary for MobProg support in OasisOLC
 */
const char *mobprog_types[] =
{
  "ERROR",
  "INFILE",
  "ACT",
  "SPEECH",
  "RAND",
  "FIGHT",
  "DEATH",
  "HITPRCNT",
  "ENTRY",
  "GREET",
  "ALL_GREET",
  "GIVE",
  "BRIBE",
  "\n"
};

const char *weekdays[] =
{
  "Starday",
  "Sunday",
  "Moonday",
  "Godsday",
  "Waterday",
  "Earthday",
  "Freeday",
  "\n"
};


const char *month_name[] =
{
  "Month of Fireseek",
  "Month of Readying",
  "Month of Coldeven",
  "Month of Planting",
  "Month of Flocktime",
  "Month of Wealsun",
  "Month of Reaping",
  "Month of Goodmonth",
  "Month of Harvester",
  "Month of Patchwall",
  "Month of Ready'reat",
  "Month of Sunsebb",
  "\n"
};

const char *char_abils[] =
{
  "Strength",
  "Inteligence",
  "Wisdom",
  "Dexterity",
  "Constitution",
  "Charisma"
};

/* saving throws priests */
/* min level, max level, para, RSW, petri, breath, spell */
const int save_priest[][7] =
{
  {1, 3, 10, 14, 13, 16, 15},
  {4, 6, 9, 13, 12, 15, 14},
  {7, 9, 7, 11, 10, 13, 12},
  {10, 12, 6, 10, 9, 12, 11},
  {13, 15, 5, 9, 8, 11, 10},
  {16, 18, 4, 8, 7, 10, 9},
  {19, 30, 2, 6, 5, 8, 7},
  {31, 34, 2, 2, 2, 2, 2}
};

/* saving throws rogues */
/* min level, max level, para, RSW, petri, breath, spell */
const int save_rogue[][7] =
{
  {1, 4, 13, 14, 12, 16, 15},
  {5, 8, 12, 12, 11, 15, 13},
  {9, 12, 11, 10, 10, 14, 11},
  {13, 16, 10, 8, 9, 13, 9},
  {17, 20, 9, 6, 8, 12, 7},
  {21, 30, 8, 4, 7, 11, 5},
  {31, 34, 2, 2, 2, 2, 2}
};

/* saving throws warriors */
/* min level, max level, para, RSW, petri, breath, spell */
const int save_warrior[][7] =
{
  {0, 0, 16, 18, 17, 20, 19},
  {1, 2, 14, 16, 15, 17, 17},
  {3, 4, 13, 15, 14, 16, 16},
  {5, 6, 11, 13, 12, 13, 14},
  {7, 8, 10, 12, 11, 12, 13},
  {9, 10, 8, 10, 9, 9, 11},
  {11, 12, 7, 9, 8, 8, 10},
  {13, 14, 5, 7, 6, 5, 8},
  {15, 16, 4, 6, 5, 4, 7},
  {17, 30, 3, 5, 4, 4, 6},
  {31, 34, 2, 2, 2, 2, 2}
};

/* saving throws wizards */
/* min level, max level, para, RSW, petri, breath, spell */
const int save_wizard[][7] =
{
  {1, 5, 14, 11, 13, 15, 12},
  {6, 10, 13, 9, 11, 13, 10},
  {11, 15, 11, 7, 9, 11, 8},
  {16, 20, 10, 5, 7, 9, 6},
  {21, 30, 8, 3, 5, 7, 4},
  {31, 34, 2, 2, 2, 2, 2}
};

const char *item_material[] =
{
  "UNDEFINED",
  "NONSUBSTANTIAL",
  "FLESH",
  "CLOTH",
  "BARK",
  "SOFTWOOD",
  "HARDWOOD",
  "SILICON",
  "CRYSTAL",
  "CERAMIC",
  "BONE",
  "STONE",
  "HIDE",
  "LEATHER",
  "CURED_LEATHER",
  "IRON",
  "STEEL",
  "BRASS",
  "MITHRIL",
  "ADAMANTIUM",
  "BRONZE",
  "COPPER",
  "SILVER",
  "ELECTRUM",
  "GOLD",
  "PLATINUM",
  "GEM",
  "DIAMOND",
  "PAPER",
  "PARCHMENT",
  "LEAVES",
  "RUBY",
  "EMERALD",
  "SAPPHIRE",
  "IVORY",
  "DRAGONSCALE",
  "OBSIDIAN",
  "GRANITE",
  "MARBLE",
  "LIMESTONE",
  "LIQUID",
  "BAMBOO",
  "REEDS",
  "HEMP",
  "GLASSTEEL",
  "EGGSHELL",
  "CHITINOUS",
  "REPTILESCALE",
  "GENERICFOOD",
  "RUBBER",
  "FEATHER",
  "WAX",
  "PEARL",
  "\n"
};

const char *material_names[] =
{
  "&=RLundefined&n",
  "&+La non-substantial material&n",
  "&+rflesh&n",
  "cloth",
  "&+ybark&n",
  "&+ysoft wood&n",
  "&+yhard wood&n",
  "&+Wglass&n",
  "&+gcrystal&n",
  "&+yclay&n",
  "&+Wbone&n",
  "&+Lstone&n",
  "&+yhide&n",
  "&+yleather&n",
  "&+ycured leather&n",
  "&+ciron&n",
  "&+Csteel&n",
  "&+ybrass&n",
  "&+mmithril&n",
  "&+Madamantium&n",
  "&+ybronze&n",
  "&+ycopper&n",
  "&+csilver&n",
  "&+Celectrum&n",
  "&+Ygold&n",
  "&+Wplatinum&n",
  "&+ggem&n",
  "&+Wdiamond&n",
  "&+Wpaper&n",
  "&+yparchment&n",
  "&+gleaves&n",
  "&+rruby&n",
  "&+gemerald&n",
  "&+bsapphire&n",
  "&+Wivory&n",
  "&+gdragonscale&n",
  "&+Lobsidian&n",
  "&+Lgranite&n",
  "&+Wmarble&n",
  "&+glimestone&n",
  "&+cliquid&n",
  "&+ybamboo&n",
  "&+greeds&n",
  "&+ghemp&n",
  "&+cglassteel&n",
  "&+Weggshell&n",
  "&+ychitin&n",
  "&+greptile scale&n",
  "&+Ygeneric food&n",
  "&+Lrubber&n",
  "&+Wfeather&n",
  "&+Wwax&n",
  "&+Wpearl&n"
};


const char *missile_types[] =
{
  "Normal Flight Arrow",
  "Composite Flight Arrow",
  "Normal Sheaf Arrow",
  "Composite Sheaf Arrow",
  "Hand Crossbow Quarrel",
  "Light Crossbow Quarrel",
  "Heavy Crossbow Quarrel",
  "Sling Stone",
  "Ballista Missile",
  "Catapult Missile",
  "\n"
};

const char *coin_names[] =
{
  "&+ycopper&N",
  "silver",
  "&+Lelectrum&N",
  "&+Ygold&N",
  "&+Wplatinum&N"
};

const char *coin_abbrev[] =
{
  "c",
  "s",
  "e",
  "g",
  "p"
};

const char *craftsmanship_names[] =
{
  "terribly made",
  "extremely poorly made",
  "very poorly made",
  "fairly poorly made",
  "of well below average quality",
  "of below average quality",
  "of slightly below average quality",
  "of average quality",
  "of slightly above average quality",
  "of above average quality",
  "of well above average quality",
  "excellently made",
  "made by a skilled artisian",
  "made by a very skilled artisian",
  "made by a master artisian",
  "of one-of-a-kind craftsmanship",
  "\n"
};

const char *alignment_types[] =
{
  "None",
  "Lawful/Good",
  "Neutral/Good",
  "Chaotic/Good",
  "Lawful/Neutral",
  "True Neutral",
  "Chaotic/Neutral",
  "Lawful/Evil",
  "Neutral/Evil",
  "Chaotic/Evil",
  "\n"
};

const char *helmet_types[] =
{
  "UNDEFINED",
  "CLOTH_CAP",
  "LEATHER_HELM",
  "CHAIN_COIF",
  "POT_HELM",
  "GREAT_HELM",
  "\n"
};

const char *shield_types[] =
{
  "UNDEFINED",
  "BUCKLER",
  "SMALL",
  "MEDIUM",
  "LARGE",
  "\n"
};

const char *armor_types[] =
{
  "UNDEFINED",
  "LEATHER",
  "CHAIN",
  "SCALE",
  "FIELD_PLATE",
  "FULL_PLATE",
  "\n"
};

const char *body_types[] =
{
  "UNDEFINED",
  "CLOTH",
  "LEATHER",
  "CHAINMAIL",
  "SCALE",
  "BREASTPLATE",
  "CORSLET",
  "HEAVY CORSLET",
  "\n"
};

const char *limb_types[] =
{
  "UNDEFINED",
  "CLOTH",
  "LEATHER",
  "CHAINMAIL",
  "SCALE",
  "PLATE",
  "HEAVY_PLATE",
  "\n"
};

const char *extremity_types[] =
{
  "UNDEFINED",
  "CLOTH GLOVES",
  "LEATHER GLOVES",
  "GAUNTLETS",
  "SHOES",
  "BOOTS",
  "SOLLERETS",
  "\n"
};

const char *weapons_types[] =
{
  "UNDIFINED",
  "BATTLE_AXE",
  "BOW",
  "CLUB",
  "CROSSBOW",
  "DAGGER",
  "DART",
  "FLAIL",
  "MACE",
  "PICK",
  "HAMMER",
  "HAND_AXE",
  "HARPOON",
  "JAVELIN",
  "KNIFE",
  "LANCE",
  "MANCATCHER",
  "MORNING_STAR",
  "POLEARM",
  "STAFF",
  "SCROUGE",
  "SICKLE",
  "SLING",
  "SPEAR",
  "STAFF_SLING",
  "BASTARD_SWORD",
  "BROAD_SWORD",
  "KHOPESH",
  "LONG_SWORD",
  "SHORT_SWORD",
  "TWO_HANDED_SWORD",
  "TRIDENT",
  "WHIP",
  "RAPIER",
  "SABER",
  "\n"
};

/* also set NUM in olc.h */
const char *activating_commands[] =
{
  "touch",
  "rub",
  "push",
  "pull",
  "pray",
  "twist",
  "enter",
  "\n"
};

const char *specialist_list[] =
{
  "Abjurer",
  "Conjurer",
  "Diviner",
  "Enchanter",
  "Invoker",
  "Necromancer",
  "Transmuter",
  "\n"
};


const char *sphere_list[] =
{
  "All",
  "Animal",
  "Astral",
  "Charm",
  "Combat",
  "Creation",
  "Divination",
  "Elemental",
  "Guardian",
  "Healing",
  "Necromantic",
  "Plant",
  "Protection",
  "Summoning",
  "Sun",
  "Weather",
  "\n"
};

const char *school_list[] =
{
  "Abjuration",
  "Alteration",
  "Conjuration",
  "Summoning",
  "Enchantment",
  "Charm",
  "Illusion",
  "Phantasm",
  "Invocation",
  "Evocation",
  "Lesser Div.",
  "Greater Div.",
  "Necromancy",
  "\n"
};

/* be sure to update NUM_COIN in olc.h */
const char *kingdom_coins_list[] =
{
  "NO KINGDOM (normal coin)",
  "&+BMarch of Bissel&n",
  "&+LBlackmoor&n",
  "&+CKingdom of Celene&n",
  "Free city of Dyvers", 
  "Kingdom of Furyondy",
  "&+yGrand Duchy of Geoff&n",
  "Gran March",
  "G&+Lrey&NH&+Lawk&n",
  "County of Idee",
  "&+LFree City of Irongate&n",
  "&+gKingdom of Keoland&n",
  "North Province",
  "Kingdom of Nyrond",
  "Free State of Onnwal",
  "Theocracy of the Pale",
  "&+bPomarj&n",
  "&+yShield Lands&n",
  "South Province",
  "Spindrift Isles",
  "Duchy of Ulek",
  "Duchy of Urnst",
  "&+mVerbobonc&n",
  "\n"
};

const char *fighting_styles[] =
{
  "aggressive",
  "normal",
  "defensive",
  "\n"
};

const int aim_mod[] = 
{
  -3, -7, -5, -4, -8, -4, -2, 0, -2, -2, -4, -4
};

const char *location_names[] =
{
  "vitals",
  "brain",
  "head",
  "shield arm",
  "shield hand",
  "weapon hand",
  "weapon arm",
  "torso",
  "right leg",
  "left leg",
  "right foot",
  "left foot"
};

const struct armor_apply_type torso_app[] =
{
  //{pd,dr}
  {0,0}, //undefined
  {1,1}, //cloth
  {2,2}, //leather
  {3,4}, //chainmail
  {3,4}, //scale
  {4,5}, //breastplate
  {4,6}, //corslet
  {4,7} //heavy corslet
};

const struct armor_apply_type limb_app[] =
{
  {0,0}, //undefined
  {1,1}, //cloth
  {2,2}, //leather
  {3,4}, //chainmail
  {3,4}, //scale
  {4,6}, //plate
  {4,7} //heavy plate
};

const struct armor_apply_type extremity_app[] =
{
  {0,0}, //undefined
  {1,1}, //cloth gloves
  {2,2}, //leather gloves
  {3,4}, //gauntlets
  {1,1}, //shoes
  {2,2}, //boots
  {3,4} //sollerets
};

const struct armor_apply_type helmet_app[] =
{
  {0,0}, //undefined
  {1,1}, //cloth cap
  {2,2}, //leather Helm
  {3,4}, //chain coif
  {3,4}, //pot helm
  {4,7}  //Great Helm
};

const struct body_armor_type armor_app[] =
{
  //{torso,arms,legs}
  {{0,0},{0,0},{0,0}}, //undefined
  {{2,2},{2,2},{2,2}}, //leather
  {{3,4},{3,4},{3,4}}, //chain
  {{3,4},{3,4},{3,4}}, //scale
  {{4,6},{4,6},{4,6}}, //field plate
  {{4,7},{4,7},{4,7}}  //full plate
};
