/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#define __ACT_OTHER_C__

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "house.h"
#include "events.h"
#include "skills.h"
#include "dg_scripts.h"
#include "ident.h"

/* extern variables */
extern struct str_app_type str_app[];
extern struct room_data *world;
extern struct descriptor_data *descriptor_list;
extern struct room_data *world;
extern struct zone_data *zone_table;
extern struct dex_skill_type dex_app_skill[];
extern struct spell_info_type spell_info[];
extern struct index_data *mob_index;
extern char *class_abbrevs[];
extern char *dirs[];
extern char *dirs_look[];
extern char *distances_look[];

/* extern procedures */
SPECIAL(shop_keeper);
void mag_objectmagic(struct char_data *ch, struct obj_data *obj, char *argument, int which_value);
extern char *coin_stringv(int amount);
extern void gain_exp(struct char_data *ch, int gain);

/* locals */
EVENTFUNC(camp_event);

struct camp_event {
  struct char_data *ch;
  int was_in;
  int x;
  int y;
  int z;
};

ACMD(do_camp)
{
  struct camp_event *ce;

  if (GET_ACTION(ch)) {
    send_to_char("You are too busy to do this!\r\n", ch);
    return;
  }
  /* create and initialize the camp event */
  CREATE(ce, struct camp_event, 1);
  ce->ch = ch;
  ce->was_in = ch->in_room;
  ce->x = GET_X_CORD(ch);
  ce->y = GET_Y_CORD(ch);
  ce->z = GET_Z_CORD(ch);
  GET_ACTION(ch) = event_create(camp_event, ce, 100);

  act("You prepare a campsite, in anticipation of leaving the realm.", FALSE, ch, NULL, NULL, TO_CHAR);
}

ACMD(do_quit)
{
  void die(struct char_data *ch, struct char_data *killer);
  void Crash_rentsave(struct char_data *ch, int cost);
  extern int free_rent;
  int save_room;
  struct descriptor_data *d, *next_d;

  if (IS_NPC(ch) || !ch->desc)
    return;

  save_room = ch->in_room;

  if (subcmd != SCMD_QUIT)
    send_to_char("You have to type quit--no less, to quit!\r\n", ch);
  else if (IS_FIGHTING(ch))
    send_to_char("No way!  You're fighting for your life!\r\n", ch);
  else if (GET_STAT(ch) < STAT_STUNNED) {
    send_to_char("You die before your time...\r\n", ch);
    die(ch, NULL);
  } else {
    if (!GET_INVIS_LEV(ch))
      act("$n has left the game.", TRUE, ch, 0, 0, PROXIMITY);
    sprintf(buf, "%s has quit the game.", GET_NAME(ch));
    mudlog(buf, NRM, MAX(LVL_IMMORT, GET_INVIS_LEV(ch)), TRUE);

    /*
     * kill off all sockets connected to the same player as the one who is
     * trying to quit.  Helps to maintain sanity as well as prevent duping.
     */
    for (d = descriptor_list; d; d = next_d) {
      next_d = d->next;
      if (d == ch->desc)
	continue;
      if (d->character && (GET_IDNUM(d->character) == GET_IDNUM(ch)))
	close_socket(d);
    }

    if (free_rent || IS_TRUSTED(ch))
      Crash_rentsave(ch, 0);
    extract_char(ch);		/* Char is saved in extract char */

    save_char(ch, save_room);
  }
}



ACMD(do_save)
{
  if (IS_NPC(ch) || !ch->desc)
    return;

  /* Only tell the char we're saving if they actually typed "save" */
  if (cmd) {
    sprintf(buf, "Saving %s.\r\n", GET_NAME(ch));
    send_to_char(buf, ch);
  }
  save_char(ch, ch->in_room);
  Crash_crashsave(ch);
  if (ROOM_FLAGGED(ch->in_room, ROOM_HOUSE_CRASH))
    House_crashsave(world[ch->in_room].number);
}


/* generic function for commands which are normally overridden by
   special procedures - i.e., shop commands, mail commands, etc. */
ACMD(do_not_here)
{
  send_to_char("Sorry, but you cannot do that here!\r\n", ch);
}

int roll_slnt_move(struct char_data *ch)
{
  int prob;

  if (IS_AFFECTED4(ch, AFF4_SLNT_MOVE))
    REMOVE_BIT(AFF4_FLAGS(ch), SKILL_SNEAK);

  // AD&D does not use any type of "skill" level for thief skills.
  // Instead, everything is based on level and circumstances...

  // simulate the huge table for class/chance in original AD&D PH.
  prob = 8;
  prob += (GET_LEVEL(ch) * 7);

  // racial bonuses as per AD&D 1 PH
  switch (GET_RACE(ch)) {
  case RACE_ELF:
  case RACE_GNOME:
    prob += 5;
    break;
  case RACE_HALFLING:
    prob += 10;
    break;
  }
  // Arcana (and AD&D2) added a modifier to chance based on armor
  // worn.  We'll simulate that with char_noise_distance().  Note
  // that if your not wearing anything of type ARMOR, this actually
  // will INCREASE your chance by 5-10%.  We subtract DIST_NOISE_NORM to
  // compensate for when its added inside of char_noise_distance().

  // also note that char_noise_distance takes into account the 'size'
  // of the character.
  prob -= ((char_noise_distance(ch) - 3) * 5);

  // finally, do the dex modifier.  taken from AD&D 1 pre-arcana
  if (GET_DEX(ch) >= 18)
    prob += 10;
  else if (GET_DEX(ch) == 17)
    prob += 5;
  else if (GET_DEX(ch) > 12)
    prob += 0;			// no affect, but speeds code by forcing no more elseif's

  else if (GET_DEX(ch) == 12)
    prob -= 5;
  else if (GET_DEX(ch) == 11)
    prob -= 10;
  else if (GET_DEX(ch) == 10)
    prob -= 15;
  else if (GET_DEX(ch) <= 9)
    prob -= 20;

  if (prob > number(1, 100)) {
    SET_BIT(AFF4_FLAGS(ch), AFF4_SLNT_MOVE);
  }
  return 0;
}

ACMD(do_sneak)
{
  // do_sneak does the interface stuff, and class checking, but the
  // actual work is done in roll_sneak().

  // note: bards don't seem to exist in AD&D1, and in AD&D2, they
  // don't have the move silently skill.
  if (!IS_ROGUE(ch) || (GET_CLASS(ch) == CLASS_BARD)) {
    if (IS_TRUSTED(ch)) {
      send_to_char("Your use your godly powers to act like a rogue...\r\n", ch);
    } else {
      send_to_char("You don't seem to know how to do this.\r\n", ch);
      return;
    }
  }
  if (!FIGHTING(ch))
    send_to_char("You lighten your footsteps.\r\n", ch);
  roll_slnt_move(ch);
}



ACMD(do_hide)
{
  byte percent;

  send_to_char("You attempt to blend into the shadows.\r\n", ch);

  if (IS_AFFECTED(ch, AFF_HIDE))
    REMOVE_BIT(AFF_FLAGS(ch), AFF_HIDE);

  // if anyone is watching, its an instant failure.
  if (ch->watchers)
    return;

  percent = number(1, 101);	/* 101% is a complete failure */

  if (percent > GET_SKILL(ch, SKILL_HIDE) + dex_app_skill[GET_DEX(ch)].hide)
    return;

  SET_BIT(AFF_FLAGS(ch), AFF_HIDE);
}



#define STEALABLE(pos) (pos == WEAR_EARRING_R || pos == WEAR_EARRING_L || \
	pos == WEAR_WRIST_R || pos == WEAR_WRIST_L || pos == WEAR_FINGER_R || \
	pos == WEAR_FINGER_L || pos == WEAR_HANDS || pos == WEAR_WAIST || \
	pos == WEAR_ATTACH_1 || pos == WEAR_ATTACH_2 || pos == WEAR_ATTACH_3)

ACMD(do_steal)
{
  struct char_data *vict;
  struct obj_data *obj;
  char vict_name[MAX_INPUT_LENGTH], obj_name[MAX_INPUT_LENGTH];
  int percent, gold, eq_pos, ohoh = 0, bonus = 0;


  ACMD(do_gen_comm);

  if (!THIEVING_CLASS(ch)) {
    send_to_char("You really wouldn't be very good at that.\r\n", ch);
    return;
  }
  argument = one_argument(argument, obj_name);
  one_argument(argument, vict_name);

  if (!(vict = get_char_room_vis(ch, vict_name))) {
    send_to_char("Steal what from who?\r\n", ch);
    return;
  } else if (vict == ch) {
    send_to_char("Come on now, that's rather stupid!\r\n", ch);
    return;
  }
  /* 101% is a complete failure */
  percent = number(1, 101) - dex_app_skill[GET_DEX(ch)].p_pocket;

#if 0
  /* Just too many ways to abuse this. */
  bonus = GET_SKILL(ch, SKILL_STEAL) - percent;
  if (bonus <= 0)
    bonus = 0;
#endif

  if (GET_STAT(vict) < STAT_SLEEPING)
    percent = -1;		/* ALWAYS SUCCESS */

  /* NO NO With Imp's */
  if (GET_LEVEL(vict) >= LVL_IMMORT)
    percent = 101;		/* Failure */

  if (str_cmp(obj_name, "coins") && str_cmp(obj_name, "gold")) {
    for (eq_pos = 0; eq_pos < NUM_WEARS; eq_pos++)
      if (GET_EQ(vict, eq_pos) &&
	  (isname(obj_name, GET_EQ(vict, eq_pos)->name)) &&
	  CAN_SEE_OBJ(ch, GET_EQ(vict, eq_pos))) {
	obj = GET_EQ(vict, eq_pos);
	break;
      }
    if (!obj) {
      act("$E hasn't got that item.", FALSE, ch, 0, vict, TO_CHAR);
      return;
    } else {			/* It is equipment */
      if (!STEALABLE(eq_pos)) {
	send_to_char("A theft like that is for the realm of magics. Try an easier target.\r\n", ch);
	return;
      } else if (AWAKE(vict) && (percent > GET_SKILL(ch, SKILL_STEAL))) {
	ohoh = TRUE;
	act("Oops..", FALSE, ch, 0, 0, TO_CHAR);
	act("$n tried to steal something from you!", FALSE, ch, 0, vict, TO_VICT);
	act("$n tries to steal something from $N.", TRUE, ch, 0, vict, TO_NOTVICT);
      } else {			/* Steal the item */
	if ((IS_CARRYING_N(ch) + 1 < CAN_CARRY_N(ch))) {
	  if ((IS_CARRYING_W(ch) + GET_OBJ_WEIGHT(obj)) < CAN_CARRY_W(ch)) {
	    act("You unequip $p and steal it.", FALSE, ch, obj, 0, TO_CHAR);
	    act("$n steals $p from $N.", FALSE, ch, obj, vict, TO_NOTVICT);
	    obj_to_char(unequip_char(vict, eq_pos), ch);
	    if (bonus) {
	      send_to_char("You gain experience from your cleverness\r\n", ch);
	      gain_exp(ch, bonus);
	    }
	    Crash_crashsave(ch);
	    Crash_crashsave(vict);
	  }
	} else
	  send_to_char("You cannot carry that much.\r\n", ch);
      }
    }
  } else {			/* Steal some coins */
    if (AWAKE(vict) && (percent > GET_SKILL(ch, SKILL_STEAL))) {
      ohoh = TRUE;
      act("Oops..", FALSE, ch, 0, 0, TO_CHAR);
      act("You discover that $n has $s hands in your wallet.", FALSE, ch, 0, vict, TO_VICT);
      act("$n tries to steal gold from $N.", TRUE, ch, 0, vict, TO_NOTVICT);
    } else {
      /* Steal some gold coins */
      gold = (int) ((GET_MONEY(vict) * number(1, 10)) / 100);
      gold = MIN(1782, gold);
      if (gold > 0) {
	ADD_MONEY(ch, gold);
	SUB_MONEY(vict, gold);
	sprintf(buf, "Bingo!  You got %s.\r\n", coin_stringv(gold));
	send_to_char(buf, ch);
	if (bonus) {
	  send_to_char("You gain experience from your cleverness\r\n", ch);
	  gain_exp(ch, bonus);
	}
	Crash_crashsave(ch);
	Crash_crashsave(vict);
      } else {
	send_to_char("You couldn't get any gold...\r\n", ch);
      }
    }
  }

  if (ohoh && IS_NPC(vict) && AWAKE(vict)) {
    hit(vict, ch, TYPE_UNDEFINED);
    if (IS_NPC(vict) && IS_CITIZEN(vict)) {
      sprintf(buf, "mcallforguards %s steal", fname(GET_NAME(ch)));
      command_interpreter(vict, buf);
    }
  }
}
#undef STEALABLE(pos)

ACMD(do_visible)
{
  void appear(struct char_data *ch);
  void perform_immort_vis(struct char_data *ch);

  if (GET_LEVEL(ch) >= LVL_IMMORT) {
    perform_immort_vis(ch);
    return;
  }
  if IS_AFFECTED
    (ch, AFF_INVISIBLE) {
    appear(ch);
    send_to_char("You break the spell of invisibility.\r\n", ch);
  } else
    send_to_char("You are already visible.\r\n", ch);
}



ACMD(do_title)
{
  char name[80];
  struct char_data *vict;

  delete_doubledollar(argument);
  half_chop(argument, name, buf);

  if (!(vict = get_player_vis(ch, name, 0))) {
    send_to_char("There is no such player.\r\n", ch);
    return;
  }
  if (IS_NPC(ch))
    send_to_char("Your title is fine... go away.\r\n", ch);
  else if (strstr(buf, "(") || strstr(buf, ")"))
    send_to_char("Titles can't contain the ( or ) characters.\r\n", ch);
  else if (strlen(buf) > MAX_TITLE_LENGTH) {
    sprintf(buf2, "Sorry, titles can't be longer than %d characters.\r\n",
	    MAX_TITLE_LENGTH);
    send_to_char(buf2, ch);
  } else {
    set_title(vict, buf);
    sprintf(buf2, "Okay, %s is now %s.\r\n", GET_NAME(vict), GET_TITLE(vict));
    send_to_char(buf2, ch);
  }
}

int perform_group(struct char_data *ch, struct char_data *vict)
{
  struct group_type *gr;

  /* restricted pcs only to viewing. Mobs don't need to be in same room */
  if (IS_AFFECTED(vict, AFF_GROUP) || (!CAN_SEE(ch, vict) && (IS_PC(ch) || IS_PC(vict))))
    return 0;

  SET_BIT(AFF_FLAGS(vict), AFF_GROUP);
  if (vict != ch)
    vict->leader = ch;

  if (ch == vict)
    return 0;

  CREATE(gr, struct group_type, 1);

  gr->follower = vict;
  gr->next = NULL;


  if (ch->group)
    gr->next = ch->group;

  ch->group = gr;

  if (ch != vict)
    act("$N is now a member of your group.", FALSE, ch, 0, vict, TO_CHAR);
  act("You are now a member of $n's group.", FALSE, ch, 0, vict, TO_VICT);
  act("$N is now a member of $n's group.", FALSE, ch, 0, vict, TO_NOTVICT);
  return 1;
}

void remove_from_group(struct char_data *ch, struct char_data *fol)
{
  struct group_type *k = NULL, *j = NULL;
  REMOVE_BIT(AFF_FLAGS(fol), AFF_GROUP);

  if (ch->group && ch != fol) {
    if (ch->group->follower == fol) {	/* Head of follower-list? */
      k = ch->group;
      ch->group = k->next;
      FREE(k);
    } else {			/* locate follower who is not head of list */
      if (ch->group->next)
	for (k = ch->group; k->next->follower != fol; k = k->next);

      if (k->follower == fol) {
	j = k->next;
	k->next = j->next;
	FREE(j);
      }				//if

    }
  }
}

void print_group(struct char_data *ch)
{
  struct char_data *k;
  struct group_type *f;
  char _groupmem[256];
  int _buflen = 0;

  if (!IS_AFFECTED(ch, AFF_GROUP))
    send_to_char("But you are not a member of any group!\r\n", ch);
  else {
    send_to_char("Your group consists of:\r\n", ch);

    k = (ch->leader ? ch->leader : ch);

    if (IS_AFFECTED(k, AFF_GROUP)) {
      sprintf(_groupmem, "     %s (Head of group)", GET_NAME(k));
      _buflen = strlen(_groupmem);
      for (_buflen; _buflen < 40; _buflen++)
	_groupmem[_buflen] = ' ';

      _groupmem[_buflen] = '\0';
      sprintf(buf, "%s %d/%d hit points", _groupmem, GET_HIT(k), GET_MAX_HIT(k));
      act(buf, FALSE, ch, 0, k, TO_CHAR);
    }
    for (f = k->group; f; f = f->next) {
      if (!IS_AFFECTED(f->follower, AFF_GROUP))
	continue;
      sprintf(_groupmem, "     %s", GET_NAME(f->follower));
      _buflen = strlen(_groupmem);

      for (_buflen; _buflen < 40; _buflen++)
	_groupmem[_buflen] = ' ';

      _groupmem[_buflen] = '\0';
      sprintf(buf, "%s %d/%d hit points", _groupmem, GET_HIT(f->follower), GET_MAX_HIT(f->follower));
      act(buf, FALSE, ch, 0, f->follower, TO_CHAR);
    }
  }
}

int members_in_group(struct char_data *ch)
{
  struct char_data *k;
  struct group_type *f;
  int found = 1;		/* count the leader! */

  k = (ch->leader ? ch->leader : ch);

  for (f = k->group; f; f = f->next) {
    if (!IS_AFFECTED(f->follower, AFF_GROUP))
      continue;
    else
      found++;
  }
  return (found);
}

ACMD(do_group)
{
  struct char_data *vict;
  struct follow_type *f;
  int found;
  struct group_type *g, *g_next;

  one_argument(argument, buf);

  if (!*buf) {
    print_group(ch);
    return;
  }
  if (!str_cmp(buf, "me") || !str_cmp(buf, GET_NAME(ch)))
    if (IS_AFFECTED(ch, AFF_GROUP)) {

      if (ch->leader) {
	act("$n is no longer a member of your group.", FALSE, ch, 0, ch->leader, TO_VICT);
	act("You leave $N's group!", FALSE, ch, 0, ch->leader, TO_CHAR);
	act("$N leaves $n's group!", FALSE, ch, 0, ch->leader, TO_OTHERS);
	remove_from_group(ch->leader, ch);
      } else {
	act("You leave the group!", FALSE, ch, 0, ch->leader, TO_CHAR);
	remove_from_group(ch, ch);
	if (!ch->group)
	  return;
	vict = ch->group->follower;
	vict->leader = NULL;
	for (g = ch->group->next; g; g = g_next) {
	  g_next = g->next;
	  g->next = vict->group;
	  vict->group = g;
	  g->follower->leader = vict;
//        act("$N leaves the group!", FALSE, g->follower, 0, ch, TO_CHAR);
	}
	ch->group = NULL;
      }
      return;
    }
  if (ch->master) {
    act("You may not enroll group members without being the head of a group.", FALSE, ch, 0, 0, TO_CHAR);
    return;
  }
  if (!str_cmp(buf, "all")) {
    perform_group(ch, ch);
    for (found = 0, f = ch->followers; f; f = f->next) {
      if (members_in_group(ch) > 10) {
	send_to_char("10 members per group max!\r\n", ch);
	return;
      }
      found += perform_group(ch, f->follower);
    }
    if (!found)
      send_to_char("Everyone following you is already in your group.\r\n", ch);
    return;
  }
  if (!(vict = get_char_room_vis(ch, buf)))
    send_to_char(NOPERSON, ch);
  else if ((vict->master != ch) && (vict != ch))
    act("$N must follow you to enter your group.", FALSE, ch, 0, vict, TO_CHAR);
  else {
    if (members_in_group(ch) > 10) {
      send_to_char("10 members per group max!\r\n", ch);
      return;
    }
    if (!IS_AFFECTED(vict, AFF_GROUP)) {
      perform_group(ch, vict);
      if (!IS_AFFECTED(ch, AFF_GROUP))
	perform_group(ch, ch);
    } else if (vict->leader != ch) {
      act("$N must be in your group to be ungrouped.", FALSE, ch, 0, vict, TO_CHAR);
    } else {
      if (ch != vict)
	act("$N is no longer a member of your group.", FALSE, ch, 0, vict, TO_CHAR);
      act("You have been kicked out of $n's group!", FALSE, ch, 0, vict, TO_VICT);
      act("$N has been kicked out of $n's group!", FALSE, ch, 0, vict, TO_NOTVICT);
      REMOVE_BIT(AFF_FLAGS(vict), AFF_GROUP);
      remove_from_group(ch, vict);
    }
  }
}



ACMD(do_ungroup)
{
  struct group_type *f, *next_fol;
  struct char_data *tch;
  void stop_follower(struct char_data *ch);

  one_argument(argument, buf);

  if (!*buf) {
    if (ch->master || !(IS_AFFECTED(ch, AFF_GROUP))) {
      send_to_char("But you lead no group!\r\n", ch);
      return;
    }
    sprintf(buf2, "%s has disbanded the group.\r\n", GET_NAME(ch));
    for (f = ch->group; f; f = next_fol) {
      next_fol = f->next;
      if (IS_AFFECTED(f->follower, AFF_GROUP)) {
	REMOVE_BIT(AFF_FLAGS(f->follower), AFF_GROUP);
	remove_from_group(ch, f->follower);
	send_to_char(buf2, f->follower);
	if (!IS_AFFECTED(f->follower, AFF_CHARM))
	  stop_follower(f->follower);
      }
    }

    REMOVE_BIT(AFF_FLAGS(ch), AFF_GROUP);
    remove_from_group(ch, ch);
    send_to_char("You disband the group.\r\n", ch);
    return;
  }
  if (!(tch = get_char_room_vis(ch, buf))) {
    send_to_char("There is no such person!\r\n", ch);
    return;
  }
  if (tch->master != ch) {
    send_to_char("That person is not following you!\r\n", ch);
    return;
  }
  if (!IS_AFFECTED(tch, AFF_GROUP)) {
    send_to_char("That person isn't in your group.\r\n", ch);
    return;
  }
  REMOVE_BIT(AFF_FLAGS(tch), AFF_GROUP);

  act("$N is no longer a member of your group.", FALSE, ch, 0, tch, TO_CHAR);
  act("You have been kicked out of $n's group!", FALSE, ch, 0, tch, TO_VICT);
  act("$N has been kicked out of $n's group!", FALSE, ch, 0, tch, TO_NOTVICT);

  if (!IS_AFFECTED(tch, AFF_CHARM))
    stop_follower(tch);
}




ACMD(do_report)
{
  struct char_data *k = NULL;
  struct obj_data *objdummy = NULL;
  struct group_type *f;
  char _target[256];

  if (!argument || !strcmp("", argument)) {
    sprintf(buf, "%s says, 'I have %d (%d) hit points.'\r\n", GET_NAME(ch), GET_HIT(ch), GET_MAX_HIT(ch));
    act(buf, FALSE, ch, 0, 0, PROXIMITY);
    sprintf(buf, "You say, 'I have %d (%d) hit points.'\r\n", GET_HIT(ch), GET_MAX_HIT(ch));
    send_to_char(buf, ch);
    return;
  }				//if

  one_argument(argument, _target);

  if (isname("group", _target) || isname("Group", _target)) {
    sprintf(buf, "&+G%s group-says, 'I have %d (%d) hit points.'&n\r\n", GET_NAME(ch), GET_HIT(ch), GET_MAX_HIT(ch));
    if (!IS_AFFECTED(ch, AFF_GROUP)) {
      send_to_char("But you are not a member of any group!\r\n", ch);
      return;
    }				//if

    CAP(buf);
    k = (ch->master ? ch->master : ch);

    for (f = k->group; f; f = f->next)
      if (IS_AFFECTED(f->follower, AFF_GROUP) && f->follower != ch)
	send_to_char(buf, f->follower);

    if (k != ch)
      send_to_char(buf, k);
    sprintf(buf, "&+GYou group-say, 'I have %d (%d) hit points.'&n\r\n", GET_HIT(ch), GET_MAX_HIT(ch));
    send_to_char(buf, ch);
    return;
  }				//if

  generic_find(_target, FIND_CHAR_WORLD, ch, &k, &objdummy);

  if (k && ch != k) {
    sprintf(buf, "&+W%s tells you, 'I have %d (%d) hit points.'&n\r\n", GET_NAME(ch), GET_HIT(ch), GET_MAX_HIT(ch));
    send_to_char(buf, k);
    sprintf(buf, "&+WYou tell %s, 'I have %d (%d) hit points.'&n\r\n", GET_NAME(k), GET_HIT(ch), GET_MAX_HIT(ch));
    send_to_char(buf, ch);
  }				//if

  else
    send_to_char("report to who?", ch);
}				//do_rep



ACMD(do_split)
{
  int amount, num, share;
  struct char_data *k;
  struct follow_type *f;

  if (IS_NPC(ch))
    return;

  one_argument(argument, buf);

  if (is_number(buf)) {
    amount = atoi(buf);
    if (amount <= 0) {
      send_to_char("Sorry, you can't do that.\r\n", ch);
      return;
    }
    if (amount > GET_MONEY(ch)) {
      send_to_char("You don't seem to have that much gold to split.\r\n", ch);
      return;
    }
    k = (ch->master ? ch->master : ch);

    if (IS_AFFECTED(k, AFF_GROUP) && (k->in_room == ch->in_room))
      num = 1;
    else
      num = 0;

    for (f = k->followers; f; f = f->next)
      if (IS_AFFECTED(f->follower, AFF_GROUP) &&
	  (!IS_NPC(f->follower)) &&
	  (f->follower->in_room == ch->in_room))
	num++;

    if (num && IS_AFFECTED(ch, AFF_GROUP))
      share = amount / num;
    else {
      send_to_char("With whom do you wish to share your gold?\r\n", ch);
      return;
    }

    SUB_MONEY(ch, share * (num + 1));

    if (IS_AFFECTED(k, AFF_GROUP) && (k->in_room == ch->in_room)
	&& !(IS_NPC(k)) && k != ch) {
      ADD_MONEY(k, share);
      sprintf(buf, "%s splits %d coins; you receive %d.\r\n", GET_NAME(ch),
	      amount, share);
      send_to_char(buf, k);
    }
    for (f = k->followers; f; f = f->next) {
      if (IS_AFFECTED(f->follower, AFF_GROUP) &&
	  (!IS_NPC(f->follower)) &&
	  (f->follower->in_room == ch->in_room) &&
	  f->follower != ch) {
	ADD_MONEY(f->follower, share);
	sprintf(buf, "%s splits %d coins; you receive %d.\r\n", GET_NAME(ch),
		amount, share);
	send_to_char(buf, f->follower);
      }
    }
    sprintf(buf, "You split %d coins among %d members -- %d coins each.\r\n",
	    amount, num, share);
    send_to_char(buf, ch);
  } else {
    send_to_char("How many coins do you wish to split with your group?\r\n", ch);
    return;
  }
}



ACMD(do_use)
{
  struct obj_data *mag_item;
  int equipped = 1, which_value = -1, tmp, i;
  char spellname[80];

  half_chop(argument, arg, buf);
  if (!*arg) {
    sprintf(buf2, "What do you want to %s?\r\n", CMD_NAME);
    send_to_char(buf2, ch);
    return;
  }
  mag_item = GET_EQ(ch, WEAR_HOLD);

  if (!mag_item || !isname(arg, mag_item->name)) {
    switch (subcmd) {
    case SCMD_RECITE:
    case SCMD_QUAFF:
      equipped = 0;
      if (!(mag_item = get_obj_in_list_vis(ch, arg, ch->carrying))) {
	sprintf(buf2, "You don't seem to have %s %s.\r\n", AN(arg), arg);
	send_to_char(buf2, ch);
	return;
      }
      break;
    case SCMD_USE:
      sprintf(buf2, "You don't seem to be holding %s %s.\r\n", AN(arg), arg);
      send_to_char(buf2, ch);
      return;
      break;
    default:
      log("SYSERR: Unknown subcmd passed to do_use");
      return;
      break;
    }
  }
  switch (subcmd) {
  case SCMD_QUAFF:
    if (GET_OBJ_TYPE(mag_item) != ITEM_POTION) {
      send_to_char("You can only quaff potions.", ch);
      return;
    }
    break;
  case SCMD_RECITE:
    if (GET_OBJ_TYPE(mag_item) != ITEM_SCROLL) {
      send_to_char("You can only recite scrolls.", ch);
      return;
    }
    break;
  case SCMD_USE:
    if ((GET_OBJ_TYPE(mag_item) != ITEM_WAND) &&
	(GET_OBJ_TYPE(mag_item) != ITEM_STAFF)) {
      send_to_char("You can't seem to figure out how to use it.\r\n", ch);
      return;
    }
    break;
  }

  if (subcmd == SCMD_RECITE) {
    half_chop(buf, buf, spellname);
    if (*spellname && spellname) {
      tmp = find_spells_num(spellname);
      for (i = 1; i < 8; i++)
	if (GET_OBJ_VAL(mag_item, i) == tmp) {
	  which_value = i;
	  break;
	}
    }
  }
  mag_objectmagic(ch, mag_item, buf, which_value);
}



ACMD(do_wimpy)
{
  int wimp_lev;

  one_argument(argument, arg);

  if (!*arg) {
    if (GET_WIMP_LEV(ch)) {
      sprintf(buf, "Your current wimp level is %d hit points.\r\n",
	      GET_WIMP_LEV(ch));
      send_to_char(buf, ch);
      return;
    } else {
      send_to_char("At the moment, you're not a wimp.  (sure, sure...)\r\n", ch);
      return;
    }
  }
  if (isdigit(*arg)) {
    if ((wimp_lev = atoi(arg))) {
      if (wimp_lev < 0)
	send_to_char("Heh, heh, heh.. we are jolly funny today, eh?\r\n", ch);
      else if (wimp_lev > GET_MAX_HIT(ch))
	send_to_char("That doesn't make much sense, now does it?\r\n", ch);
      else if (wimp_lev > (GET_MAX_HIT(ch) >> 1))
	send_to_char("You can't set your wimp level above half your hit points.\r\n", ch);
      else {
	sprintf(buf, "Okay, you'll wimp out if you drop below %d hit points.\r\n",
		wimp_lev);
	send_to_char(buf, ch);
	GET_WIMP_LEV(ch) = wimp_lev;
      }
    } else {
      send_to_char("Okay, you'll now tough out fights to the bitter end.\r\n", ch);
      GET_WIMP_LEV(ch) = 0;
    }
  } else
    send_to_char("Specify at how many hit points you want to wimp out at.  (0 to disable)\r\n", ch);

  return;

}

ACMD(do_gen_write)
{
  FILE *fl;
  char *tmp, *filename, buf[MAX_STRING_LENGTH];
  struct stat fbuf;
  extern int max_filesize;
  time_t ct;

  switch (subcmd) {
  case SCMD_BUG:
    filename = BUG_FILE;
    break;
  case SCMD_TYPO:
    filename = TYPO_FILE;
    break;
  case SCMD_IDEA:
    filename = IDEA_FILE;
    break;
  default:
    return;
  }

  ct = time(0);
  tmp = asctime(localtime(&ct));

  if (IS_NPC(ch)) {
    send_to_char("Monsters can't have ideas - Go away.\r\n", ch);
    return;
  }
  skip_spaces(&argument);
  delete_doubledollar(argument);

  if (!*argument) {
    send_to_char("That must be a mistake...\r\n", ch);
    return;
  }
  sprintf(buf, "%s %s: %s", GET_NAME(ch), CMD_NAME, argument);
  mudlog(buf, CMP, LVL_IMMORT, FALSE);

  if (stat(filename, &fbuf) < 0) {
    perror("Error statting file");
    return;
  }
  if (fbuf.st_size >= max_filesize) {
    send_to_char("Sorry, the file is full right now.. try again later.\r\n", ch);
    return;
  }
  if (!(fl = fopen(filename, "a"))) {
    perror("do_gen_write");
    send_to_char("Could not open the file.  Sorry.\r\n", ch);
    return;
  }
  fprintf(fl, "%-8s (%6.6s) [%5d] %s\n", GET_NAME(ch), (tmp + 4),
	  world[ch->in_room].number, argument);
  fclose(fl);
  send_to_char("Okay.  Thanks!\r\n", ch);
}



#define TOG_OFF 0
#define TOG_ON  1

#define PRF_TOG_CHK(ch,flag) ((TOGGLE_BIT(PRF_FLAGS(ch), (flag))) & (flag))

ACMD(do_gen_tog)
{
  long result;
  extern int nameserver_is_slow;

  char *tog_messages[][2] =
  {
    {"Nohassle disabled.\r\n",
     "Nohassle enabled.\r\n"},
    {"Compact mode off.\r\n",
     "Compact mode on.\r\n"},
    {"You can now hear tells.\r\n",
     "You are now deaf to tells.\r\n"},
    {"You can now hear shouts.\r\n",
     "You are now deaf to shouts.\r\n"},
    {"You can now hear the Wiz-channel.\r\n",
     "You are now deaf to the Wiz-channel.\r\n"},
    {"You will no longer see vnums and flags.\r\n",
     "You will now see vnums and flags.\r\n"},
    {"You will now have your communication repeated.\r\n",
     "You will no longer have your communication repeated.\r\n"},
    {"Trusted mode off.\r\n",
     "Trusted mode on.\r\n"},
    {"Nameserver_is_slow changed to NO; IP addresses will now be resolved.\r\n",
     "Nameserver_is_slow changed to YES; sitenames will no longer be resolved.\r\n"},
    {"Page breaking is off.\r\n",
     "Page breaking is on.\r\n"},
#ifdef MUDFTP
    {"mudFTP transfers disabled.\r\n",
     "mudFTP transfers enabled.\r\n"},
#endif
    {"Ident changed to YES;  remote usernames lookups will be attempted.\r\n",
     "Ident changed to NO;  remote username lookups will not be attempted.\r\n"},
    {"You're ready to again battle for blood.\r\n",
     "You decide to pull your punches for a bit, and cause no real damage.\r\n"}
  };


  if (IS_NPC(ch))
    return;

  switch (subcmd) {
  case SCMD_NOHASSLE:
    result = PRF_TOG_CHK(ch, PRF_NOHASSLE);
    if (FIGHTING(ch)) {
      stop_fighting(FIGHTING(ch));
      stop_fighting(ch);
    }
    break;
  case SCMD_COMPACT:
    result = PRF_TOG_CHK(ch, PRF_COMPACT);
    break;
  case SCMD_NOTELL:
    result = PRF_TOG_CHK(ch, PRF_NOTELL);
    break;
  case SCMD_DEAF:
    result = PRF_TOG_CHK(ch, PRF_DEAF);
    break;
  case SCMD_NOWIZ:
    result = PRF_TOG_CHK(ch, PRF_NOWIZ);
    break;
  case SCMD_ROOMFLAGS:
    result = PRF_TOG_CHK(ch, PRF_ROOMFLAGS);
    break;
  case SCMD_NOREPEAT:
    result = PRF_TOG_CHK(ch, PRF_NOREPEAT);
    break;
  case SCMD_HOLYLIGHT:
    result = PRF_TOG_CHK(ch, PRF_HOLYLIGHT);
    break;
  case SCMD_SLOWNS:
    result = (nameserver_is_slow = !nameserver_is_slow);
    break;
  case SCMD_PAGING:
    result = PRF_TOG_CHK(ch, PRF_PAGING);
    break;
#ifdef MUDFTP
  case SCMD_MUDFTP:
    result = PRF_TOG_CHK(ch, PRF_MUDFTP);
    break;
#endif
  case SCMD_IDENT:
    result = (ident = !ident);
    break;
  case SCMD_PUMMEL:
    result = PRF_TOG_CHK(ch, PRF_PUMMEL);
    break;
  default:
    log("SYSERR: Unknown subcmd in do_gen_toggle");
    return;
    break;
  }

  if (result)
    send_to_char(tog_messages[subcmd][TOG_ON], ch);
  else
    send_to_char(tog_messages[subcmd][TOG_OFF], ch);

  return;
}

EVENTFUNC(camp_event)
{
  struct camp_event *ce = (struct camp_event *) event_obj;
  struct char_data *ch = NULL;
  int was_in, now_in, x, y, z;
  void Crash_rentsave(struct char_data *ch, int cost);

  /* extract all the info from ce */
  ch = ce->ch;
  was_in = ce->was_in;
  now_in = ch->in_room;
  z = ce->z;
  x = ce->x;
  y = ce->y;

  /* make sure we're supposed to be here */
  if (!GET_ACTION(ch))
    return 0;

  GET_ACTION(ch) = NULL;

  if (IS_NPC(ch) || !ch->desc)
    return 0;

  if (IS_FIGHTING(ch)) {
    act("You decide now is not the best time for camping.", FALSE, ch, NULL, NULL, TO_CHAR);
    return 0;
  }
  if (now_in != was_in || GET_X_CORD(ch) != x || GET_Y_CORD(ch) != y ||
      GET_Z_CORD(ch) != z) {
    act("You are no longer near where you began the campsite.", FALSE, ch, NULL, NULL, TO_CHAR);
    return 0;
  }
  act("You complete your campsite, and leave this world for awhile.", FALSE, ch, NULL, NULL, TO_CHAR);
  if (!GET_INVIS_LEV(ch))
    act("$n camps out, leaving this realm.", TRUE, ch, 0, 0, TO_PROXIMITY);
  sprintf(buf, "%s camped out", GET_NAME(ch));
  mudlog(buf, CMP, LVL_GOD1, TRUE);
  Crash_rentsave(ch, 0);
  extract_char(ch);
  save_char(ch, now_in);

  FREE(event_obj);

  return 0;
}


struct search_event_obj {
  struct char_data *ch;		/* The character searching           */
  long target;			/* The UID of what is being searched */
};

EVENTFUNC(learn_event)
{
  int i, target;
  byte found = FALSE;
  struct char_data *ch;
  struct search_event_obj *search_obj = (struct search_event_obj *) event_obj;

  ch = search_obj->ch;
  target = search_obj->target;

  if (!GET_ACTION(ch))
    return 0;

  GET_ACTION(ch) = NULL;

  if (target != ch->in_room) {
    send_to_char("You have to stay in the room while learning it.\r\n", ch);
    return 0;
  }
  for (i = 0; i < GET_INT(ch); i++) {
    if (ch->player_specials->saved.known_rooms[i] == target) {
      send_to_char("Hey, this room looks familiar. You already know it.\r\n", ch);
      return 0;
    }
    if (!ch->player_specials->saved.known_rooms[i]) {
      found = TRUE;
      ch->player_specials->saved.known_rooms[i] = target;
      break;
    }
  }

  if (!found) {
    for (i = 0; i < (GET_INT(ch) - 1); i++) {
      ch->player_specials->saved.known_rooms[i] =
	  ch->player_specials->saved.known_rooms[i + 1];
    }
    ch->player_specials->saved.known_rooms[GET_INT(ch) - 1] = target;
  }
  act("You have learned this room.",
      FALSE, ch, NULL, NULL, TO_CHAR);
  return 0;
}

EVENTFUNC(search_event)
{
  struct search_event_obj *search_obj = (struct search_event_obj *) event_obj;
  struct char_data *ch, *vict = NULL, *next_ch;
  struct obj_data *obj, *next_obj;
  int chance, i, found = FALSE;
  long target;
  int Distance = 5, Direction = -1;

  ch = search_obj->ch;
  target = search_obj->target;

/*  FREE(event_obj); */

  if (!GET_ACTION(ch))
    return 0;

  GET_ACTION(ch) = NULL;

  sprintf(buf, "%c%ld", UID_CHAR, target);

  if ((obj = get_obj_in_list_vis(ch, buf, ch->carrying)));
  else
    obj = get_obj_in_list_vis(ch, buf, world[ch->in_room].contents);

  if (obj) {
    /* nothing to find on objects, yet */
    act("You didn't find anything new about $o.",
	FALSE, ch, obj, NULL, TO_CHAR);
  } else {
    if (ch->in_room == NOWHERE) {
      log("SYSERR: Character in NOWHERE when searching.");
      return 0;
    }
    if (ch->in_room == target) {
      chance = 100;		/* temp */

      /* secret doors */
      for (i = 0; i < NUM_OF_DIRS; i++)
	if (EXIT(ch, i) && IS_SET(EXIT(ch, i)->exit_info, EX_SECRET) &&
	    (number(0, 100) < chance)) {
	  if (charWithinRangeofDoor(ch, i, Distance)) {
	    Distance = distBetweenCharAndDoor(ch, i);
	    Direction = angleBetweenCharAndDoor(ch, i);
	    sprintf(buf, "You notice a hidden %s",
	    EXIT(ch, i)->keyword ? fname(EXIT(ch, i)->keyword) : "exit");
	    sprintf(buf, "%s %s %s", buf, distances_look[(Distance <= 7 ? Distance : 7)], dirs_look[Direction]);
	    send_to_char(buf, ch);
	    found = TRUE;
	  }
	}
      /* hidden objects */
      for (obj = world[ch->in_room].contents; obj; obj = next_obj) {
	next_obj = obj->next_content;
	if (CAN_SEE_OBJ(ch, obj) && IS_OBJ_STAT(obj, ITEM_SECRET) &&
	    (number(0, 100) < chance) &&
	    objWithinRangeofChar(ch, obj, Distance)) {
	  act("You discover $p hidden here.", FALSE, ch, obj, NULL, TO_CHAR);
	  found = TRUE;
	}
      }

      /* hidden mobs */
      for (vict = world[ch->in_room].people; vict; vict = next_ch) {
	next_ch = vict->next_in_room;
	if (IS_AFFECTED(vict, AFF_HIDE)) {
	  /* remove the hide so it doesn't mess up with CAN_SEE */
	  REMOVE_BIT(AFF_FLAGS(vict), AFF_HIDE);
	  if (CAN_SEE(ch, vict) && (number(0, 100) < chance))
	    /* make a skill check for victim */
	    if (number(0, 100) >= 50) {		/* temp skillchance */
	      act("You discover $N hiding here.",
		  FALSE, ch, NULL, vict, TO_CHAR);
	      act("$n has discovered you hiding!",
		  TRUE, ch, NULL, vict, TO_VICT);
	      found = TRUE;
	      continue;
	    }
	  SET_BIT(AFF_FLAGS(vict), AFF_HIDE);
	}
      }

      if (!found)
	act("You didn't find anything unusual from your searching.",
	    FALSE, ch, NULL, NULL, TO_CHAR);
    } else
      act("You can't find what you were searching!",
	  FALSE, ch, NULL, NULL, TO_CHAR);
  }
  return 1;
}


ACMD(do_search)
{
  int target = -1;
  struct search_event_obj *search_obj;
  char *name = NULL;
  struct obj_data *obj = NULL;

  if (GET_ACTION(ch)) {
    act("You are already busy!", FALSE, ch, NULL, NULL, TO_CHAR);
    return;
  }
  if (!argument || !*argument) {
    target = ch->in_room;
    name = "the room";
  } else {
    one_argument(argument, arg);

    if ((obj = get_obj_in_list_vis(ch, arg, ch->carrying)));
    else
      obj = get_obj_in_list_vis(ch, arg, world[ch->in_room].contents);
  }

  if (obj) {
    target = GET_ID(obj);
    name = "$p";
  } else {
    if (!str_cmp(arg, "room")) {
      target = ch->in_room;
      name = "the room";
    }
  }

  if (!name)
    act("You can't find what you are looking for.",
	FALSE, ch, NULL, NULL, TO_CHAR);

  else {
    CREATE(search_obj, struct search_event_obj, 1);
    search_obj->ch = ch;
    search_obj->target = target;
    /* timer is 4-7.5 seconds */
    GET_ACTION(ch) = event_create(search_event, search_obj,
				  (75 - GET_INT(ch) - GET_WIS(ch)));


    WAIT_STATE(ch, 20);
    sprintf(buf, "You start to search %s.", name);
    act(buf, FALSE, ch, obj, NULL, TO_CHAR);
    sprintf(buf, "$n starts to search %s.", name);
    act(buf, TRUE, ch, obj, NULL, PROXIMITY);
  }
/*
   sprintf(buf, "&+CDEBUG:&n You're looking for %d\r\n", target);
   send_to_char(buf, ch);
 */
}

ACMD(do_learn)
{
  int target = -1, i = 0;
  char *name = NULL;
  struct search_event_obj *search_obj;

  if (GET_ACTION(ch)) {
    act("You are already busy!", FALSE, ch, NULL, NULL, TO_CHAR);
    return;
  }
  if (!argument || !*argument) {
    for (i = 0; i < GET_INT(ch); i++) {
      if (ch->player_specials->saved.known_rooms[i]) {
	sprintf(buf, "%d: %s\r\n", i + 1,
		world[ch->player_specials->saved.known_rooms[i]].name);
	send_to_char(buf, ch);
      }
    }
    return;
  }
  target = ch->in_room;
  name = "the room";

  CREATE(search_obj, struct search_event_obj, 1);
  search_obj->ch = ch;
  search_obj->target = target;

  GET_ACTION(ch) = event_create(learn_event, search_obj, (IS_TRUSTED(ch)) ? 10 :
				((30 * SECS_PER_MUD_MINUTE * PASSES_PER_SEC) - 10 * (GET_INT(ch) + GET_WIS(ch))));


  WAIT_STATE(ch, 20);
  sprintf(buf, "You start to learn %s.", name);
  act(buf, FALSE, ch, NULL, NULL, TO_CHAR);
  sprintf(buf, "$n starts to examine %s carefully.", name);
  act(buf, TRUE, ch, NULL, NULL, PROXIMITY);
/*
   sprintf(buf, "&+CDEBUG:&n You're learning room: %d\r\n", target);
   send_to_char(buf, ch);
 */
}

ACMD(do_clair)
{
  int location = 0, length = 0;
  struct location_data original_loc;
  char *line;
  struct affected_type *hjp, *next;
  extern void list_obj_to_char(struct obj_data *list, struct char_data *ch, int mode,
			       bool show, int xyz);
  extern void list_char_to_char(struct char_data *list, struct char_data *ch);
  extern void smallmap_look(struct char_data *ch, sh_int size, int dif_room);

  if (!affected_by_spell(ch, SPELL_CLAIRVOYANCE)) {
    send_to_char("You cant do that without first casting the spell.\r\n", ch);
    return;
  }
  if (!*argument) {
    send_to_char("What do you want to do there?\r\n", ch);
    return;
  }
  for (hjp = ch->affected; hjp; hjp = next) {
    next = hjp->next;
    if (hjp->type == SPELL_CLAIRVOYANCE) {
      location = hjp->modifier;
#ifdef DEBUG_GRANOR
      sprintf(buf, "Room: %d", location);
      send_to_char(buf, ch);
#endif
      break;
    }
  }

  if (!location) {
    send_to_char("Where did you target this spell?\r\n", ch);
    return;
  }
  /* a location has been found. */
  original_loc.real_room = ch->in_room;
  original_loc.x_cord = GET_X_CORD(ch);
  original_loc.y_cord = GET_Y_CORD(ch);
  original_loc.z_cord = GET_Z_CORD(ch);

  line = any_one_arg(argument, arg);

  for (length = strlen(arg), cmd = 0; *complete_cmd_info[cmd].command != '\n'; cmd++)
    if (!strncmp(complete_cmd_info[cmd].command, arg, length))
      if (GET_LEVEL(ch) >= complete_cmd_info[cmd].minimum_level)
	break;

  if (*complete_cmd_info[cmd].command == '\n') {
    send_to_char("Unknown command.\r\n", ch);
    return;
  } else if (complete_cmd_info[cmd].command_pointer == NULL) {
    send_to_char("Sorry, that command hasn't been implemented yet.\r\n", ch);
    return;
  } else if (str_cmp(complete_cmd_info[cmd].command, "look")) {
    send_to_char("You can only look with clair.\r\n", ch);
    return;
  }
  if (!*line) {
    smallmap_look(ch, view_distance(ch), location);
    list_obj_to_char(world[location].contents, ch, 0, FALSE, FALSE);
    list_char_to_char(world[location].people, ch);
    return;
  }
  char_from_room(ch);
  char_to_room(ch, location, -1);

  SET_BIT(ch->char_specials.saved.affected_by4, AFF4_CLAIR);

  ((*complete_cmd_info[cmd].command_pointer) (ch, line, cmd, complete_cmd_info[cmd].subcmd));

  REMOVE_BIT(ch->char_specials.saved.affected_by4, AFF4_CLAIR);

  /* check if the char is still there */
  if (ch->in_room == location) {
    char_from_room(ch);
    char_to_room(ch, original_loc.real_room, -1);
    GET_X_CORD(ch) = original_loc.x_cord;
    GET_Y_CORD(ch) = original_loc.y_cord;
    GET_Z_CORD(ch) = original_loc.z_cord;
  }
}

ACMD(do_levitate)
{
  char *line;
  struct affected_type *hjp, *next, *hjp2;
  long charid = 0;
  int found, up = FALSE, down = FALSE, distance = 0, length = 0;
  struct char_data *tch = NULL;
  struct obj_data *tobj = NULL;
  extern int find_spell_target(struct char_data *ch, char *arg, int spellnum,
			 struct char_data **tch, struct obj_data **tobj);

  if (!affected_by_spell(ch, SPELL_LEVITATE)) {
    send_to_char("You cant do that without first casting the spell.\r\n", ch);
    return;
  }
  if (!*argument) {
    send_to_char("Up or down?\r\n", ch);
    return;
  }
  for (hjp = ch->affected; hjp; hjp = next) {
    next = hjp->next;
    if (hjp->type == SPELL_LEVITATE) {
      charid = hjp->modifier;
      break;
    }
  }

  sprintf(arg, "%c%ld", UID_CHAR, charid);

  found = find_spell_target(ch, arg, SPELL_LEVITATE, &tch, &tobj);

  if (!found) {
    send_to_char("Upon what/whom did you target this spell?\r\n", ch);
    return;
  }
  for (hjp2 = tch->affected; hjp2; hjp2 = next) {
    next = hjp2->next;
    if (hjp2->type == SPELL_LEVITATE) {
      charid = hjp2->modifier;
      break;
    }
  }

  if (GET_ID(ch) != charid) {
    send_to_char("Someone has taken control of your victim.\r\n", ch);
    affect_remove(ch, hjp);
    return;
  }
  line = any_one_arg(argument, arg);

  for (length = strlen(arg), cmd = 0; *complete_cmd_info[cmd].command != '\n'; cmd++)
    if (!strncmp(complete_cmd_info[cmd].command, arg, length))
      if (GET_LEVEL(ch) >= complete_cmd_info[cmd].minimum_level)
	break;

  if (*complete_cmd_info[cmd].command == '\n') {
    send_to_char("Unknown command.\r\n", ch);
    return;
  } else if (complete_cmd_info[cmd].command_pointer == NULL) {
    send_to_char("Sorry, that command hasn't been implemented yet.\r\n", ch);
    return;
  } else if ((str_cmp(complete_cmd_info[cmd].command, "up")) &&
	     (str_cmp(complete_cmd_info[cmd].command, "down"))) {
    send_to_char("You can only move them up or down.\r\n", ch);
    return;
  }
  if (str_cmp(complete_cmd_info[cmd].command, "up"))
    down = TRUE;

  if (str_cmp(complete_cmd_info[cmd].command, "down")) {
    if ((GET_WEIGHT(tch) + IS_CARRYING_W(tch)) > (30 * GET_LEVEL(ch))) {
      send_to_char("That person is too heavy to lift any higher.\r\n", ch);
      return;
    }
    up = TRUE;
  }
  skip_spaces(&line);

  if ((!*line) || (line == NULL) || !str_cmp(line, "")) {
    send_to_char("How far do you want to move them?\r\n", ch);
    return;
  }
  if (!is_number(line)) {
    send_to_char("You must specify an actual number.\r\n", ch);
    return;
  }
  distance = atoi(line);

  if (distance < 0) {
    send_to_char("You must specify a positive number.\r\n", ch);
    return;
  }
  distance = BOUNDED(0, distance, (tch == ch) ? 6 : 3);

  if (down)
    distance *= -1;

  GET_Z_CORD(tch) += distance;
  GET_Z_CORD(tch) = BOUNDED(0, GET_Z_CORD(tch), world[tch->in_room].z_size);

  sprintf(buf, "$N is %s by an invisible force.", (up) ? ("lifted up") :
	  ("lowered"));
  act(buf, FALSE, ch, NULL, tch, TO_NOTVICT);

  sprintf(buf, "You %s $N", (up) ? ("lift") : ("lower"));
  act(buf, FALSE, ch, NULL, tch, TO_CHAR);

  sprintf(buf, "You are %s by an invisible force.", (up) ? ("lifted up") :
	  ("lowered"));
  act(buf, FALSE, ch, NULL, tch, TO_VICT);

  if (GET_Z_CORD(tch) == world[tch->in_room].z_size) {
    act("$N is bounced off the ceiling.", FALSE, ch, NULL, tch, TO_NOTVICT);
    act("You smash $N's head against the ceiling.", FALSE, ch, NULL, tch, TO_CHAR);
    act("You crack your head against the ceiling.", FALSE, ch, NULL, tch, TO_VICT);
  }
  if (!GET_Z_CORD(tch)) {
    act("$N is dropped to the ground.", FALSE, ch, NULL, tch, TO_NOTVICT);
    act("You drop $N to the ground.", FALSE, ch, NULL, tch, TO_CHAR);
    act("You are dropped to the ground.", FALSE, ch, NULL, tch, TO_VICT);
  }
}

ACMD(do_angle)
{
  if (*argument) {
    send_to_char("This command doesnt take any arguments.\r\n", ch);
    return;
  }
  if (ch->player_specials->saved.angle_type)
    ch->player_specials->saved.angle_type--;
  else
    ch->player_specials->saved.angle_type++;

  sprintf(buf, "You are now using %s angles.\r\n", (ch->player_specials->saved.angle_type) ? "cardinal" : "cartesian");
  send_to_char(buf, ch);
}

ACMD(do_condition)
{
  struct char_data *vict = NULL;
  char vict_name[MAX_INPUT_LENGTH];

  if (!argument || !*argument)
    vict = ch;
  else {
    one_argument(argument, vict_name);

    if (!(vict = get_char_room_vis(ch, vict_name))) {
      send_to_char("Get whos condition?\r\n", ch);
      return;
    }
  }

  if (!vict) {
    send_to_char("error. contact a coder.\r\n", ch);
    return;
  }
  sprintf(buf,
	  "               _\r\n"
	  "              / \\ %d\r\n"
	  "              \\ /\r\n"
	  "               |\r\n"
	  "            /-----\\ \r\n"
	  "       %3d /   |   \\ %d\r\n"
	  "  WEAPON   |   |%3d|   SHIELD\r\n"
	  "       %3d |   |   | %d\r\n"
	  "              / \\   \r\n"
	  "         %3d /   \\ %d\r\n"
	  "    LEFT    /     \\    RIGHT\r\n"
	  "       %3d /       \\ %d\r\n",
	  (400 * GET_HIT_HD(vict)) / GET_MAX_HIT(vict),
	  (200 * GET_HIT_WA(vict)) / GET_MAX_HIT(vict),
	  (200 * GET_HIT_SA(vict)) / GET_MAX_HIT(vict),
	  (100 * GET_HIT_TO(vict)) / GET_MAX_HIT(vict),
	  (400 * GET_HIT_WH(vict)) / GET_MAX_HIT(vict),
	  (400 * GET_HIT_SH(vict)) / GET_MAX_HIT(vict),
	  (200 * GET_HIT_LL(vict)) / GET_MAX_HIT(vict),
	  (200 * GET_HIT_RL(vict)) / GET_MAX_HIT(vict),
	  (400 * GET_HIT_RF(vict)) / GET_MAX_HIT(vict),
	  (400 * GET_HIT_RF(vict)) / GET_MAX_HIT(vict));
  send_to_char(buf, ch);
}

ACMD(do_aim)
{
  int i = 0;
  extern char *location_names[];
  char *target = NULL;

  if (!*argument) {
    sprintf(buf, "You are targetting your victim's %s.\n",
	    location_names[GET_AIMING(ch)]);
    send_to_char(buf, ch);
    return;
  }
  skip_spaces(&argument);
  target = strtok(argument, "\0");

  for (i = 0; i < NUMBER_LOCATIONS; i++) {
    if (!str_cmp(target, location_names[i])) {
      ch->char_specials.target_location = i;
      break;
    }
  }

  if (i == NUMBER_LOCATIONS)
    send_to_char("That is not a valid location.\n", ch);

  sprintf(buf, "You are targetting your victim's %s.\n",
	  location_names[GET_AIMING(ch)]);
  send_to_char(buf, ch);
}
