/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#define __DB_C__

#include "conf.h"
#include "sysdep.h"
#include <signal.h>

#include "guild.h"
#include "structs.h"
#include "utils.h"
#include "db.h"

#include "comm.h"
#include "handler.h"
#include "spells.h"
#include "mail.h"
#include "map.h"
#include "interpreter.h"
#include "house.h"
#include "quest.h"
#include "boards.h"
#include "olc.h"
#include "chatlink.h"
#include "dg_scripts.h"

char err_buf[MAX_STRING_LENGTH];
void init_spells(struct char_data *ch);
void read_mud_date_from_file(void);
void write_mud_date_to_file(void);
void init_language(struct char_data *ch);
void load_path_from_mob(FILE * mob_f, struct index_data *mob_index);
char fread_letter(FILE * fp);
char *fread_word(FILE * fp);
void mag_assign_skills(void);
void generate_mapgrid(FILE * fl, int room);
void read_quest(FILE * fp, int mob);
void random_gems(struct obj_data *, int);
void random_jewelry(struct obj_data *, int);
void unload_room_maps(int zone);
void convert_mob_if_needed(int i);
void GiveSpellsToMob(struct char_data *ch);
extern void olc_add_to_save_list(int zone, byte type);
extern char *three_arguments(char *argument, char *first_arg, char *second_arg, char *third_arg);
extern void randomize_mob(struct char_data *ch);
extern int rev_dir[];
int special(struct char_data *ch, int cmd, char *command, char *arg, int why_called);
extern void char_to_object(struct char_data *ch, struct obj_data *obj);
extern void add_follower(struct char_data *ch, struct char_data *leader);
extern ACMD(do_mount);

/**************************************************************************
*  declarations of most of the 'global' variables                         *
**************************************************************************/

struct room_data *world = NULL;	/* array of rooms                */
int top_of_world = 0;		/* ref to top element of world   */
long top_of_grid_count = 0;
long top_of_loaded_mobs_table = 0;	/* counts structs in game usage */
struct char_data *character_list = NULL;	/* global linked list of chars */
struct char_data *mob_path_list = NULL;		/* Thanks to GG for the idea */
struct index_data *mob_index;	/* index table for mobile file   */
struct char_data *mob_proto;	/* prototypes for mobs           */
int top_of_mobt = 0;		/* top of mobile index table     */
int top_of_door_table = 0;	/* just used for mreport         */
struct path_q *mobile_paths = NULL;	/* path memory stuff for mobs */
struct guild_info *guild_index = NULL;	/*  Clan global */
int guild_top = 0;		/* Top of guild struct */
struct index_data **trig_index;	/* index table for triggers      */
int top_of_trigt = 0;		/* top of trigger index table    */
int max_id = 150000;		/* Next unique identifier to use */
struct obj_data *object_list = NULL;	/* global linked list of objs    */
struct index_data *obj_index;	/* index table for object file   */
struct obj_data *obj_proto;	/* prototypes for objs           */
int top_of_objt = 0;		/* top of object index table     */

struct zone_data *zone_table;	/* zone table                    */
int top_of_zone_table = 0;	/* top element of zone tab       */
struct message_list fight_messages[MAX_MESSAGES];	/* fighting messages     */
struct aq_data *aquest_table;
int top_of_aquestt = 0;
struct player_index_element *player_table = NULL;	/* index to plr file     */
FILE *player_fl = NULL;		/* file desc of player file      */
int top_of_p_table = 0;		/* ref to top of table           */
int top_of_p_file = 0;		/* ref of size of p file         */
long top_idnum = 0;		/* highest idnum in use          */

int no_mail = 0;		/* mail disabled?                */
int mini_mud = 0;		/* mini-mud mode?                */
int no_rent_check = 0;		/* skip rent check on boot?      */
time_t boot_time = 0;		/* time of mud boot              */
int restricted = 0;		/* level of game restrictedion   */
int r_mortal_start_room;	/* rnum of mortal start room     */
int r_immort_start_room;	/* rnum of immort start room     */
int r_frozen_start_room;	/* rnum of frozen start room     */

char *credits = NULL;		/* game credits                  */
char *news = NULL;		/* mud news                      */
char *motd = NULL;		/* message of the day - mortals */
char *imotd = NULL;		/* message of the day - immorts */
char *help = NULL;		/* help screen                   */
char *info = NULL;		/* info page                     */
char *wizlist = NULL;		/* list of higher gods           */
char *immlist = NULL;		/* list of peon gods             */
char *background = NULL;	/* background story              */
char *handbook = NULL;		/* handbook for new immortals    */
char *policies = NULL;		/* policies page                 */

struct help_index_element *help_table = 0;	/* the help table        */
int top_of_helpt = 0;		/* top of help index table       */
int map_loaded = FALSE;		/* temp var used during room loading    */
struct time_info_data time_info;	/* the infomation about the time */
struct player_special_data dummy_mob;	/* dummy spec area for mobs      */
struct reset_q_type reset_q;	/* queue of zones to be reset    */
struct board_type_info *board_info;

/* local functions */
void setup_dir(FILE * fl, int room, int dir);
void index_boot(int mode);
void discrete_load(FILE * fl, int mode);
void parse_trigger(FILE * fl, int virtual_nr);
void parse_room(FILE * fl, int virtual_nr);
void parse_mobile(FILE * mob_f, int nr);
char *parse_object(FILE * obj_f, int nr);
void parse_quest(FILE * quest_f, int nr);
void load_zones(FILE * fl, char *zonename);
void load_help(FILE * fl, int saving);
void assign_mobiles(void);
void assign_objects(void);
void assign_rooms(void);
void assign_zones(void);
void assign_kanthas_procs(void);
void assign_istus_procs(void);
void assign_the_shopkeepers(void);
void build_player_index(void);
void char_to_store(struct char_data *ch, struct char_file_u *st);
void store_to_char(struct char_file_u *st, struct char_data *ch);
int is_empty(int zone_nr);
void reset_zone(int zone);
int file_to_string(char *name, char *buf);
int file_to_string_alloc(char *name, char **buf);
void check_start_rooms(void);
void renum_world(void);
void convert_rooms(void);
void renum_zone_table(void);
void log_zone_error(int zone, int cmd_no, char *message);
void reset_time(void);
void clear_char(struct char_data *ch);

/* external functions */
extern void parse_guild(FILE * fl, int virtual_nr);
extern struct descriptor_data *descriptor_list;
void load_messages(void);
void weather_and_time(int mode);
void mag_assign_spells(void);
void boot_social_messages(void);
void create_command_list(void);
void boot_desc_data_pc(void);
void boot_desc_data_mob(void);
void boot_fake_chars(void);
void update_obj_file(void);	/* In objsave.c */
void sort_commands(void);
void sort_spells(void);
void load_banned(void);
void Read_Invalid_List(void);
void boot_the_shops(FILE * shop_f, char *filename, int rec_count);
int hsort(const void *a, const void *b);
void copy_mobile(struct char_data *tmob, struct char_data *fmob);
void clearMemory(struct char_data *ch);
void set_saving_throws(struct char_data *ch);
void weather_setup(void);
SPECIAL(mob_generating_room);

/* external vars */
extern int no_specials;

#define READ_SIZE 256

struct time_write {
  int year, month, day;
};

/*************************************************************************
*  routines for booting the system                                       *
*************************************************************************/

/* this is necessary for the autowiz system */
void reboot_wizlists(void)
{
  file_to_string_alloc(WIZLIST_FILE, &wizlist);
  file_to_string_alloc(IMMLIST_FILE, &immlist);
}


ACMD(do_reload)
{
  int i;

  one_argument(argument, arg);

  if (!str_cmp(arg, "all") || *arg == '*') {
    file_to_string_alloc(WIZLIST_FILE, &wizlist);
    file_to_string_alloc(IMMLIST_FILE, &immlist);
    file_to_string_alloc(NEWS_FILE, &news);
    file_to_string_alloc(CREDITS_FILE, &credits);
    file_to_string_alloc(MOTD_FILE, &motd);
    file_to_string_alloc(IMOTD_FILE, &imotd);
    file_to_string_alloc(HELP_PAGE_FILE, &help);
    file_to_string_alloc(INFO_FILE, &info);
    file_to_string_alloc(POLICIES_FILE, &policies);
    file_to_string_alloc(HANDBOOK_FILE, &handbook);
    file_to_string_alloc(BACKGROUND_FILE, &background);
  } else if (!str_cmp(arg, "wizlist"))
    file_to_string_alloc(WIZLIST_FILE, &wizlist);
  else if (!str_cmp(arg, "immlist"))
    file_to_string_alloc(IMMLIST_FILE, &immlist);
  else if (!str_cmp(arg, "news"))
    file_to_string_alloc(NEWS_FILE, &news);
  else if (!str_cmp(arg, "credits"))
    file_to_string_alloc(CREDITS_FILE, &credits);
  else if (!str_cmp(arg, "motd"))
    file_to_string_alloc(MOTD_FILE, &motd);
  else if (!str_cmp(arg, "imotd"))
    file_to_string_alloc(IMOTD_FILE, &imotd);
  else if (!str_cmp(arg, "help"))
    file_to_string_alloc(HELP_PAGE_FILE, &help);
  else if (!str_cmp(arg, "info"))
    file_to_string_alloc(INFO_FILE, &info);
  else if (!str_cmp(arg, "policy"))
    file_to_string_alloc(POLICIES_FILE, &policies);
  else if (!str_cmp(arg, "handbook"))
    file_to_string_alloc(HANDBOOK_FILE, &handbook);
  else if (!str_cmp(arg, "background"))
    file_to_string_alloc(BACKGROUND_FILE, &background);
  else if (!str_cmp(arg, "xhelp")) {
    if (help_table) {
      for (i = 0; i <= top_of_helpt; i++) {
	if (help_table[i].keyword)
	  FREE(help_table[i].keyword);
	if (help_table[i].entry && !help_table[i].duplicate)
	  FREE(help_table[i].entry);
      }
      FREE(help_table);
    }
    top_of_helpt = 0;
    index_boot(DB_BOOT_HLP);
  } else {
    send_to_char("Unknown reload option.\r\n", ch);
    return;
  }

  send_to_char(OK, ch);
}


void current_temp_code(void)
{
  int room_nr;

  for (room_nr = 0; room_nr <= top_of_world; room_nr++)
/* lets try adding to all rooms, just in case a river was drawn */
/*    if (WILDERNESS(room_nr) && IS_WATER_ROOM(room_nr)) { */
    world[room_nr].current_direction = SOUTH;
  if (world[room_nr].dir_option[SOUTH] &&
      (world[room_nr].dir_option[SOUTH]->to_room) &&
      IS_WATER_ROOM(world[room_nr].dir_option[SOUTH]->to_room))
    world[room_nr].current_direction = SOUTH;
  else if (world[room_nr].dir_option[SOUTHEAST] &&
	   (world[room_nr].dir_option[SOUTHEAST]->to_room) &&
	   IS_WATER_ROOM(world[room_nr].dir_option[SOUTHEAST]->to_room))
    world[room_nr].current_direction = SOUTHEAST;
  else if (world[room_nr].dir_option[SOUTHWEST] &&
	   (world[room_nr].dir_option[SOUTHWEST]->to_room) &&
	   IS_WATER_ROOM(world[room_nr].dir_option[SOUTHWEST]->to_room))
    world[room_nr].current_direction = SOUTHWEST;
  else if (world[room_nr].dir_option[EAST] &&
	   (world[room_nr].dir_option[EAST]->to_room) &&
	   IS_WATER_ROOM(world[room_nr].dir_option[EAST]->to_room))
    world[room_nr].current_direction = EAST;
  else if (world[room_nr].dir_option[NORTHEAST] &&
	   (world[room_nr].dir_option[NORTHEAST]->to_room) &&
	   IS_WATER_ROOM(world[room_nr].dir_option[NORTHEAST]->to_room))
    world[room_nr].current_direction = NORTHEAST;
  if (WILDERNESS(room_nr) && IS_WATER_ROOM(room_nr))
    world[room_nr].current_speed = 50;
  else
    world[room_nr].current_speed = 25;
/*    }  */
}

void boot_world(void)
{
  log("Loading zone table.");
  index_boot(DB_BOOT_ZON);

  log("Loading triggers and generating index.");
  index_boot(DB_BOOT_TRG);

  log("Loading rooms.");
  index_boot(DB_BOOT_WLD);

  log("Renumbering rooms.");
  renum_world();

  log("Converting rooms.");
  convert_rooms();

/**** temp ***/
  current_temp_code();

  log("Checking start rooms.");
  check_start_rooms();

  log("Loading mobs and generating index.");
  index_boot(DB_BOOT_MOB);

  log("Loading objs and generating index.");
  index_boot(DB_BOOT_OBJ);

  log("Loading guilds and generating index.");
  index_boot(DB_BOOT_GUILD);

  log("Renumbering zone table.");
  renum_zone_table();

  log("Loading boards.");
  Board_init_boards();

  if (!no_specials) {
    log("Loading shops.");
    index_boot(DB_BOOT_SHP);
  }
#if 1
  log("Loading quests.");
  index_boot(DB_BOOT_QST);
#endif
}



/* body of the booting system */
void boot_db(void)
{
  int i;

  log("Boot db -- BEGIN.");

  log("Resetting the game time:");
  reset_time();

  log("Reading news, credits, help, bground, info & motds.");
  file_to_string_alloc(NEWS_FILE, &news);
  file_to_string_alloc(CREDITS_FILE, &credits);
  file_to_string_alloc(MOTD_FILE, &motd);
  file_to_string_alloc(IMOTD_FILE, &imotd);
  file_to_string_alloc(HELP_PAGE_FILE, &help);
  file_to_string_alloc(INFO_FILE, &info);
  file_to_string_alloc(WIZLIST_FILE, &wizlist);
  file_to_string_alloc(IMMLIST_FILE, &immlist);
  file_to_string_alloc(POLICIES_FILE, &policies);
  file_to_string_alloc(HANDBOOK_FILE, &handbook);
  file_to_string_alloc(BACKGROUND_FILE, &background);


  boot_world();

  log("Loading help entries.");
  index_boot(DB_BOOT_HLP);

  log("Generating player index.");
  build_player_index();

  log("Loading fight messages.");
  load_messages();

  log("Loading social messages.");
  boot_social_messages();

  log("Loading chat data.");
#ifdef ELIZA
  startchat(CHAT_FILE);
#endif
  log("Create command list.");
  create_command_list();

  log("Loading character description data.");
  boot_desc_data_pc();
  boot_desc_data_mob();
  boot_fake_chars();

  log("Assigning function pointers:");

  if (!no_specials) {
    log("   Mobiles.");
    assign_mobiles();
    log("   Shopkeepers.");
    assign_the_shopkeepers();
    log("   Objects.");
    assign_objects();
    log("   Rooms.");
    assign_rooms();
    log("   Zones.");
    assign_zones();
    log("  Coder Procs.");
    assign_kanthas_procs();
    assign_istus_procs();
  }
  log("   Spells.");
  mag_assign_spells();

  log("Assigning spell levels.");
  init_spell_levels();

  log("   Skills.");
  mag_assign_skills();

  log("Sorting command list and spells.");
  sort_commands();
  sort_spells();

  log("Setting up weather");
  weather_setup();

  log("Booting mail system.");
  if (!scan_file()) {
    log("    Mail boot failed -- Mail system disabled");
    no_mail = 1;
  }
  log("Reading banned site and invalid-name list.");
  load_banned();
  Read_Invalid_List();

  if (!no_rent_check) {
    log("Deleting timed-out crash and rent files:");
    update_obj_file();
    log("Done.");
  }
  for (i = 0; i <= top_of_zone_table; i++) {
    sprintf(buf2, "Resetting %s (rooms %d-%d).",
	    zone_table[i].name, (i ? (zone_table[i - 1].top + 1) : 0),
	    zone_table[i].top);
/*    log(buf2); */
    reset_zone(i);
    zone_table[i].unloaded = FALSE;
  }

  reset_q.head = reset_q.tail = NULL;

  if (!mini_mud) {
    log("Booting houses.");
    House_boot();
  }
  boot_time = time(0);

  /* time, everything else setup - lets check the paths */
  check_path();

  log("Boot db -- DONE.");
}


/* reset the time in the game from file */
void reset_time(void)
{
  long beginning_of_time = 650336715;
  struct time_info_data mud_time_passed(time_t t2, time_t t1);

  time_info = mud_time_passed(time(0), beginning_of_time);
  read_mud_date_from_file();

  if (time_info.hours <= 4)
    sunlight = SUN_DARK;
  else if (time_info.hours == 5)
    sunlight = SUN_RISE;
  else if (time_info.hours <= 20)
    sunlight = SUN_LIGHT;
  else if (time_info.hours == 21)
    sunlight = SUN_SET;
  else
    sunlight = SUN_DARK;

  sprintf(buf, "   Current Gametime: %dH %dD %dM %dY.", time_info.hours,
	  time_info.day, time_info.month, time_info.year);
  log(buf);
}

/* generate index table for the player file */
void build_player_index(void)
{
  int nr = -1, i;
  long size, recs;
  struct char_file_u dummy;

  if (!(player_fl = fopen(PLAYER_FILE, "r+b"))) {
    if (errno != ENOENT) {
      perror("SYSERR: fatal error opening playerfile");
      raise(SIGSEGV);
    } else {
      log("No playerfile.  Creating a new one.");
      touch(PLAYER_FILE);
      if (!(player_fl = fopen(PLAYER_FILE, "r+b"))) {
	perror("SYSERR: fatal error opening playerfile");
	raise(SIGSEGV);
      }
    }
  }
  fseek(player_fl, 0L, SEEK_END);
  size = ftell(player_fl);
  rewind(player_fl);
  if (size % sizeof(struct char_file_u)) {
    fprintf(stderr, "\aWARNING:  PLAYERFILE IS PROBABLY CORRUPT!\n");
    log("WARNING: PLAYERFILE IS PROBABLY CORRUPT!");
  }
  recs = size / sizeof(struct char_file_u);
  if (recs) {
    sprintf(buf, "   %ld players in database.", recs);
    log(buf);
    CREATE(player_table, struct player_index_element, recs);
  } else {
    player_table = NULL;
    top_of_p_file = top_of_p_table = -1;
    return;
  }

  for (; !feof(player_fl);) {
    fread(&dummy, sizeof(struct char_file_u), 1, player_fl);
    if (!feof(player_fl)) {	/* new record */
      nr++;
      CREATE(player_table[nr].name, char, strlen(dummy.name) + 1);
      for (i = 0;
	 (*(player_table[nr].name + i) = LOWER(*(dummy.name + i))); i++);
      player_table[nr].id = dummy.char_specials_saved.idnum;
      top_idnum = MAX(top_idnum, dummy.char_specials_saved.idnum);
    }
  }

  top_of_p_file = top_of_p_table = nr;
}



/* function to count how many hash-mark delimited records exist in a file */
int count_hash_records(FILE * fl)
{
  char buf[128];
  int count = 0;

  while (fgets(buf, 128, fl))
    if (*buf == '#')
      count++;

  return count;
}



void index_boot(int mode)
{
  char *index_filename, *prefix = NULL;
  FILE *index, *db_file;
  int rec_count = 0;

  switch (mode) {
  case DB_BOOT_TRG:
    prefix = TRG_PREFIX;
    break;
  case DB_BOOT_WLD:
    prefix = WLD_PREFIX;
    break;
  case DB_BOOT_MOB:
    prefix = MOB_PREFIX;
    break;
  case DB_BOOT_OBJ:
    prefix = OBJ_PREFIX;
    break;
  case DB_BOOT_ZON:
    prefix = ZON_PREFIX;
    break;
  case DB_BOOT_SHP:
    prefix = SHP_PREFIX;
    break;
  case DB_BOOT_HLP:
    prefix = HLP_PREFIX;
    break;
  case DB_BOOT_GUILD:
    prefix = GUILD_PREFIX;
    break;
  case DB_BOOT_QST:
    prefix = QST_PREFIX;
    break;
  default:
    log("SYSERR: Unknown subcommand to index_boot!");
    raise(SIGSEGV);
    break;
  }

  if (mini_mud)
    index_filename = MINDEX_FILE;
  else
    index_filename = INDEX_FILE;

  sprintf(buf2, "%s/%s", prefix, index_filename);

  if (!(index = fopen(buf2, "r"))) {
    sprintf(buf1, "SYSERR: opening index file '%s'", buf2);
    perror(buf1);
    raise(SIGSEGV);
  }
  /* first, count the number of records in the file so we can malloc */
  fscanf(index, "%s\n", buf1);
  while (*buf1 != '$') {
    sprintf(buf2, "%s/%s", prefix, buf1);
    if (!(db_file = fopen(buf2, "r"))) {
      perror(buf2);
      log("SYSERR: file listed in index not found");
      fscanf(index, "%s\n", buf1);
      continue;
    } else {
      if (mode == DB_BOOT_ZON)
	rec_count++;
      else
	rec_count += count_hash_records(db_file);
    }

    fclose(db_file);
    fscanf(index, "%s\n", buf1);
  }

  /* Exit if 0 records, unless this is shops */
  if (!rec_count) {
    if (mode == DB_BOOT_SHP)
      return;
    log("SYSERR: boot error - 0 records counted");
    raise(SIGSEGV);
  }
  rec_count++;

  switch (mode) {
  case DB_BOOT_TRG:
    CREATE(trig_index, struct index_data *, rec_count);
    break;
  case DB_BOOT_WLD:
    CREATE(world, struct room_data, rec_count);
    break;
  case DB_BOOT_MOB:
    CREATE(mob_proto, struct char_data, rec_count);
    CREATE(mob_index, struct index_data, rec_count);
    break;
  case DB_BOOT_OBJ:
    CREATE(obj_proto, struct obj_data, rec_count);
    CREATE(obj_index, struct index_data, rec_count);
    break;
  case DB_BOOT_ZON:
    CREATE(zone_table, struct zone_data, rec_count);
    break;
  case DB_BOOT_HLP:
    CREATE(help_table, struct help_index_element, rec_count * 2);
    break;
  case DB_BOOT_GUILD:
    CREATE(guild_index, struct guild_info, rec_count + 10);
    guild_top = rec_count - 1;
    break;
  case DB_BOOT_QST:
    CREATE(aquest_table, struct aq_data, rec_count);
    break;
  }

  rewind(index);
  fscanf(index, "%s\n", buf1);
  while (*buf1 != '$') {
    sprintf(buf2, "%s/%s", prefix, buf1);
    if (!(db_file = fopen(buf2, "r"))) {
      perror(buf2);
      raise(SIGSEGV);
    }
    switch (mode) {
    case DB_BOOT_TRG:
    case DB_BOOT_WLD:
    case DB_BOOT_OBJ:
    case DB_BOOT_MOB:
    case DB_BOOT_GUILD:
    case DB_BOOT_QST:
      discrete_load(db_file, mode);
      break;
    case DB_BOOT_ZON:
      load_zones(db_file, buf2);
      break;
    case DB_BOOT_HLP:
      if (!strcmp(buf1, "hedit.hlp"))
	load_help(db_file, TRUE);
      else
	load_help(db_file, FALSE);
      break;
    case DB_BOOT_SHP:
      boot_the_shops(db_file, buf2, rec_count);
      break;
    }

    fclose(db_file);
    fscanf(index, "%s\n", buf1);
  }

  /* sort the help index */
  if (mode == DB_BOOT_HLP) {
    qsort(help_table, top_of_helpt, sizeof(struct help_index_element), hsort);
    top_of_helpt--;
  }
  fclose(index);
}


void discrete_load(FILE * fl, int mode)
{
  int nr = -1, last = 0;
  char line[256];

  char *modes[] =
  {"wld", "mob", "obj", "zon", "shp", "hlp", "qld", "qst", "trg"};

  for (;;) {
    /*
     * we have to do special processing with the obj files because they have
     * no end-of-record marker :(
     */
    if (mode != DB_BOOT_OBJ || nr < 0)
      if (!get_line(fl, line)) {
	fprintf(stderr, "SYSERR: Format error after %s #%d (file not terminated with '$'?)\n", buf2, /* modes[mode], */ nr);
	raise(SIGSEGV);
      }
    if (*line == '$')
      return;

    if (*line == '#') {
      last = nr;
      if (sscanf(line, "#%d", &nr) != 1) {
	fprintf(stderr, "SYSERR: Format error after %s #%d\n", modes[mode], last);
	raise(SIGSEGV);
      }
      if (nr >= 999999)
	return;
      else
	switch (mode) {
	case DB_BOOT_TRG:
	  parse_trigger(fl, nr);
	  break;
	case DB_BOOT_WLD:
	  parse_room(fl, nr);
	  break;
	case DB_BOOT_MOB:
	  parse_mobile(fl, nr);
	  break;
	case DB_BOOT_OBJ:
	  strcpy(line, parse_object(fl, nr));
	  break;
	case DB_BOOT_GUILD:
	  parse_guild(fl, nr);
	  break;
	case DB_BOOT_QST:
	  parse_quest(fl, nr);
	  break;
	}
    } else {
      fprintf(stderr, "SYSERR: Format error in %s file near %s #%d\n",
	      modes[mode], modes[mode], nr);
      fprintf(stderr, "Offending line: '%s'\n", line);
      raise(SIGSEGV);
    }
  }
  if (mode == DB_BOOT_WLD)	/* we'll load them again later */
    unload_room_maps(nr / 100);
}


long asciiflag_conv(char *flag)
{
  long flags = 0;
  int is_number = 1;
  register char *p;

  for (p = flag; *p; p++) {
    if (islower(*p))
      flags |= 1 << (*p - 'a');
    else if (isupper(*p))
      flags |= 1 << (26 + (*p - 'A'));

    if (!isdigit(*p))
      is_number = 0;
  }

  if (is_number)
    flags = atol(flag);

  return flags;
}

char fread_letter(FILE * fp)
{
  char c;
  do {
    c = getc(fp);
  } while (isspace(c));
  return c;
}

/* load the rooms */
void parse_room(FILE * fl, int virtual_nr)
{
  static int room_nr = 0, zone = 0;
  int t[10], i;
  char line[256], flags[128];
  struct extra_descr_data *new_descr;
  char letter = 0;

  sprintf(buf2, "room #%d", virtual_nr);

  if (virtual_nr <= (zone ? zone_table[zone - 1].top : -1)) {
    fprintf(stderr, "SYSERR: Room #%d is below zone %d.\n", virtual_nr, zone);
    raise(SIGSEGV);
  }
  while (virtual_nr > zone_table[zone].top)
    if (++zone > top_of_zone_table) {
      fprintf(stderr, "SYSERR: Room %d is outside of any zone.\n", virtual_nr);
      raise(SIGSEGV);
    }
  world[room_nr].zone = zone;
  world[room_nr].number = virtual_nr;
  world[room_nr].id = 10000 + virtual_nr;
  world[room_nr].name = fread_string(fl, buf2);
  world[room_nr].description = fread_string(fl, buf2);

  if (!get_line(fl, line) || sscanf(line, " %d %s %d ", t, flags, t + 2) != 3) {
    fprintf(stderr, "SYSERR: Format error in room #%d\n", virtual_nr);
    raise(SIGSEGV);
  }
  world[room_nr].resources = t[0];
  world[room_nr].room_flags = asciiflag_conv(flags);
  world[room_nr].sector_type = t[2];

  if (!get_line(fl, line) || sscanf(line, " %d %d %d %d", t, t + 1, t + 2, t + 3) != 4) {
    fprintf(stderr, "SYSERR: Format error in room #%d\n", virtual_nr);
    raise(SIGSEGV);
  }
  world[room_nr].kingdom_num = t[0];
  world[room_nr].kingdom_type = t[1];
  world[room_nr].current_speed = t[2];
  world[room_nr].current_direction = t[3];

  if (!get_line(fl, line) || sscanf(line, " %d %d %d %d %d", t, t + 1, t + 2, t + 3, t + 4) != 5) {
    fprintf(stderr, "SYSERR: Format error in room #%d\n", virtual_nr);
    raise(SIGSEGV);
  }
  world[room_nr].shape = t[0];
/********* temp ************/

  world[room_nr].x_size = t[1];
  world[room_nr].y_size = t[2];
  world[room_nr].z_size = t[3];
  world[room_nr].map_specials = t[4];

  top_of_grid_count += (GET_X_SIZE(room_nr) * GET_Y_SIZE(room_nr));

  world[room_nr].func = NULL;
  world[room_nr].contents = NULL;
  world[room_nr].people = NULL;
  world[room_nr].spell_affects = NULL;
  world[room_nr].map = NULL;
  world[room_nr].light = 0;	/* Zero light sources */

  for (i = 0; i < NUM_OF_DIRS; i++)
    world[room_nr].dir_option[i] = NULL;

  world[room_nr].ex_description = NULL;

  sprintf(buf, "SYSERR: Format error in room #%d zone#%d(expecting D/E/S)", virtual_nr, zone);

  for (;;) {
    if ((letter == '#') || (letter == '$'))
      break;
    if (!get_line(fl, line)) {
      fprintf(stderr, "%s", buf);
      fprintf(stderr, " from missing line\n");
      raise(SIGSEGV);
    }
    switch (*line) {
    case 'D':
      setup_dir(fl, room_nr, atoi(line + 1));
      top_of_door_table++;
      break;
    case 'F':
      if (!get_line(fl, line) || sscanf(line, " %d ", t) != 1) {
	fprintf(stderr, "SYSERR: Format error in room #%d\n", virtual_nr);
	raise(SIGSEGV);
      }
      world[room_nr].chance_fall = t[0];
    case 'M':
      generate_mapgrid(fl, room_nr);
      map_loaded = TRUE;
      break;
    case 'E':
      CREATE(new_descr, struct extra_descr_data, 1);
      new_descr->keyword = fread_string(fl, buf2);
      new_descr->description = fread_string(fl, buf2);
      new_descr->next = world[room_nr].ex_description;
      world[room_nr].ex_description = new_descr;
      break;
    case 'S':			/* end of room */
      /* DG triggers -- script is defined after the end of the room */
      letter = fread_letter(fl);
      ungetc(letter, fl);
      while (letter == 'T') {
	dg_read_trigger(fl, &world[room_nr], WLD_TRIGGER);
	letter = fread_letter(fl);
	ungetc(letter, fl);
      }
      top_of_world = room_nr++;
//      return; 
      break;
    default:
      fprintf(stderr, "%s", buf);
      fprintf(stderr, " from default case\n");
      fprintf(stderr, "Got instead: %s, letter: %c", line, letter);
      raise(SIGSEGV);
      break;
    }
  }
}

void convert_rooms(void)
{
  register int room_nr;

  for (room_nr = 0; room_nr <= top_of_world; room_nr++) {
/* temp to fix old zones */
    if (SECT(room_nr) >= NUM_ROOM_SECTORS)
      SECT(room_nr) = SECT_INSIDE;
    if (world[room_nr].sector_type == SECT_CAVE)
      SET_BIT(ROOM_FLAGS(room_nr), ROOM_INDOORS);
    if (world[room_nr].sector_type == SECT_INSIDE)
      SET_BIT(ROOM_FLAGS(room_nr), ROOM_INDOORS);

/* allows code to draw its own room */
    if (!map_loaded)
      world[room_nr].shape = ROOM_SHAPE_SQUARE;

    if (GET_X_SIZE(room_nr) < 1)
      GET_X_SIZE(room_nr) = 3;
    if (GET_Y_SIZE(room_nr) < 1)
      GET_Y_SIZE(room_nr) = 3;

    if (!ROOM_FLAGGED(room_nr, ROOM_INDOORS))
      world[room_nr].z_size = -1;
    else if (world[room_nr].z_size < 3)
      world[room_nr].z_size = 3;

    if (ROOM_FLAGGED(room_nr, ROOM_BFS_MARK))
      REMOVE_BIT(room_nr, ROOM_BFS_MARK);

    if (IS_SET(ROOM_FLAGS(room_nr), ROOM_MOB_GEN))
      world[room_nr].func = mob_generating_room;
  }
}

void generate_mapgrid(FILE * fl, int room)
{
  int t[GET_Y_SIZE(room) + 1], count, count2, n, x, i;
  char *s;

  if (world[room].map == NULL) {
    CREATE(world[room].map, struct map_data, 1);
#ifdef NEWMAP
    CREATE(world[room].map->map_grid, sh_int *, GET_X_SIZE(room) + 1);
    for (i = 0; i < GET_X_SIZE(room) + 1; i++)
      CREATE(world[room].map->map_grid[i], sh_int, GET_Y_SIZE(room) + 1);
#endif
  }
  /* init map grid for the room */
  for (n = 0; n < GET_X_SIZE(room) + 1; n++) {
    for (x = 0; x < GET_Y_SIZE(room) + 1; x++) {
      world[room].map->map_grid[n][x] = MAP_BLANK;
    }
  }

  sprintf(buf2, "room #%d, case M", world[room].number);
  for (count = 0; count < GET_X_SIZE(room); count++) {
    for (count2 = 0; count2 < GET_Y_SIZE(room); count2++) {
      if (feof(fl)) {
	vlogf("SYSERR: Format error - EOF, %s #1\n", buf2);
	return;
      }
      s = fread_word(fl);
      if (!*s || !is_number(s)) {
	vlogf("SYSERR: Format error, %s, mapline %d, position %d\n",
	      buf2, count, count2);
	return;
      }
      t[count2] = atoi(s);
    }
    for (count2 = 0; count2 < GET_Y_SIZE(room); count2++) {
      if (t[count2] >= MAP_ERROR)
	t[count2] = MAP_NOTBLANK;
      world[room].map->map_grid[count][count2] = t[count2];
    }
  }
}

/* read direction data */
void setup_dir(FILE * fl, int room, int dir)
{
  int t[10];
  char line[256];

  sprintf(buf2, "room #%d, direction D%d", world[room].number, dir);

  CREATE(world[room].dir_option[dir], struct room_direction_data, 1);
  world[room].dir_option[dir]->general_description = fread_string(fl, buf2);
  world[room].dir_option[dir]->keyword = fread_string(fl, buf2);

  if (!get_line(fl, line)) {
    fprintf(stderr, "SYSERR: Format error, %s\n", buf2);
    raise(SIGSEGV);
  }
  if (sscanf(line, " %d %d %d %d %d %d", t, t + 1, t + 2, t + 3, t + 4, t + 5) != 6) {
    fprintf(stderr, "SYSERR: Format error, %s\n", buf2);
    raise(SIGSEGV);
  }
  world[room].dir_option[dir]->exit_info = t[0];
  world[room].dir_option[dir]->key = t[1];
  world[room].dir_option[dir]->to_room = t[2];
  world[room].dir_option[dir]->hitpoints = 25;
  world[room].dir_option[dir]->x_cord = t[3];
  world[room].dir_option[dir]->y_cord = t[4];
  world[room].dir_option[dir]->z_cord = t[5];

  if (IS_SET(world[room].dir_option[dir]->exit_info, EX_STONE) ||
      IS_SET(world[room].dir_option[dir]->exit_info, EX_TWOOD))
    world[room].dir_option[dir]->hitpoints = 50;
  if (IS_SET(world[room].dir_option[dir]->exit_info, EX_METAL) ||
      IS_SET(world[room].dir_option[dir]->exit_info, EX_TSTONE))
    world[room].dir_option[dir]->hitpoints = 100;
  if (IS_SET(world[room].dir_option[dir]->exit_info, EX_TMETAL))
    world[room].dir_option[dir]->hitpoints = 200;
  if (IS_SET(world[room].dir_option[dir]->exit_info, EX_CLOSED))
    SET_BIT(world[room].dir_option[dir]->exit_info, EX_ISDOOR);
  if (IS_SET(world[room].dir_option[dir]->exit_info, EX_NOSEE))
    SET_BIT(world[room].dir_option[dir]->exit_info, EX_ISDOOR);
  if (IS_SET(world[room].dir_option[dir]->exit_info, EX_ISDOOR) &&
      !world[room].dir_option[dir]->keyword)
    world[room].dir_option[dir]->keyword = str_dup("door");

  if (world[room].dir_option[dir]->z_cord < 0)
    world[room].dir_option[dir]->z_cord = 0;
  if (world[room].dir_option[dir]->z_cord > GET_Z_SIZE(room))
    GET_Z_SIZE(room) = world[room].dir_option[dir]->z_cord + 3;

  if ((world[room].dir_option[dir]->x_cord == 0 &&
       world[room].dir_option[dir]->y_cord == 0) ||
      world[room].dir_option[dir]->x_cord > GET_X_SIZE(room) - 1 ||
      world[room].dir_option[dir]->y_cord > GET_Y_SIZE(room) - 1) {
    switch (dir) {
    case NORTH:
      world[room].dir_option[dir]->x_cord = 0;
      world[room].dir_option[dir]->y_cord = GET_Y_SIZE(room) / 2;
      break;
    case EAST:
      world[room].dir_option[dir]->x_cord = GET_X_SIZE(room) / 2;
      world[room].dir_option[dir]->y_cord = GET_Y_SIZE(room) - 1;
      break;
    case SOUTH:
      world[room].dir_option[dir]->x_cord = GET_X_SIZE(room) - 1;
      world[room].dir_option[dir]->y_cord = GET_Y_SIZE(room) / 2;
      break;
    case WEST:
      world[room].dir_option[dir]->x_cord = GET_X_SIZE(room) / 2;
      world[room].dir_option[dir]->y_cord = 0;
      break;
    case UP:
      world[room].dir_option[dir]->x_cord = GET_X_SIZE(room) / 2;
      world[room].dir_option[dir]->y_cord = (GET_Y_SIZE(room) / 2) + 1;
      break;
    case DOWN:
      world[room].dir_option[dir]->x_cord = GET_X_SIZE(room) / 2;
      world[room].dir_option[dir]->y_cord = (GET_Y_SIZE(room) / 2) - 1;
      break;
    }
  }
}


/* make sure the start rooms exist & resolve their vnums to rnums */
void check_start_rooms(void)
{
  extern int mortal_start_room;
  extern int immort_start_room;
  extern int frozen_start_room;

  if ((r_mortal_start_room = real_room(mortal_start_room)) < 0) {
    log("SYSERR:  Mortal start room does not exist.  Change in config.c.");
    raise(SIGSEGV);
  }
  if ((r_immort_start_room = real_room(immort_start_room)) < 0) {
    if (!mini_mud)
      log("SYSERR:  Warning: Immort start room does not exist.  Change in config.c.");
    r_immort_start_room = r_mortal_start_room;
  }
  if ((r_frozen_start_room = real_room(frozen_start_room)) < 0) {
    if (!mini_mud)
      log("SYSERR:  Warning: Frozen start room does not exist.  Change in config.c.");
    r_frozen_start_room = r_mortal_start_room;
  }
}


/* resolve all vnums into rnums in the world */
void renum_world(void)
{
  register int room, door;

  for (room = 0; room <= top_of_world; room++)
    for (door = 0; door < NUM_OF_DIRS; door++)
      if (world[room].dir_option[door])
	if (world[room].dir_option[door]->to_room != NOWHERE)
	  world[room].dir_option[door]->to_room =
	      real_room(world[room].dir_option[door]->to_room);
}


#define ZCMD zone_table[zone].cmd[cmd_no]

/* resulve vnums into rnums in the zone reset tables */
void renum_zone_table(void)
{
  int zone, cmd_no, a, b, c, olda, oldb, oldc;
  char buf[128];

  for (zone = 0; zone <= top_of_zone_table; zone++)
    for (cmd_no = 0; ZCMD.command != 'S'; cmd_no++) {
      a = b = c = 0;
      olda = ZCMD.arg1;
      oldb = ZCMD.arg2;
      oldc = ZCMD.arg3;
      switch (ZCMD.command) {
      case 'M':
      case 'F':
      case 'H':
      case 'C':
	a = ZCMD.arg1 = real_mobile(ZCMD.arg1);
	c = ZCMD.arg3 = real_room(ZCMD.arg3);
	break;
      case 'O':
	a = ZCMD.arg1 = real_object(ZCMD.arg1);
	if (ZCMD.arg3 != NOWHERE)
	  c = ZCMD.arg3 = real_room(ZCMD.arg3);
	break;
      case 'G':
	a = ZCMD.arg1 = real_object(ZCMD.arg1);
	break;
      case 'E':
	a = ZCMD.arg1 = real_object(ZCMD.arg1);
	break;
      case 'P':
	a = ZCMD.arg1 = real_object(ZCMD.arg1);
	c = ZCMD.arg3 = real_object(ZCMD.arg3);
	break;
      case 'D':
	a = ZCMD.arg1 = real_room(ZCMD.arg1);
	break;
      case 'R':		/* rem obj from room */
	a = ZCMD.arg1 = real_room(ZCMD.arg1);
	b = ZCMD.arg2 = real_object(ZCMD.arg2);
	break;
      case 'T':		/* a trigger */
	/* designer's choice: convert this later */
	/* b = ZCMD.arg2 = real_trigger(ZCMD.arg2); */
	b = real_trigger(ZCMD.arg2);	/* leave this in for validation */
	break;
      case 'V':		/* trigger variable assignment */
	if (ZCMD.arg1 == WLD_TRIGGER)
	  b = ZCMD.arg2 = real_room(ZCMD.arg2);
	break;
      }
      if (a < 0 || b < 0 || c < 0) {
	if (!mini_mud) {
	  sprintf(buf, "Invalid vnum %d, cmd disabled",
		  (a < 0) ? olda : ((b < 0) ? oldb : oldc));
	  log_zone_error(zone, cmd_no, buf);
	}
	ZCMD.command = '*';
      }
    }
}


void interp_attack(char *line, int i, int nr)
{
  struct mob_attack_data *new_attack;
  int t[3];

  if ((sscanf(line, " %d %d %d ", t, t + 1, t + 2) != 3)) {
    fprintf(stderr, "SYSERR: Format error in mob #%d, attack line\n"
	    "...expecting line of form '# # #'\n", nr);
    raise(SIGSEGV);
  }
  CREATE(new_attack, struct mob_attack_data, 1);
  new_attack->min_damage = t[0];
  new_attack->max_damage = MAX(2, t[1]);
  new_attack->attack_type = t[2];
  new_attack->next = NULL;

  if (mob_proto[i].mob_specials.mob_attack == NULL) {
    mob_proto[i].mob_specials.mob_attack = new_attack;
  } else {
    new_attack->next = mob_proto[i].mob_specials.mob_attack;
    mob_proto[i].mob_specials.mob_attack = new_attack;
  }

}

void parse_attack_mob(FILE * mob_f, int i, int nr)
{
  char line[256];

  while (get_line(mob_f, line)) {
    if (!strcmp(line, "A")) {	/* end of the attack section */
      return;
    } else if (*line == '#') {	/* we've hit the next mob, maybe? */
      fprintf(stderr, "SYSERR: Unterminated A section in mob #%d\n", nr);
      raise(SIGSEGV);
    } else {
      interp_attack(line, i, nr);
    }
  }
}


void parse_simple_mob(FILE * mob_f, int i, int nr)
{
  int j, t[12], hitpoints;
  char line[256];

  mob_proto[i].real_abils.str = 11 + dice(1, 6);
  mob_proto[i].real_abils.intel = 11 + dice(1, 6);
  mob_proto[i].real_abils.wis = 11 + dice(1, 6);
  mob_proto[i].real_abils.dex = 11 + dice(1, 6);
  mob_proto[i].real_abils.con = 11 + dice(1, 6);
  mob_proto[i].real_abils.cha = 11 + dice(1, 6);

  if (!get_line(mob_f, line) || (sscanf(line, " %d %d %d %dd%d+%d %d %d %d ",
     t, t + 1, t + 2, t + 3, t + 4, t + 5, t + 8, t + 9, t + 10) != 9)) {
    fprintf(stderr, "SYSERR: Format error in mob #%d, first line after S flag\n"
	    "...expecting line of form '# # # #d#+# # # #'\n", nr);
    raise(SIGSEGV);
  }
  GET_LEVEL(mob_proto + i) = t[0];
  mob_proto[i].points.hitroll = t[1];
  mob_proto[i].points.armor = t[2];

  mob_proto[i].mob_specials.hp_die = t[4];
  mob_proto[i].mob_specials.hp_plus = t[5];
  if (t[0] > 0 && t[4] > 0)
    hitpoints = (t[0] * t[4]) + t[5];
  else
    hitpoints = number(2, 8) + t[5];
  mob_proto[i].points.hit = hitpoints;
  mob_proto[i].points.max_hit = mob_proto[i].points.hit;

  mob_proto[i].points.damroll = t[8];

  mob_proto[i].char_specials.magic_resistance = t[9];

  mob_proto[i].mob_specials.thac0 = t[10];

  t[2] = 0;			/* hometown */
  if (!get_line(mob_f, line) ||
      ((sscanf(line, " %d %d %d", t, t + 1, t + 2) != 3) &&
       (sscanf(line, " %d %d", t, t + 1) != 2))) {
    fprintf(stderr, "SYSERR: Format error in mob #%d, second line after S flag\n"
	    "...expecting line of form '# # #'\n", nr);
    raise(SIGSEGV);
  }
  GET_MOB_TREASURE(mob_proto + i) = t[0];
  GET_EXP(mob_proto + i) = t[1];
  GET_HOMETOWN(mob_proto + i) = t[2];

  if (!get_line(mob_f, line) || (sscanf(line, " %d %d %d %d ", t, t + 1, t + 2, t + 3) != 3)) {
    fprintf(stderr, "SYSERR: Format error in last line of mob #%d\n"
	    "...expecting line of form '# # #'\n", nr);
    raise(SIGSEGV);
  }
  mob_proto[i].char_specials.position = t[0] + STAT_NORMAL;
  mob_proto[i].mob_specials.default_pos = t[1];
  mob_proto[i].player.sex = t[2];

  mob_proto[i].player.weight = 200;
  mob_proto[i].player.height = 198;

  for (j = 0; j < 3; j++)
    GET_COND(mob_proto + i, j) = -1;

  /*
   * these are now save applies; base save numbers for MOBs are now from
   * the warrior save table.
   */
  for (j = 0; j < 5; j++)
    GET_SAVE(mob_proto + i, j) = 0;
}


/*
 * interpret_espec is the function that takes espec keywords and values
 * and assigns the correct value to the mob as appropriate.  Adding new
 * e-specs is absurdly easy -- just add a new CASE statement to this
 * function!  No other changes need to be made anywhere in the code.
 */

#define CASE(test) if (!matched && !str_cmp(keyword, test) && (matched = 1))
#define RANGE(low, high) (num_arg = MAX((low), MIN((high), (num_arg))))

void interpret_espec(char *keyword, char *value, int i, int nr)
{
  int num_arg, matched = 0;

  num_arg = atoi(value);

  CASE("BareHandAttack") {
    RANGE(0, 99);
    mob_proto[i].mob_specials.attack_type = num_arg;
  }

  CASE("PrototypeOf") {
    RANGE(1, 5000);
    mob_proto[i].mob_specials.orig_vnum = num_arg;
  }

  CASE("Str") {
    RANGE(3, 25);
    mob_proto[i].real_abils.str = num_arg;
  }

  CASE("StrAdd") {
    RANGE(0, 100);
    mob_proto[i].real_abils.str_add = num_arg;
  }

  CASE("Int") {
    RANGE(3, 25);
    mob_proto[i].real_abils.intel = num_arg;
  }

  CASE("Wis") {
    RANGE(3, 25);
    mob_proto[i].real_abils.wis = num_arg;
  }

  CASE("Dex") {
    RANGE(3, 25);
    mob_proto[i].real_abils.dex = num_arg;
  }

  CASE("Con") {
    RANGE(3, 25);
    mob_proto[i].real_abils.con = num_arg;
  }

  CASE("Cha") {
    RANGE(3, 25);
    mob_proto[i].real_abils.cha = num_arg;
  }

  CASE("Spell") {
    RANGE(1, LAST_SPELL);
    if (mob_proto[i].char_specials.saved.spells[num_arg] < MEMINFO_READY)
      mob_proto[i].char_specials.saved.spells[num_arg] = MEMINFO_READY;
    else
      mob_proto[i].char_specials.saved.spells[num_arg] += 1;
  }

  if (!matched) {
    fprintf(stderr, "SYSERR: Warning: unrecognized espec keyword %s in mob #%d\n",
	    keyword, nr);
  }
}

#undef CASE
#undef RANGE

void parse_espec(char *buf, int i, int nr)
{
  char *ptr;

  if ((ptr = strchr(buf, ':')) != NULL) {
    *(ptr++) = '\0';
    while (isspace(*ptr))
      ptr++;
  } else
    ptr = "";

  interpret_espec(buf, ptr, i, nr);
}


void parse_enhanced_mob(FILE * mob_f, int i, int nr)
{
  char line[256];

  parse_simple_mob(mob_f, i, nr);

  while (get_line(mob_f, line)) {
    if (!strcmp(line, "E"))	/* end of the ehanced section */
      return;
    else if (*line == '#') {	/* we've hit the next mob, maybe? */
      fprintf(stderr, "SYSERR: Unterminated E section in mob #%d\n", nr);
      raise(SIGSEGV);
    } else
      parse_espec(line, i, nr);
  }

  fprintf(stderr, "SYESRR: Unexpected end of file reached after mob #%d\n", nr);
  raise(SIGSEGV);
}


void parse_mobile(FILE * mob_f, int nr)
{
  static int i = 0;
  int j, t[10];
  char line[256], *tmpptr, letter;
  char f1[128], f2[128];
  char f3[128], f4[128];
  char f5[128];

  mob_index[i].virtual = nr;
  mob_index[i].number = 0;
  mob_index[i].func = NULL;
  mob_index[i].paths = NULL;

  clear_char(mob_proto + i);

  mob_proto[i].player_specials = &dummy_mob;
  sprintf(buf2, "mob vnum %d", nr);

/***** Things that need defaults *****/
  mob_proto[i].mob_specials.orig_vnum = nr;

/***** String data *****/
  mob_proto[i].player.name = fread_string(mob_f, buf2);
  tmpptr = mob_proto[i].player.short_descr = fread_string(mob_f, buf2);
  if (tmpptr && *tmpptr)
    if (!str_cmp(fname(tmpptr), "a") || !str_cmp(fname(tmpptr), "an") ||
	!str_cmp(fname(tmpptr), "the"))
      *tmpptr = LOWER(*tmpptr);
  mob_proto[i].player.long_descr = fread_string(mob_f, buf2);
  mob_proto[i].player.description = fread_string(mob_f, buf2);
  mob_proto[i].player.title = NULL;

  /* *** Numeric data *** */
  if (!get_line(mob_f, line) || (sscanf(line, "%s %s %s %s %s %d %d %d %d %c", f1, f2, f3, f4, f5, t + 2, t + 3, t + 4, t + 5, &letter) != 10)) {
    fprintf(stderr, "SYSERR: Format error after string section of mob #%d\n"
	  "...expecting line of form '# # # # # # # # # {S | E}'\n", nr);
    raise(SIGSEGV);
  }
  MOB_FLAGS(mob_proto + i) = asciiflag_conv(f1);
  SET_BIT(MOB_FLAGS(mob_proto + i), MOB_ISNPC);
  AFF_FLAGS(mob_proto + i) = asciiflag_conv(f2);
  AFF2_FLAGS(mob_proto + i) = asciiflag_conv(f3);
  AFF3_FLAGS(mob_proto + i) = asciiflag_conv(f4);
  AFF4_FLAGS(mob_proto + i) = asciiflag_conv(f5);
  SET_ALIGNMENT(mob_proto + i, t[2]);
  GET_SIZE(mob_proto + i) = t[3];
  GET_RACE(mob_proto + i) = t[4];
  GET_CLASS(mob_proto + i) = t[5];

  switch (UPPER(letter)) {
  case 'S':			/* Simple monsters */
    parse_simple_mob(mob_f, i, nr);
    break;
  case 'E':			/* Circle3 Enhanced monsters */
    parse_enhanced_mob(mob_f, i, nr);
    break;
    /* add new mob types here.. */
  default:
    fprintf(stderr, "SYSERR: Unsupported mob type '%c' in mob #%d\n", letter, nr);
    raise(SIGSEGV);
    break;
  }

  parse_attack_mob(mob_f, i, nr);

  /* DG triggers -- script info follows mob S/E section */
  letter = fread_letter(mob_f);
  ungetc(letter, mob_f);
  while (letter == 'T') {
    dg_read_trigger(mob_f, &mob_proto[i], MOB_TRIGGER);
    letter = fread_letter(mob_f);
    ungetc(letter, mob_f);
  }
  letter = fread_letter(mob_f);
  ungetc(letter, mob_f);
  while (letter == 'Q') {
    read_quest(mob_f, i);
    letter = fread_letter(mob_f);
    ungetc(letter, mob_f);
  }
  mob_proto[i].aff_abils = mob_proto[i].real_abils;

  for (j = 0; j < NUM_WEARS; j++)
    mob_proto[i].equipment[j] = NULL;

  mob_proto[i].nr = i;
  mob_proto[i].desc = NULL;

  set_saving_throws(mob_proto + i);

/* add path stuff here? sure.. I could have defined it as an espec
   case, but that wouldn't be any fun. It would be work, and besides
   the espec stuff wasn't really _really_ meant for multi-line multiple
   pieces of data... which this is */

  while ((letter = fread_letter(mob_f)) == '!') {
    ungetc(letter, mob_f);
    load_path_from_mob(mob_f, &mob_index[i]);
  }
  /* no matter what happens, we'll end up grabbing an extra
     letter.  So, lets rewind a bit, and we'll be okay */
  ungetc(letter, mob_f);

  top_of_mobt = i++;
}


/* read all objects from obj file; generate index and prototypes */
char *parse_object(FILE * obj_f, int nr)
{
  SPECIAL(item_switch);
  SPECIAL(portal_object);
  SPECIAL(pit_object);
  SPECIAL(door_object);
  SPECIAL(outdoor_zone_connecter_object);
  SPECIAL(generic_drow_eq);
  static int i = 0, retval;
  static char line[256], line2[256];
  int t[10], j;
  float fx;
  char *tmpptr;
  char f1[256], f2[256], f3[256], f4[256];
  struct extra_descr_data *new_descr;
  char *temp;

  obj_index[i].virtual = nr;
  obj_index[i].number = 0;
  obj_index[i].func = NULL;
  obj_index[i].paths = NULL;

  clear_object(obj_proto + i);
  obj_proto[i].in_room = NOWHERE;
  obj_proto[i].item_number = i;
  obj_proto[i].idnum_owner = -2;
  obj_proto[i].obj_flags.orig_vnum = nr;

  sprintf(buf2, "object #%d", nr);

  /* *** string data *** */
  if ((obj_proto[i].name = fread_string(obj_f, buf2)) == NULL) {
    fprintf(stderr, "SYSERR: Null obj name or format error at or near %s\n", buf2);
    raise(SIGSEGV);
  }
  tmpptr = obj_proto[i].short_description = fread_string(obj_f, buf2);
  if (tmpptr && *tmpptr)
    if (!str_cmp(fname(tmpptr), "a") || !str_cmp(fname(tmpptr), "an") ||
	!str_cmp(fname(tmpptr), "the"))
      *tmpptr = LOWER(*tmpptr);

  tmpptr = obj_proto[i].description = fread_string(obj_f, buf2);
  if (tmpptr && *tmpptr) {
    if (!str_cmp(fname(tmpptr), "Undefined")) {
      sprintf(buf, "%s lies here.", obj_proto[i].short_description);
      tmpptr = obj_proto[i].description = str_dup(buf);
    }
    *tmpptr = UPPER(*tmpptr);
  }
  temp = fread_string(obj_f, buf2);
  obj_proto[i].action_description = temp;


  /* *** numeric data *** */
  if (!get_line(obj_f, line) ||
    (retval = sscanf(line, " %d %s %s %s %s", t, f1, f2, f3, f4)) != 5) {
    fprintf(stderr, "SYSERR: Format error in first numeric line (expecting 5 args, got %d), %s\n", retval, buf2);
    raise(SIGSEGV);
  }
  obj_proto[i].obj_flags.type_flag = t[0];
  obj_proto[i].obj_flags.extra_flags = asciiflag_conv(f1);
  obj_proto[i].obj_flags.wear_flags = asciiflag_conv(f2);
  obj_proto[i].obj_flags.extra2_flags = asciiflag_conv(f3);
  obj_proto[i].obj_flags.anti_flags = asciiflag_conv(f4);

  if (!get_line(obj_f, line) ||
      (retval = sscanf(line, "%d %d %d %d %d %d %d %d", t, t + 1, t + 2, t + 3, t + 4, t + 5, t + 6, t + 7)) != 8) {
    fprintf(stderr, "SYSERR: Format error in second numeric line (expecting 8 args, got %d), %s\n", retval, buf2);
    raise(SIGSEGV);
  }
  obj_proto[i].obj_flags.value[0] = t[0];
  obj_proto[i].obj_flags.value[1] = t[1];
  obj_proto[i].obj_flags.value[2] = t[2];
  obj_proto[i].obj_flags.value[3] = t[3];
  obj_proto[i].obj_flags.value[4] = t[4];
  obj_proto[i].obj_flags.value[5] = t[5];
  obj_proto[i].obj_flags.value[6] = t[6];
  obj_proto[i].obj_flags.value[7] = t[7];

  if (!get_line(obj_f, line) ||
      (retval = sscanf(line, "%d %d %d %d %d %d %d", t, t + 1, t + 2, t + 3, t + 4, t + 5, t + 6)) != 7) {
    fprintf(stderr, "SYSERR: Format error in third numeric line (expecting 7 args, got %d), %s\n", retval, buf2);
    raise(SIGSEGV);
  }
  obj_proto[i].obj_flags.size = t[0];
  obj_proto[i].obj_flags.space = t[1];
  obj_proto[i].obj_flags.condition = t[2];
/********** temp ***********/
  obj_proto[i].obj_flags.condition = 100;

  obj_proto[i].obj_flags.max_cond = obj_proto[i].obj_flags.condition;

  obj_proto[i].obj_flags.status = t[3];
  obj_proto[i].obj_flags.material = t[4];
  obj_proto[i].obj_flags.craftsmanship = t[5];
  obj_proto[i].obj_flags.race = t[6];

  if (!get_line(obj_f, line) ||
      (retval = sscanf(line, "%f %d %d", &fx, t, t + 1)) != 3) {
    fprintf(stderr, "SYSERR: Format error in third numeric line (expecting 3 args, got %d), %s\n", retval, buf2);
    raise(SIGSEGV);
  }
  obj_proto[i].obj_flags.weight = fx;
  obj_proto[i].obj_flags.cost = t[0];
  obj_proto[i].obj_flags.orig_vnum = t[1];

  /* check to make sure that weight of containers exceeds curr. quantity */
  if (obj_proto[i].obj_flags.type_flag == ITEM_DRINKCON ||
      obj_proto[i].obj_flags.type_flag == ITEM_FOUNTAIN) {
    if (obj_proto[i].obj_flags.weight < obj_proto[i].obj_flags.value[1])
      obj_proto[i].obj_flags.weight = obj_proto[i].obj_flags.value[1] + 5;
  }
  /* make sure containers are viable */
  if (obj_proto[i].obj_flags.type_flag == ITEM_CONTAINER ||
      obj_proto[i].obj_flags.type_flag == ITEM_STORAGE) {
    /* key */
    if (obj_proto[i].obj_flags.value[2] == 0)
      obj_proto[i].obj_flags.value[2] = -1;
    /* space */
    if (obj_proto[i].obj_flags.value[4] == 0)
      obj_proto[i].obj_flags.value[4] = obj_proto[i].obj_flags.value[0];
  }
  /* make furniture tough to carry */
  if (obj_proto[i].obj_flags.type_flag == ITEM_FURNITURE) {
    obj_proto[i].obj_flags.extra_flags = obj_proto[i].obj_flags.extra_flags | ITEM_TWOHANDS;
  }
  /* assign procs */
  if (obj_proto[i].obj_flags.type_flag == ITEM_SWITCH)
    obj_index[i].func = item_switch;
  if (obj_proto[i].obj_flags.type_flag == ITEM_TELEPORT)
    obj_index[i].func = portal_object;
  if (obj_proto[i].obj_flags.type_flag == ITEM_PIT)
    obj_index[i].func = pit_object;
  if (obj_proto[i].obj_flags.type_flag == ITEM_DOOR)
    obj_index[i].func = door_object;
  if (obj_proto[i].obj_flags.type_flag == ITEM_MAP_CONNECTOR)
    obj_index[i].func = outdoor_zone_connecter_object;
  if (isname("drow_decay", obj_proto[i].name))
    obj_index[i].func = generic_drow_eq;

  /* fix minor errors */
  if (obj_proto[i].obj_flags.orig_vnum >= 2000 && obj_proto[i].obj_flags.orig_vnum <= 4000)
    obj_proto[i].obj_flags.extra_flags = obj_proto[i].obj_flags.extra_flags | ITEM_MAGIC;

  /* *** extra descriptions and affect fields *** */

  for (j = 0; j < MAX_OBJ_AFFECT; j++) {
    obj_proto[i].affected[j].location = APPLY_NONE;
    obj_proto[i].affected[j].modifier = 0;
  }
  /* clean data */
  obj_proto[i].obj_flags.trap_eff = 0;
  obj_proto[i].obj_flags.trap_dam = 0;
  obj_proto[i].obj_flags.trap_charge = 0;
  obj_proto[i].obj_flags.trap_level = 0;

  strcat(buf2, ", after numeric constants\n...expecting 'E', 'A', '$', or next object number");
  j = 0;

  for (;;) {
    if (!get_line(obj_f, line)) {
      fprintf(stderr, "SYSERR: Format error in %s\n", buf2);
      raise(SIGSEGV);
    }
    switch (*line) {
    case 'E':
      CREATE(new_descr, struct extra_descr_data, 1);
      new_descr->keyword = fread_string(obj_f, buf2);
      new_descr->description = fread_string(obj_f, buf2);
      new_descr->next = obj_proto[i].ex_description;
      obj_proto[i].ex_description = new_descr;
      break;
    case 'A':
      if (j >= MAX_OBJ_AFFECT) {
	fprintf(stderr, "SYSERR: Too many A fields (%d max), %s\n", MAX_OBJ_AFFECT, buf2);
	raise(SIGSEGV);
      }
      if (!get_line(obj_f, line) || (sscanf(line, " %d %d ", t, t + 1) != 2)) {
	fprintf(stderr, "SYSERR: Format error in 'A' field, %s\n", buf2);
	fprintf(stderr, "...offending line: '%s'\n", line);
      }
      obj_proto[i].affected[j].location = t[0];
      obj_proto[i].affected[j].modifier = t[1];
      j++;
      break;
    case 'B':
      get_line(obj_f, line2);
      sscanf(line2, "%d ", t);
      switch (atoi(line + 1)) {
      case 1:
	obj_proto[i].obj_flags.bitvector = obj_proto[i].obj_flags.bitvector | t[0];
	break;
      case 2:
	obj_proto[i].obj_flags.bitvector2 = obj_proto[i].obj_flags.bitvector2 | t[0];
	break;
      case 3:
	obj_proto[i].obj_flags.bitvector3 = obj_proto[i].obj_flags.bitvector3 | t[0];
	break;
      case 4:
	obj_proto[i].obj_flags.bitvector4 = obj_proto[i].obj_flags.bitvector4 | t[0];
	break;
      default:
	log("Hmm, unknown value for B in obj file");
	break;
      }
      break;
      /* The appended numbers below are NOT redundant, they ensure the
         letters weren't an error */
    case 'P':
      get_line(obj_f, line2);
      sscanf(line2, "%d ", t);
      switch (atoi(line + 1)) {
      case 0:
	obj_proto[i].obj_flags.protection = obj_proto[i].obj_flags.protection | t[0];
	break;
      default:
	log("Hmm, unknown value for P in obj file");
	break;
      }
      break;
    case 'D':
      get_line(obj_f, line2);
      sscanf(line2, "%d ", t);
      switch (atoi(line + 1)) {
      case 0:
	obj_proto[i].obj_flags.detecting = obj_proto[i].obj_flags.detecting | t[0];
	break;
      default:
	log("Hmm, unknown value for D in obj file");
	break;
      }
    case 'C':
      get_line(obj_f, line2);
      sscanf(line2, "%d ", t);
      switch (atoi(line + 1)) {
      case 0:
	obj_proto[i].obj_flags.controlling = obj_proto[i].obj_flags.controlling | t[0];
	break;
      default:
	log("Hmm, unknown value for C in obj file");
	break;
      }
      break;
    case 'T':			/* DG triggers */
      dg_obj_trigger(line, &obj_proto[i]);
      break;
    case 'X':			/* trap */
      if (!get_line(obj_f, line) ||
	  (sscanf(line, " %d %d %d %d %d", t, t + 1, t + 2, t + 3, t + 4) != 5)) {
	fprintf(stderr, "SYSERR: Format error in 'X' field, %s\n", buf2);
	fprintf(stderr, "...offending line: '%s'\n", line);
      }
      obj_proto[i].obj_flags.trap_eff = t[0];
      obj_proto[i].obj_flags.trap_dam = t[1];
      obj_proto[i].obj_flags.trap_charge = t[2];
      obj_proto[i].obj_flags.trap_level = t[3];
/*   obj_proto[i].obj_flags.trap_  = t[4];  future needs */
      break;
    case '$':
    case '#':
      top_of_objt = i++;
      return line;
      break;
    default:
      fprintf(stderr, "SYSERR: Format error in %s\n", buf2);
      raise(SIGSEGV);
      break;
    }
  }
}


#define Z	zone_table[zone]

/* load the zone table and command tables */
void load_zones(FILE * fl, char *zonename)
{
  static int zone = 0;
  int cmd_no = 0, num_of_cmds = 0, line_num = 0, tmp, error;
  char *ptr, buf[256], zname[256];
  char t1[80], t2[80];

  strcpy(zname, zonename);

  while (get_line(fl, buf))
    num_of_cmds++;		/* this should be correct within 3 or so */
  rewind(fl);

  if (num_of_cmds == 0) {
    fprintf(stderr, "SYSERR: %s is empty!\n", zname);
    raise(SIGSEGV);
  } else
    CREATE(Z.cmd, struct reset_com, num_of_cmds);

  line_num += get_line(fl, buf);

  if (sscanf(buf, "#%d", &Z.number) != 1) {
    fprintf(stderr, "SYSERR: Format error in %s, line %d\n", zname, line_num);
    raise(SIGSEGV);
  }
  sprintf(buf2, "beginning of zone #%d", Z.number);

  line_num += get_line(fl, buf);
  if ((ptr = strchr(buf, '~')) != NULL)		/* take off the '~' if it's there */
    *ptr = '\0';
  Z.name = str_dup(buf);
  Z.closed = FALSE;
  Z.kingdom = 0;

  line_num += get_line(fl, buf);

  if (sscanf(buf, " %d %d %d %d %d", &Z.top, &Z.lifespan, &Z.reset_mode, &Z.closed, &Z.kingdom) != 5) {
    if (sscanf(buf, " %d %d %d %d", &Z.top, &Z.lifespan, &Z.reset_mode, &Z.closed) != 4) {
      if (sscanf(buf, " %d %d %d ", &Z.top, &Z.lifespan, &Z.reset_mode) != 3) {
	fprintf(stderr, "SYSERR: Format error in 3/4/5-constant line of %s", zname);
	raise(SIGSEGV);
      }
    }
  }
  Z.pressure = 960;
  if ((time_info.month >= 7) && (time_info.month <= 12))
    Z.pressure += dice(1, 50);
  else
    Z.pressure += dice(1, 80);
  Z.change = 0;
  if (Z.pressure <= 980)
    Z.sky = SKY_LIGHTNING;
  else if (Z.pressure <= 1000)
    Z.sky = SKY_RAINING;
  else if (Z.pressure <= 1020)
    Z.sky = SKY_CLOUDY;
  else
    Z.sky = SKY_CLOUDLESS;

  Z.func = NULL;

  cmd_no = 0;

  for (;;) {
    if ((tmp = get_line(fl, buf)) == 0) {
      fprintf(stderr, "Format error in %s - premature end of file\n", zname);
      raise(SIGSEGV);
    }
    line_num += tmp;
    ptr = buf;
    skip_spaces(&ptr);

    if ((ZCMD.command = *ptr) == '*')
      continue;

    ptr++;

    if (ZCMD.command == 'S' || ZCMD.command == '$') {
      ZCMD.command = 'S';
      break;
    }
    error = 0;
    ZCMD.arg7 = 100;

    if (strchr("MOEGPDCHFT", ZCMD.command) != NULL) {	/* a 7-arg command */
      if (sscanf(ptr, " %d %d %d %d %d %d %d %d", &tmp, &ZCMD.arg1, &ZCMD.arg2, &ZCMD.arg3, &ZCMD.arg4, &ZCMD.arg5, &ZCMD.arg6, &ZCMD.arg7) != 8)
	if (sscanf(ptr, " %d %d %d %d %d %d %d ", &tmp, &ZCMD.arg1, &ZCMD.arg2, &ZCMD.arg3, &ZCMD.arg4, &ZCMD.arg5, &ZCMD.arg6) != 7)
	  error = 1;
    } else if (strchr("R", ZCMD.command) != NULL) {	/* a 3-arg command */
      if (sscanf(ptr, " %d %d %d %d ", &tmp, &ZCMD.arg1, &ZCMD.arg2, &ZCMD.arg7) != 4)
	if (sscanf(ptr, " %d %d %d ", &tmp, &ZCMD.arg1, &ZCMD.arg2) != 3)
	  error = 1;
    } else if (ZCMD.command == 'V') {	/* a string-arg command */
      if (sscanf(ptr, " %d %d %d %d %s %s", &tmp, &ZCMD.arg1, &ZCMD.arg2,
		 &ZCMD.arg3, t1, t2) != 6)
	error = 1;
      else {
	ZCMD.sarg1 = str_dup(t1);
	ZCMD.sarg2 = str_dup(t2);
      }
    } else {
      log("SYSERR: load_zone bypassed normal loads, and fell into default.");
      if (sscanf(ptr, " %d %d %d %d %d ", &tmp, &ZCMD.arg1, &ZCMD.arg2, &ZCMD.arg3, &ZCMD.arg7) != 5)
	if (sscanf(ptr, " %d %d %d %d ", &tmp, &ZCMD.arg1, &ZCMD.arg2, &ZCMD.arg3) != 4)
	  error = 1;
      vlogf("-     %d %d %d", ZCMD.arg1, ZCMD.arg2, ZCMD.arg3);
    }

    ZCMD.if_flag = tmp;
/*
   if (ZCMD.arg7 < 1)
   ZCMD.arg7 = 100;
 */
    if (error) {
      fprintf(stderr, "SYSERR: Format error in %s, line %d: '%s'\n", zname, line_num, buf);
      raise(SIGSEGV);
    }
    ZCMD.line = line_num;
    cmd_no++;
  }

  top_of_zone_table = zone++;
}

#undef Z


void get_one_line(FILE * fl, char *buf)
{
  if (fgets(buf, READ_SIZE, fl) == NULL) {
    log("SYSERR: error reading help file: not terminated with $?");
    raise(SIGSEGV);
  }
  buf[strlen(buf) - 1] = '\0';	/* take off the trailing \n */
}


void load_help(FILE * fl, int saving)
{
  char key[READ_SIZE + 1], next_key[READ_SIZE + 1], entry[32384];
  char line[READ_SIZE + 1], *scan;
  struct help_index_element el;
  static int num_total = 0;
  static int num_uni = 0;

  /* get the first keyword line */
  get_one_line(fl, key);
  while (*key != '$') {
    /* read in the corresponding help entry */
    strcpy(entry, strcat(key, "\r\n"));
    get_one_line(fl, line);
    while (*line != '#') {
      strcat(entry, strcat(line, "\r\n"));
      get_one_line(fl, line);
    }

    el.min_level = 0;
    if ((*line == '#') && (*(line + 1) != 0))
      el.min_level = atoi((line + 1));

    el.min_level = MAX(0, MIN(el.min_level, LVL_IMPL));

    /* now, add the entry to the index with each keyword on the keyword line */
    el.duplicate = 0;
    el.save = saving;
    el.entry = str_dup(entry);
    num_uni++;
    scan = one_word(key, next_key);
    while (*next_key) {
      el.keyword = str_dup(next_key);
      help_table[top_of_helpt++] = el;
      el.duplicate++;
      num_total++;
      scan = one_word(scan, next_key);
    }

    /* get next keyword line (or $) */
    get_one_line(fl, key);
  }
#if 1
/*
   vlogf("Number help entries : %d", num_uni);
   vlogf("Number help keywords: %d", num_total);
   vlogf("Ratio : %f", (float) num_total / (float) num_uni);
 */
  if (((float) num_total / (float) num_uni) > 2) {
    log("SYSERR: The help table is too small. Tell the coder to increase the buffer limit!!!!!");
    vlogf("SYSERR: entries: %d keywords: %d ratio: %f", num_uni, num_total,
	  (float) num_total / (float) num_uni);

  }
#endif
}


int hsort(const void *a, const void *b)
{
  struct help_index_element *a1, *b1;

  a1 = (struct help_index_element *) a;
  b1 = (struct help_index_element *) b;

  return (str_cmp(a1->keyword, b1->keyword));
}


/*************************************************************************
*  procedures for resetting, both play-time and boot-time	 	 *
*************************************************************************/



int vnum_mobile(char *searchname, struct char_data *ch)
{
  int nr, found = 0;
  char pattern[MAX_INPUT_LENGTH], mobile_name[MAX_INPUT_LENGTH];

  strcpy(pattern, searchname);
  strToLower(pattern);

  strcpy(buf, "&+C__________Mobiles__________&n\r\n");
  for (nr = 0; nr <= top_of_mobt; nr++) {

    strcpy(mobile_name, strip_color(mob_proto[nr].player.short_descr));
    strToLower(mobile_name);

    if (strstr(mobile_name, pattern) != NULL) {
      if (strlen(buf) > MAX_STRING_LENGTH - 50) {
	sprintf(buf, "%s\r\n...and the list goes on...\r\n", buf);
	break;
      } else {
	sprintf(buf, "%s%3d. &+B[%5d] &n%s\r\n", buf, ++found,
		mob_index[nr].virtual,
		mob_proto[nr].player.short_descr);
      }
    }
  }
  page_string(ch->desc, buf, 1);

  return (found);
}



int vnum_object(char *searchname, struct char_data *ch)
{
  int nr, found = 0;
  char pattern[MAX_INPUT_LENGTH], object_name[MAX_INPUT_LENGTH];

  strcpy(pattern, searchname);
  strToLower(pattern);

  strcpy(buf, "&+C__________Objects__________&n\r\n");
  for (nr = 0; nr <= top_of_objt; nr++) {

    strcpy(object_name, strip_color(obj_proto[nr].short_description));
    strToLower(object_name);

    if (strstr(object_name, pattern) != NULL) {
      if (strlen(buf) > MAX_STRING_LENGTH - 50) {
	sprintf(buf, "%s\r\n...and the list goes on...\r\n", buf);
	break;
      } else {
	sprintf(buf, "%s%3d. &+B[%5d] &n%s\r\n", buf, ++found,
		obj_index[nr].virtual,
		obj_proto[nr].short_description);
      }
    }
  }
  page_string(ch->desc, buf, 1);

  return (found);
}

int vnum_room(char *searchname, struct char_data *ch)
{
  int nr, found = 0;
  char pattern[MAX_INPUT_LENGTH], room_name[MAX_INPUT_LENGTH];

  strcpy(pattern, searchname);
  strToLower(pattern);

  strcpy(buf, "&+C__________Rooms__________&n\r\n");

  for (nr = 0; nr <= top_of_world; nr++) {
    strcpy(room_name, strip_color(world[nr].name));
    strToLower(room_name);

    if (strstr(room_name, pattern) != NULL) {
      if (strlen(buf) > MAX_STRING_LENGTH - 50) {
	sprintf(buf, "%s\r\n...and the list goes on...\r\n", buf);
	break;
      } else {
	sprintf(buf, "%s%3d. &+B[%5d]&n %s\r\n", buf, ++found,
		world[nr].number, world[nr].name);
      }
    }
  }
  page_string(ch->desc, buf, 1);

  return (found);
}
/* create a character, and add it to the char list */
struct char_data *create_char(void)
{
  struct char_data *ch;

  CREATE(ch, struct char_data, 1);
  clear_char(ch);
  ch->next = character_list;
  character_list = ch;

  GET_ID(ch) = max_id++;
  return ch;
}


/* create a new mobile from a prototype */
struct char_data *read_mobile(int nr, int type)
{
  int i, temp;
  struct char_data *mob;

  if (type == VIRTUAL) {
    if ((i = real_mobile(nr)) < 0) {
      sprintf(buf, "Mobile (V) %d does not exist in database.", nr);
      log(buf);
      return NULL;
    }
  } else
    i = nr;

  CREATE(mob, struct char_data, 1);
  top_of_loaded_mobs_table++;
  clear_char(mob);
  *mob = mob_proto[i];
  mob->next = character_list;
  character_list = mob;

  mob->points.max_hit = dice(GET_LEVEL(mob), 4) + GET_LEVEL(mob) * (4 + GET_HPPLUS(mob));
  temp = (GET_LEVEL(mob) * GET_HPDIE(mob)) + GET_HPPLUS(mob);

  if (GET_LEVEL(mob) > 20) {
    mob->points.max_hit = MAX(mob->points.max_hit, (int) (temp / 1.5));
  } else if (GET_LEVEL(mob) > 10) {
    mob->points.max_hit = MAX(mob->points.max_hit, (temp / 2));
  } else if (GET_LEVEL(mob) > 5) {
    mob->points.max_hit = MAX(mob->points.max_hit, (int) (temp / 2.5));
  } else {
    mob->points.max_hit = MAX(mob->points.max_hit, (temp / 3));
  }

  mob->points.max_hit = BOUNDED(3, mob->points.max_hit, 1000);
  mob->points.hit = mob->points.max_hit;

  /* Set body part hp */
  GET_HIT_WA(mob) = GET_HIT(mob) / 2;
  GET_HIT_WH(mob) = GET_HIT(mob) / 4;
  GET_HIT_SA(mob) = GET_HIT(mob) / 2;
  GET_HIT_SH(mob) = GET_HIT(mob) / 4;
  GET_HIT_TO(mob) = GET_HIT(mob);
  GET_HIT_LL(mob) = GET_HIT(mob) / 2;
  GET_HIT_LF(mob) = GET_HIT(mob) / 4;
  GET_HIT_RL(mob) = GET_HIT(mob) / 2;
  GET_HIT_RF(mob) = GET_HIT(mob) / 4;
  GET_HIT_HD(mob) = GET_HIT(mob) / 4;

  /* randomize where they start walking */
  mob->mob_specials.last_direction = number(0, 9);

  mob->player.time.birth = time(0);
  mob->player.time.played = 0;
  mob->player.time.logon = time(0);
  GET_ID(mob) = max_id++;
  assign_triggers(mob, MOB_TRIGGER);

  GET_X_CORD(mob) = GET_Y_CORD(mob) = GET_Z_CORD(mob) = 0;

  mob_index[i].number++;
  init_language(mob);
  /* battle defaults */
  GET_AIMING(mob) = LOCATION_TORSO;
  GET_FORM(mob) = FORM_DEFAULT;
  GET_FACING(mob) = number(0, NUM_OF_DIRS);

  MORPH_MOB(mob) = NULL;
  CHAR_POLYMORPH_OBJ(mob) = NULL;
  GET_MOB_MOVE(mob) = NULL;

  convert_mob_if_needed(i);
  randomize_mob(mob);
  GiveSpellsToMob(mob);

  return mob;
}


/* create an object, and add it to the object list */
struct obj_data *create_obj(void)
{
  struct obj_data *obj;

  CREATE(obj, struct obj_data, 1);
  clear_object(obj);
  obj->next = object_list;
  object_list = obj;

  GET_ID(obj) = max_id++;
  assign_triggers(obj, OBJ_TRIGGER);

  return obj;
}


/* create a new object from a prototype */
struct obj_data *read_object(int nr, int type)
{
  struct obj_data *obj;
  int i;

  if (nr < 0) {
    log("SYSERR: trying to create obj with negative num!");
    return NULL;
  }
  if (type == VIRTUAL) {
    if ((i = real_object(nr)) < 0) {
      sprintf(buf, "Object (V) %d does not exist in database.", nr);
      log(buf);
      return NULL;
    }
  } else
    i = nr;

  CREATE(obj, struct obj_data, 1);
  clear_object(obj);
  *obj = obj_proto[i];
  obj->next = object_list;
  object_list = obj;

  obj_index[i].number++;
  GET_ID(obj) = max_id++;
  assign_triggers(obj, OBJ_TRIGGER);
  MORPH_OBJ(obj) = NULL;

  /* If its a random item, let's fill in the details */
  if (GET_OBJ_TYPE(obj) == ITEM_TREASURE)
    switch (GET_OBJ_VAL(obj, 0)) {
    case 1:
      random_gems(obj, 0);
      break;
    case 2:
      random_jewelry(obj, 0);
      break;
    }
  return obj;
}



#define ZO_DEAD  999

/* update zone ages, queue for reset if necessary, and dequeue when possible */
void zone_update(void)
{
  int i;
  struct reset_q_element *update_u, *temp;
  static int timer = 0;
  char buf[128];

  /* jelson 10/22/92 */
  if (((++timer * PULSE_ZONE) / PASSES_PER_SEC) >= 60) {
    /* one minute has passed */
    /*
     * NOT accurate unless PULSE_ZONE is a multiple of PASSES_PER_SEC or a
     * factor of 60
     */

    timer = 0;

    /* since one minute has passed, increment zone ages */
    /* skip the first 50 god zones */
    for (i = 49; i <= top_of_zone_table; i++) {
      if (zone_table[i].age < zone_table[i].lifespan &&
	  zone_table[i].reset_mode && !zone_table[i].unloaded && !zone_table[i].closed)
	(zone_table[i].age)++;
      if (zone_table[i].age >= zone_table[i].lifespan &&
	  zone_table[i].age < ZO_DEAD && zone_table[i].reset_mode) {
	/* enqueue zone */

	CREATE(update_u, struct reset_q_element, 1);

	update_u->zone_to_reset = i;
	update_u->next = 0;

	if (!reset_q.head)
	  reset_q.head = reset_q.tail = update_u;
	else {
	  reset_q.tail->next = update_u;
	  reset_q.tail = update_u;
	}

	zone_table[i].age = ZO_DEAD;
      }
    }
  }				/* end - one minute has passed */
  /* dequeue zones (if possible) and reset */
  /* this code is executed every 10 seconds (i.e. PULSE_ZONE) */
  for (update_u = reset_q.head; update_u; update_u = update_u->next) {
    if (zone_table[update_u->zone_to_reset].reset_mode == 2 ||
	is_empty(update_u->zone_to_reset)) {
      reset_zone(update_u->zone_to_reset);
      sprintf(buf, "Auto zone reset: %s",
	      zone_table[update_u->zone_to_reset].name);
      mudlog(buf, CMP, LVL_GOD, FALSE);
    } else
      break;

    /* dequeue */
    if (update_u == reset_q.head)
      reset_q.head = reset_q.head->next;
    else {
      for (temp = reset_q.head; temp->next != update_u; temp = temp->next);
      if (!update_u->next)
	reset_q.tail = temp;
      temp->next = update_u->next;
    }
    FREE(update_u);
    break;
  }

  check_path();
}

void log_zone_error(int zone, int cmd_no, char *message)
{
  char buf[256];

  sprintf(buf, "SYSERR: zone file %d: %s", zone, message);
  mudlog(buf, BRF, LVL_GOD, TRUE);

  sprintf(buf, "SYSERR: ...offending cmd: '%c' cmd in zone #%d, line %d",
	  ZCMD.command, zone_table[zone].number, ZCMD.line);
  mudlog(buf, NRM, LVL_GOD, TRUE);
}

#define ZONE_ERROR(message) \
	{ log_zone_error(zone, cmd_no, message); last_cmd = 0; }

/* execute the reset command table of a given zone */
void reset_zone(int zone)
{
  int cmd_no, last_cmd = 0, back, last_mob_load = 0, done;
  int mob_load = FALSE;
  int obj_load = FALSE;
  struct char_data *mob = NULL, *tmp_mob = NULL, *last_mob = NULL;
  struct char_data *tmob = NULL, *last_mob_followable = NULL, *ch, *next_ch;
  struct obj_data *obj, *obj_to, *tmp_obj, *tobj = NULL;
  int room_vnum, room_rnum;

  if (is_empty(zone))
    unload_room_maps(zone);
#if 1
  /* this block does things to all mobs within the zone.
   * for the moment, I only have it resetting their spells.
   */
  for (ch = character_list; ch; ch = next_ch) {
    next_ch = ch->next;

    if (!IS_MOB(ch))
      continue;
    if (!IS_NPC(ch))
      continue;
    if (MORPH_MOB(ch))		/* how annoying to auto do shit */
      continue;
    if (zone_table[world[ch->in_room].zone].closed)
      continue;
    if (world[ch->in_room].zone == zone) {
      GiveSpellsToMob(ch);
    }
  }
#endif
  for (cmd_no = 0; ZCMD.command != 'S'; cmd_no++) {

    if (ZCMD.if_flag && !last_cmd && !mob_load && !obj_load)
      continue;

    if (!ZCMD.if_flag) {	/* ### */
      mob_load = FALSE;
      obj_load = FALSE;
    }
    switch (ZCMD.command) {
    case '*':			/* ignore command */
      last_cmd = 0;
      break;

    case 'M':			/* read a mobile */
      if ((mob_index[ZCMD.arg1].number < ZCMD.arg2) && number(1, 100) <= ZCMD.arg7) {
	mob = read_mobile(ZCMD.arg1, REAL);
	put_char_to_room(mob, ZCMD.arg3, -1, ZCMD.arg4, ZCMD.arg5, ZCMD.arg6);
	GET_BIRTHPLACE(mob) = world[ZCMD.arg3].number;
	load_mtrigger(mob);
	if (!no_specials)
	  special(mob, 0, "", "", PROC_NORMAL);
	tmp_mob = mob;
	last_mob = last_mob_followable = mob;
	last_cmd = last_mob_load = 1;
	mob_load = TRUE;
      } else {
	last_cmd = last_mob_load = 0;
	last_mob_followable = 0;
      }
      tobj = NULL;
      break;

    case 'O':			/* read an object */
      if ((obj_index[ZCMD.arg1].number < ZCMD.arg2) && number(1, 100) <= ZCMD.arg7) {
	if (ZCMD.arg3 >= 0) {
	  obj = read_object(ZCMD.arg1, REAL);
	  put_obj_to_room(obj, ZCMD.arg3, ZCMD.arg4, ZCMD.arg5, ZCMD.arg6);
	  load_otrigger(obj);
	  tobj = obj;
	  last_cmd = 1;
	  obj_load = TRUE;
	} else {
	  obj = read_object(ZCMD.arg1, REAL);
	  obj->in_room = NOWHERE;
	  last_cmd = 1;
	  tobj = obj;
	  obj_load = TRUE;
	}
      } else
	last_cmd = 0;
      tmob = NULL;
      break;

    case 'P':			/* object to object */
      if ((obj_index[ZCMD.arg1].number < ZCMD.arg2) && number(1, 100) <= ZCMD.arg7) {
	obj = read_object(ZCMD.arg1, REAL);
	if (!(obj_to = get_obj_num(ZCMD.arg3))) {
	  ZONE_ERROR("target obj not found");
	  break;
	}
	obj_to_obj(obj, obj_to);
	load_otrigger(obj);
	last_cmd = 1;
	tobj = obj;
	obj_load = TRUE;
      } else
	last_cmd = 0;
      tmob = NULL;
      break;

    case 'G':			/* obj_to_char */
      if (!mob) {
	if (mob_load == TRUE) {
	  sprintf(buf, "attempt to give obj to a NULL loaded mob");
	  ZONE_ERROR(buf);
	}
	break;
      }
      if ((obj_index[ZCMD.arg1].number < ZCMD.arg2) && number(1, 100) <= ZCMD.arg7) {
	obj = read_object(ZCMD.arg1, REAL);
	obj_to_char(obj, mob);
	load_otrigger(obj);
	tobj = obj;
	last_cmd = 1;
	obj_load = TRUE;
      } else
	last_cmd = 0;
      tmob = NULL;
      break;

    case 'E':			/* object to equipment list */
      if (!mob) {
	ZONE_ERROR("trying to equip non-existant mob");
	break;
      }
      if ((obj_index[ZCMD.arg1].number < ZCMD.arg2) && number(1, 100) <= ZCMD.arg7) {
	if (ZCMD.arg3 < 0 || ZCMD.arg3 >= NUM_WEARS) {
	  ZONE_ERROR("invalid equipment pos number");
	} else {
	  obj = read_object(ZCMD.arg1, REAL);
	  load_otrigger(obj);
	  if (wear_otrigger(obj, mob, ZCMD.arg3))
	    equip_char(mob, obj, ZCMD.arg3);
	  else
	    obj_to_char(obj, mob);
	  tobj = obj;
	  last_cmd = 1;
	  obj_load = TRUE;
	}
      } else
	last_cmd = 0;
      tmob = NULL;
      break;

    case 'R':			/* rem obj from room */
      if ((obj = get_obj_in_list_num(ZCMD.arg2, world[ZCMD.arg1].contents)) != NULL) {
	obj_from_room(obj);
	extract_obj(obj);
      }
      last_cmd = 1;
      tmob = NULL;
      tobj = NULL;
      break;


    case 'D':			/* set state of door */
      if (ZCMD.arg2 < 0 || ZCMD.arg2 >= NUM_OF_DIRS ||
	  (world[ZCMD.arg1].dir_option[ZCMD.arg2] == NULL)) {
	ZONE_ERROR("door does not exist");
      } else {
	if (world[ZCMD.arg1].dir_option[ZCMD.arg2]->to_room &&
	    world[world[ZCMD.arg1].dir_option[ZCMD.arg2]->to_room].dir_option[rev_dir[ZCMD.arg2]] &&
	    world[world[ZCMD.arg1].dir_option[ZCMD.arg2]->to_room].dir_option[rev_dir[ZCMD.arg2]]->to_room ==
	    ZCMD.arg1)
	  back = world[ZCMD.arg1].dir_option[ZCMD.arg2]->to_room;
	else
	  back = 0;
	switch (ZCMD.arg3) {
	case 0:
	  REMOVE_BIT(world[ZCMD.arg1].dir_option[ZCMD.arg2]->exit_info,
		     EX_LOCKED);
	  REMOVE_BIT(world[ZCMD.arg1].dir_option[ZCMD.arg2]->exit_info,
		     EX_CLOSED);
	  if (back) {
	    REMOVE_BIT(world[back].dir_option[rev_dir[ZCMD.arg2]]->exit_info,
		       EX_LOCKED);
	    REMOVE_BIT(world[back].dir_option[rev_dir[ZCMD.arg2]]->exit_info,
		       EX_CLOSED);
	  }
	  break;
	case 1:
	  SET_BIT(world[ZCMD.arg1].dir_option[ZCMD.arg2]->exit_info,
		  EX_CLOSED);
	  REMOVE_BIT(world[ZCMD.arg1].dir_option[ZCMD.arg2]->exit_info,
		     EX_LOCKED);
	  if (back) {
	    REMOVE_BIT(world[back].dir_option[rev_dir[ZCMD.arg2]]->exit_info,
		       EX_LOCKED);
	    SET_BIT(world[back].dir_option[rev_dir[ZCMD.arg2]]->exit_info,
		    EX_CLOSED);
	  }
	  break;
	case 2:
	  SET_BIT(world[ZCMD.arg1].dir_option[ZCMD.arg2]->exit_info,
		  EX_LOCKED);
	  SET_BIT(world[ZCMD.arg1].dir_option[ZCMD.arg2]->exit_info,
		  EX_CLOSED);
	  if (back) {
	    SET_BIT(world[back].dir_option[rev_dir[ZCMD.arg2]]->exit_info,
		    EX_LOCKED);
	    SET_BIT(world[back].dir_option[rev_dir[ZCMD.arg2]]->exit_info,
		    EX_CLOSED);
	  }
	  break;
	}
      }
      last_cmd = 1;
      tmob = NULL;
      tobj = NULL;
      break;
    case 'F':			/* follow last mob M loaded */
      if ((mob_index[ZCMD.arg1].number < ZCMD.arg2) && number(1, 100) <= ZCMD.arg7) {
	mob = read_mobile(ZCMD.arg1, REAL);
	put_char_to_room(mob, ZCMD.arg3, -1, ZCMD.arg4, ZCMD.arg5, ZCMD.arg6);
	GET_BIRTHPLACE(mob) = world[ZCMD.arg3].number;
	add_follower(mob, last_mob_followable);
	SET_BIT(MOB_FLAGS(mob), MOB_SENTINEL);
	strcpy(buf, "group all");
	command_interpreter(last_mob, buf);
	load_mtrigger(mob);
	if (!no_specials)
	  special(mob, 0, "", "", PROC_NORMAL);
	tmp_mob = mob;
	last_mob = last_mob_followable = mob;
	last_cmd = last_mob_load = 1;
	mob_load = TRUE;
      } else {
	last_cmd = last_mob_load = 0;
	last_mob_followable = 0;
      }
      tobj = NULL;
      break;

    case 'H':			/* last mob loaded with M/F command will mount this */
      if ((mob_index[ZCMD.arg1].number < ZCMD.arg2) && number(1, 100) <= ZCMD.arg7) {
	mob = read_mobile(ZCMD.arg1, REAL);
	put_char_to_room(mob, ZCMD.arg3, -1, ZCMD.arg4, ZCMD.arg5, ZCMD.arg6);
	GET_BIRTHPLACE(mob) = world[ZCMD.arg3].number;
	load_mtrigger(mob);
	if (!no_specials)
	  special(mob, 0, "", "", PROC_NORMAL);
	tmp_mob = mob;
	sprintf(buf, "mount %s", FirstWord(GET_NAME(mob)));
	if (tmp_mob) {
	  do_mount(tmp_mob, buf, 0, 0);
	  add_follower(mob, tmp_mob);
	} else {
	  do_mount(last_mob, buf, 0, 0);
	  add_follower(mob, last_mob);
	}
	if (!IS_SET(MOB_FLAGS(mob), MOB_SENTINEL))
	  SET_BIT(MOB_FLAGS(mob), MOB_SENTINEL);
	last_mob = last_mob_followable = mob;
	last_cmd = last_mob_load = 1;
	mob_load = TRUE;
      } else {
	last_cmd = last_mob_load = 0;
	last_mob_followable = 0;
      }
      tobj = NULL;
      break;

    case 'C':			/* utilize furniture. position is what mobs default */
      if ((mob_index[ZCMD.arg1].number < ZCMD.arg2) && number(1, 100) <= ZCMD.arg7) {
	mob = read_mobile(ZCMD.arg1, REAL);
	put_char_to_room(mob, ZCMD.arg3, -1, ZCMD.arg4, ZCMD.arg5, ZCMD.arg6);
	GET_BIRTHPLACE(mob) = world[ZCMD.arg3].number;
	done = FALSE;
	for (tmp_obj = world[ZCMD.arg3].contents; tmp_obj && !done; tmp_obj = tmp_obj->next)
	  if (GET_OBJ_X_CORD(tmp_obj) == ZCMD.arg4 &&
	      GET_OBJ_Y_CORD(tmp_obj) == ZCMD.arg5 &&
	      GET_OBJ_Z_CORD(tmp_obj) == ZCMD.arg6) {
	    char_to_object(mob, tmp_obj);
	    done = TRUE;
	  }
	load_mtrigger(mob);
	if (!no_specials)
	  special(mob, 0, "", "", PROC_NORMAL);
	tmp_mob = mob;
	last_mob = last_mob_followable = mob;
	last_cmd = last_mob_load = 1;
	mob_load = TRUE;
      } else {
	last_cmd = last_mob_load = 0;
	last_mob_followable = 0;
      }
      tobj = NULL;
      break;

    case 'T':			/* trigger command; details to be filled in later */
      if (ZCMD.arg1 == MOB_TRIGGER && tmob) {
	if (!SCRIPT(tmob))
	  CREATE(SCRIPT(tmob), struct script_data, 1);
	add_trigger(SCRIPT(tmob), read_trigger(real_trigger(ZCMD.arg2)), -1);
	last_cmd = 1;
      } else if (ZCMD.arg1 == OBJ_TRIGGER && tobj) {
	if (!SCRIPT(tobj))
	  CREATE(SCRIPT(tobj), struct script_data, 1);
	add_trigger(SCRIPT(tobj), read_trigger(real_trigger(ZCMD.arg2)), -1);
	last_cmd = 1;
      }
      break;

    case 'V':
      if (ZCMD.arg1 == MOB_TRIGGER && tmob) {
	if (!SCRIPT(tmob)) {
	  ZONE_ERROR("Attempt to give variable to scriptless mobile");
	} else
	  add_var(&(SCRIPT(tmob)->global_vars), ZCMD.sarg1, ZCMD.sarg2,
		  ZCMD.arg3);
	last_cmd = 1;
      } else if (ZCMD.arg1 == OBJ_TRIGGER && tobj) {
	if (!SCRIPT(tobj)) {
	  ZONE_ERROR("Attempt to give variable to scriptless object");
	} else
	  add_var(&(SCRIPT(tobj)->global_vars), ZCMD.sarg1, ZCMD.sarg2,
		  ZCMD.arg3);
	last_cmd = 1;
      } else if (ZCMD.arg1 == WLD_TRIGGER) {
	if (ZCMD.arg2 < 0 || ZCMD.arg2 > top_of_world) {
	  ZONE_ERROR("Invalid room number in variable assignment");
	} else {
	  if (!(world[ZCMD.arg2].script)) {
	    ZONE_ERROR("Attempt to give variable to scriptless object");
	  } else
	    add_var(&(world[ZCMD.arg2].script->global_vars),
		    ZCMD.sarg1, ZCMD.sarg2, ZCMD.arg3);
	  last_cmd = 1;
	}
      }
      break;
    default:
      ZONE_ERROR("unknown cmd in reset table; cmd disabled");
      ZCMD.command = '*';
      break;
    }
  }

  zone_table[zone].age = 0;

  /* handle reset_wtrigger's */
  room_vnum = zone_table[zone].number * 100;
  while (room_vnum <= zone_table[zone].top) {
    room_rnum = real_room(room_vnum);
    if (room_rnum != NOWHERE)
      reset_wtrigger(&world[room_rnum]);
    room_vnum++;
  }
}



/* for use in reset_zone; return TRUE if zone 'nr' is free of PC's  */
int is_empty(int zone_nr)
{
  struct descriptor_data *i;

  for (i = descriptor_list; i; i = i->next)
    if (!i->connected)
      if (world[i->character->in_room].zone == zone_nr)
	return 0;

  return 1;
}
/*************************************************************************
*  stuff related to the save/load player system				 *
*************************************************************************/ long get_id_by_name(char *name)
{
  int i;

  one_argument(name, arg);
  for (i = 0; i <= top_of_p_table; i++)
    if (!strcmp((player_table + i)->name, arg))
      return ((player_table + i)->id);

  return -1;
}
char *get_name_by_id(long id)
{
  int i;

  for (i = 0; i <= top_of_p_table; i++)
    if ((player_table + i)->id == id)
      return ((player_table + i)->name);

  return NULL;
}				/* Load a char, TRUE if loaded, FALSE if not */
int load_char(char *name, struct char_file_u *char_element)
{
  int player_i;

  int find_name(char *name);

  if ((player_i = find_name(name)) >= 0) {
    fseek(player_fl, (long) (player_i * sizeof(struct char_file_u)), SEEK_SET);
    fread(char_element, sizeof(struct char_file_u), 1, player_fl);
    return (player_i);
  } else
    return (-1);
}




/*
 * write the vital data of a player to the player file
 *
 * NOTE: load_room should be an *RNUM* now.  It is converted to a vnum here.
 */
void save_char(struct char_data *ch, int load_room)
{
  struct char_file_u st;

  if (IS_NPC(ch) || !ch->desc || GET_PFILEPOS(ch) < 0)
    return;

  char_to_store(ch, &st);

  strncpy(st.host, ch->desc->host, HOST_LENGTH);
  st.host[HOST_LENGTH] = '\0';
  /* If we were called with nowhere, then we didnt want to overwrite
     the last saved loadroom. Such as when a god is messing with them */
  if (!PLR_FLAGGED(ch, PLR_LOADROOM) && load_room > 0) {
    st.player_specials_saved.load_room = world[load_room].number;
    GET_LOADROOM(ch) = world[load_room].number;
  }
  fseek(player_fl, GET_PFILEPOS(ch) * sizeof(struct char_file_u), SEEK_SET);
  fwrite(&st, sizeof(struct char_file_u), 1, player_fl);
}



/* copy data from the file structure to a char struct */
void store_to_char(struct char_file_u *st, struct char_data *ch)
{
  int i;

  /* to save memory, only PC's -- not MOB's -- have player_specials */
  if (ch->player_specials == NULL)
    CREATE(ch->player_specials, struct player_special_data, 1);

  GET_SEX(ch) = st->sex;
  GET_CLASS(ch) = st->class;
  GET_RACE(ch) = st->race;
  GET_LEVEL(ch) = st->level;
  GET_SIZE(ch) = st->size;

  ch->player.long_descr = NULL;
  ch->player.title = str_dup(st->title);
  ch->player.description = str_dup(st->description);
  ch->player.short_descr = str_dup(st->short_descr);

  ch->player.hometown = st->hometown;
  ch->player.time.birth = st->birth;
  ch->player.time.played = st->played;
  ch->player.time.logon = time(0);

  ch->player.weight = st->weight;
  ch->player.height = st->height;
/*
   POOFIN(ch) = st->player_specials_saved.poofin;
   POOFOUT(ch) = st->player_specials_saved.poofout;
 */
  ch->current_quest = st->current_quest;
  ch->num_completed_quests = -1;

  for (i = 0; st->completed_quests[i] != -1; i++)
    add_completed_quest(ch, st->completed_quests[i]);

  ch->real_abils = st->abilities;
  ch->aff_abils = st->abilities;
  ch->points = st->points;
  ch->char_specials.saved = st->char_specials_saved;
  ch->player_specials->saved = st->player_specials_saved;

  ch->char_specials.carry_weight = 0;
  ch->char_specials.carry_items = 0;
  ch->points.armor = 10;
  ch->points.hitroll = 0;
  ch->points.damroll = 0;

  if (ch->player.name == NULL)
    CREATE(ch->player.name, char, strlen(st->name) + 1);
  strcpy(ch->player.name, st->name);
  strcpy(ch->player.passwd, st->pwd);

  /* Add all spell effects */
  for (i = 0; i < MAX_AFFECT; i++) {
    if (st->affected[i].type)
      affect_to_char(ch, &st->affected[i]);
  }				/*
				 * If you're not poisioned and you've been away for more than an hour of
				 * real time, we'll set your HMV back to full
				 */
  if (!IS_AFFECTED(ch, AFF_POISON) &&
      (((long) (time(0) - st->last_logon)) >= SECS_PER_REAL_HOUR)) {
    GET_HIT(ch) = GET_MAX_HIT(ch);
  }
}				/* store_to_char */




/* copy vital data from a players char-structure to the file structure */
void char_to_store(struct char_data *ch, struct char_file_u *st)
{
  int i;
  struct affected_type *af;
  struct obj_data *char_eq[NUM_WEARS];
  sh_int hit;

  /*
   * save current values, because unequip_char will call
   * affect_total(), which will reset points to lower values
   * if player is wearing +points items.  We will restore old
   * points level after reequiping.
   */
  hit = GET_HIT(ch);

  /* Unaffect everything a character can be affected by */

  for (i = 0; i < NUM_WEARS; i++) {
    if (GET_EQ(ch, i)) {
      char_eq[i] = unequip_char(ch, i);
      remove_otrigger(char_eq[i], ch);
    } else
      char_eq[i] = NULL;
  }
  for (af = ch->affected, i = 0; i < MAX_AFFECT; i++) {
    if (af) {
      st->affected[i] = *af;
      st->affected[i].next = 0;
      af = af->next;
    } else {
      st->affected[i].type = 0;	/* Zero signifies not used */
      st->affected[i].duration = 0;
      st->affected[i].modifier = 0;
      st->affected[i].location = 0;
      st->affected[i].bitvector = 0;
      st->affected[i].next = 0;
    }
  }


  /*
   * remove the affections so that the raw values are stored; otherwise the
   * effects are doubled when the char logs back in.
   */

  while (ch->affected)
    affect_remove(ch, ch->affected);

  if ((i >= MAX_AFFECT) && af && af->next)
    log("SYSERR: WARNING: OUT OF STORE ROOM FOR AFFECTED TYPES!!!");

  ch->aff_abils = ch->real_abils;

  st->birth = ch->player.time.birth;
  st->played = ch->player.time.played;
  st->played += (long) (time(0) - ch->player.time.logon);
  st->last_logon = time(0);

  ch->player.time.played = st->played;
  ch->player.time.logon = time(0);

  st->hometown = ch->player.hometown;
  st->weight = GET_WEIGHT(ch);
  st->height = GET_HEIGHT(ch);
#if 0
  if (POOFIN(ch))
    strcpy(st->player_specials_saved.poofin, POOFIN(ch));
  if (POOFOUT(ch))
    strcpy(st->player_specials_saved.poofout, POOFOUT(ch));
#endif
  st->sex = GET_SEX(ch);
  st->class = GET_CLASS(ch);
  st->race = GET_RACE(ch);
  st->level = GET_LEVEL(ch);
  st->size = GET_SIZE(ch);
  st->abilities = ch->real_abils;
  st->points = ch->points;
  st->char_specials_saved = ch->char_specials.saved;
  st->player_specials_saved = ch->player_specials->saved;

  st->points.armor = 10;
  st->points.hitroll = 0;
  st->points.damroll = 0;

  st->current_quest = ch->current_quest;

  for (i = 0; i < MAX_COMPLETED_QUESTS; i++)
    st->completed_quests[i] = -1;
  for (i = 0; i <= ch->num_completed_quests; i++)
    st->completed_quests[i] = ch->completed_quests[i];

  if (GET_TITLE(ch))
    strcpy(st->title, GET_TITLE(ch));
  else
    *st->title = '\0';

  if (ch->player.description)
    strcpy(st->description, ch->player.description);
  else
    *st->description = '\0';

  if (ch->player.short_descr)
    strcpy(st->short_descr, ch->player.short_descr);
  else
    *st->short_descr = '\0';

  strcpy(st->name, GET_NAME(ch));
  strcpy(st->pwd, GET_PASSWD(ch));

  /* add spell and eq affections back in now */
  for (i = 0; i < MAX_AFFECT; i++) {
    if (st->affected[i].type)
      affect_to_char(ch, &st->affected[i]);
  }

  for (i = 0; i < NUM_WEARS; i++) {
    if (char_eq[i]) {
      if (wear_otrigger(char_eq[i], ch, i))
	equip_char(ch, char_eq[i], i);
      else
	obj_to_char(char_eq[i], ch);
    }
  }
/*   affect_total(ch); unnecessary, I think !?! */

  /* restore our points to previous level */
  GET_HIT(ch) = hit;
}				/* Char to store */



void save_etext(struct char_data *ch)
{
/* this will be really cool soon */

}

/* create a new entry in the in-memory index table for the player file */
int create_entry(char *name)
{
  int i;

  if (top_of_p_table == -1) {
    CREATE(player_table, struct player_index_element, 1);
    top_of_p_table = 0;
  } else if (!(player_table = (struct player_index_element *)
	       realloc(player_table, sizeof(struct player_index_element) *
		        (++top_of_p_table + 1)))) {
    perror("SYSERR: create entry");
    raise(SIGSEGV);
  }
  CREATE(player_table[top_of_p_table].name, char, strlen(name) + 1);

  /* copy lowercase equivalent of name to table field */
  for (i = 0; (*(player_table[top_of_p_table].name + i) = LOWER(*(name + i)));
       i++);

  return (top_of_p_table);
}



/************************************************************************
*  funcs of a (more or less) general utility nature			*
************************************************************************/


/* read and allocate space for a '~'-terminated string from a given file */
char *fread_string(FILE * fl, char *error)
{
  char buf[MAX_STRING_LENGTH], tmp[512], *rslt;
  register char *point;
  int done = 0, length = 0, templength = 0;

  *buf = '\0';

  do {
    if (!fgets(tmp, 512, fl)) {
      fprintf(stderr, "SYSERR: fread_string: format error at or near %s\n",
	      error);
      raise(SIGSEGV);
    }
    /* If there is a '~', end the string; else put an "\r\n" over the '\n'. */ if ((point = strchr(tmp, '~')) != NULL) {
      *point = '\0';
      done = 1;
    } else {
      point = tmp + strlen(tmp) - 1;
      *(point++) = '\r';
      *(point++) = '\n';
      *point = '\0';
    }

    templength = strlen(tmp);

    if (length + templength >= MAX_STRING_LENGTH) {
      log("SYSERR: fread_string: string too large (db.c)");
      log(error);
      raise(SIGSEGV);
    } else {
      strcat(buf + length, tmp);
      length += templength;
    }
  } while (!done);

  /* allocate space for the new string and copy it */
  if (strlen(buf) > 0) {
    CREATE(rslt, char, length + 1);
    strcpy(rslt, buf);
  } else
    rslt = NULL;

  return rslt;
}


/* release memory allocated for a char struct */
void free_char(struct char_data *ch)
{
  int i;
  struct alias *a;
  void free_alias(struct alias *a);

  if (ch->player_specials != NULL && ch->player_specials != &dummy_mob) {
    while ((a = GET_ALIASES(ch)) != NULL) {
      GET_ALIASES(ch) = (GET_ALIASES(ch))->next;
      free_alias(a);
    }
    if (ch->player_specials->poofin != NULL)
      FREE(ch->player_specials->poofin);
    if (ch->player_specials->poofout != NULL)
      FREE(ch->player_specials->poofout);
    FREE(ch->player_specials);
    if (IS_NPC(ch))
      log("SYSERR: Mob had player_specials allocated!");
  }
  if (!IS_NPC(ch) || (IS_NPC(ch) && GET_MOB_RNUM(ch) == -1)) {
    /* if this is a player, or a non-prototyped non-player, free all */
    if (GET_NAME(ch))
      FREE(GET_NAME(ch));
    if (ch->player.title)
      FREE(ch->player.title);
    if (ch->player.short_descr)
      FREE(ch->player.short_descr);
    if (ch->player.long_descr)
      FREE(ch->player.long_descr);
    if (ch->player.description)
      FREE(ch->player.description);
    if (ch->completed_quests)
      FREE(ch->completed_quests);
  } else if ((i = GET_MOB_RNUM(ch)) > -1) {
    /* otherwise, free strings only if the string is not pointing at proto */
    if (ch->player.name && ch->player.name != mob_proto[i].player.name)
      FREE(ch->player.name);
    if (ch->player.title && ch->player.title != mob_proto[i].player.title)
      FREE(ch->player.title);
    if (ch->player.short_descr && ch->player.short_descr != mob_proto[i].player.short_descr)
      FREE(ch->player.short_descr);
    if (ch->player.long_descr && ch->player.long_descr != mob_proto[i].player.long_descr)
      FREE(ch->player.long_descr);
    if (ch->player.description && ch->player.description != mob_proto[i].player.description)
      FREE(ch->player.description);
  }
  while (ch->affected)
    affect_remove(ch, ch->affected);

  FREE(ch);
}




/* release memory allocated for an obj struct */
void free_obj(struct obj_data *obj)
{
  int nr;
  struct extra_descr_data *this, *next_one;

  if ((nr = GET_OBJ_RNUM(obj)) == -1) {
    if (obj->name)
      FREE(obj->name);
    if (obj->description)
      FREE(obj->description);
    if (obj->short_description)
      FREE(obj->short_description);
    if (obj->action_description)
      FREE(obj->action_description);
    if (obj->ex_description)
      for (this = obj->ex_description; this; this = next_one) {
	next_one = this->next;
	if (this->keyword)
	  FREE(this->keyword);
	if (this->description)
	  FREE(this->description);
	FREE(this);
      }
  } else {
    if (obj->name && obj->name != obj_proto[nr].name)
      FREE(obj->name);
    if (obj->description && obj->description != obj_proto[nr].description)
      FREE(obj->description);
    if (obj->short_description && obj->short_description != obj_proto[nr].short_description)
      FREE(obj->short_description);
    if (obj->action_description && obj->action_description != obj_proto[nr].action_description)
      FREE(obj->action_description);
    if (obj->ex_description && obj->ex_description != obj_proto[nr].ex_description)
      for (this = obj->ex_description; this; this = next_one) {
	next_one = this->next;
	if (this->keyword)
	  FREE(this->keyword);
	if (this->description)
	  FREE(this->description);
	FREE(this);
      }
  }

  FREE(obj);
}



/* read contets of a text file, alloc space, point buf to it */
int file_to_string_alloc(char *name, char **buf)
{
  char temp[MAX_STRING_LENGTH];
  if (*buf)
    FREE(*buf);

  if (file_to_string(name, temp) < 0) {
    *buf = "";
    return -1;
  } else {
    *buf = str_dup(temp);
    return 0;
  }
}


/* read contents of a text file, and place in buf */
int file_to_string(char *name, char *buf)
{
  FILE *fl;
  char tmp[READ_SIZE + 3];

  *buf = '\0';

  if (!(fl = fopen(name, "r"))) {
    sprintf(tmp, "SYSERR: reading %s", name);
    perror(tmp);
    return (-1);
  }
  do {
    fgets(tmp, READ_SIZE, fl);
    tmp[strlen(tmp) - 1] = '\0';	/* take off the trailing \n */
    strcat(tmp, "\r\n");

    if (!feof(fl)) {
      if (strlen(buf) + strlen(tmp) + 1 > MAX_STRING_LENGTH) {
	sprintf(buf, "SYSERR: %s: string too big (%d max)", name,
		MAX_STRING_LENGTH);
	log(buf);
	*buf = '\0';
	return -1;
      }
      strcat(buf, tmp);
    }
  } while (!feof(fl));

  fclose(fl);

  return (0);
}



/* clear some of the the working variables of a char */
void reset_char(struct char_data *ch)
{
  int i;

  for (i = 0; i < NUM_WEARS; i++)
    GET_EQ(ch, i) = NULL;

  ch->followers = NULL;
  ch->watchers = NULL;
  ch->master = NULL;
  ch->watching = NULL;
  ch->in_room = NOWHERE;
  ch->carrying = NULL;
  ch->next = NULL;
  ch->next_in_room = NULL;
  FIGHTING(ch) = NULL;
  GET_BLINK(ch) = NULL;
  GET_FACING(ch) = NORTH;
  GET_AIMING(ch) = LOCATION_TORSO;
  GET_FORM(ch) = FORM_DEFAULT;
  GET_ACTION(ch) = NULL;
  ch->char_specials.position = POS_SITTING + STAT_STUNNED;
  ch->mob_specials.default_pos = POS_STANDING;
  ch->char_specials.carry_weight = 0;
  ch->char_specials.carry_items = 0;
  ch->char_specials.attacks_left = 0;
  VEHICLE(ch) = NULL;

  if (GET_HIT(ch) <= 0)
    GET_HIT(ch) = 1;

  GET_ID(ch) = GET_IDNUM(ch);

  check_regen_rates(ch);	/* start regening points */

  GET_LAST_TELL(ch) = NOBODY;
}

/* clear ALL the working variables of a char; do NOT free any space alloc'ed */
void clear_char(struct char_data *ch)
{
  memset((char *) ch, 0, sizeof(struct char_data));

  ch->in_room = NOWHERE;
  GET_PFILEPOS(ch) = -1;
  GET_WAS_IN(ch) = NOWHERE;
  SET_POS(ch, POS_STANDING + STAT_NORMAL);
  FURNITURE(ch) = NULL;
  ch->mob_specials.default_pos = POS_STANDING;
  RIDING(ch) = NULL;
  RIDDEN_BY(ch) = NULL;
  VEHICLE(ch) = NULL;
  GET_BLINK(ch) = NULL;
  GET_ACTION(ch) = NULL;
  GET_AC(ch) = 10;		/* Basic Armor */
  GET_MOB_TREASURE(ch) = -1;
  GET_MOB_ORIG_VNUM(ch) = 0;
  AFF_FLAGS(ch) = 0;
  AFF2_FLAGS(ch) = 0;
  AFF3_FLAGS(ch) = 0;
  AFF4_FLAGS(ch) = 0;
  SET_ALIGNMENT(ch, ALIGN_NONE);
}

void clear_object(struct obj_data *obj)
{
  memset((char *) obj, 0, sizeof(struct obj_data));

  obj->item_number = NOTHING;
  obj->in_room = NOWHERE;
  obj->worn_on = NOWHERE;
  obj->people = NULL;
}


/* returns the real number of the room with given virtual number */
int real_room(int virtual)
{
  int bot, top, mid;

  bot = 0;
  top = top_of_world;

  if (virtual <= 0)
    return -1;

  /* perform binary search on world-table */
  for (;;) {
    mid = (bot + top) / 2;

    if ((world + mid)->number == virtual)
      return mid;
    if (bot >= top)
      return NOWHERE;
    if ((world + mid)->number > virtual)
      top = mid - 1;
    else
      bot = mid + 1;
  }
}

void load_path_from_mob(FILE * mob_f, struct index_data *mob_index)
{
  struct path_info *path = NULL, *temp = NULL;
  char line[MAX_STRING_LENGTH];
  int t[10];			/* don't really need ten, but it makes me feel good */

  vlogf("Mob %d contains a path", mob_index->virtual);

  if (!get_line(mob_f, line) || !get_line(mob_f, line)) {
    /* the first line is just a bang...!.... discarded here, and
       replaced immediately with the second... */
    log("Error while reading path. Bailing out.");
    raise(SIGSEGV);
  }
  if (!sscanf(line, "%d %d %d %d %d ", t, t + 1, t + 2, t + 3, t + 4)) {
    log("Error while reading path. Bailing like hell.");
    raise(SIGSEGV);
  }
  /* lets grab pre-existing, or not list. */
  if (mob_index->paths)
    temp = mob_index->paths;

/* actually create the info required */
  CREATE(path, struct path_info, 1);
  CREATE(path->performance, struct path_perform, 1);

  path->performance->hrmin = t[0];
  path->performance->day = t[1];
  path->performance->week = t[2];
  path->performance->month = t[3];
  path->type = t[4];

  get_line(mob_f, line);

  if (path->type == PATH_GOTO)
    path->target_room = atoi(line);
  else
    path->arguments = str_dup(line);

  mob_index->paths = path;
  if (temp)
    mob_index->paths->next = temp;
}

void dequeue_paths(void)
{
  extern struct path_q *mobile_paths;
  extern struct time_info_data time_info;

  struct path_q *temp, *latemp;
  struct path_info *temp_path;
  struct path_perform *perf;

  int changes = 0, pcount = 0;

  temp = mobile_paths;

  while (temp) {
    if (!temp->mob) {
      temp->pnum = -1;
      changes = 1;
      log("No mob when path is called!");
    } else {
      temp_path = GET_PATH(temp->mob);

      for (pcount = temp->pnum; pcount != 0; pcount--) {
	if (temp_path && temp_path->next) {
	  temp_path = temp_path->next;
	} else {
	  log("Serious error in dequeue_paths.. attempting to continue");
	  /* maybe I make better error handling here later */
	  temp_path = GET_PATH(temp->mob);
	}
      }
      perf = temp_path->performance;
      /* now this checks if each entry in the first queue is ready for 
         execution in the following second.  - should only check hourmin */
      /*
         if (perf->hrmin == -1)
         log("PATH: found an immediate request");
       */
      if (((((time_info.hours) * 60) + time_info.minutes) == perf->hrmin)
	  || (perf->hrmin == -1)) {
	/* it is good */
	switch (temp_path->type) {
	case PATH_GOTO:
	  TARGET_ROOM(temp->mob) = temp_path->target_room;
	  break;
	case PATH_ACTION:
	  command_interpreter(temp->mob, temp_path->arguments);
	  break;
	default:
	  sprintf(buf, "Error - no type when parsing path for mob vnum(%d)", GET_MOB_VNUM(temp->mob));
	  break;
	}
	changes = 1;
	temp->pnum = -1;

      }
    }

    /* next path please */
    temp = temp->next;
  }

  /* now we have to go through and re-make the queue */
  /* probably a muchmuch better way to do this.. */
  if (changes) {
    temp = mobile_paths;
    mobile_paths = NULL;
    while (temp) {
      latemp = temp->next;
      if (temp->pnum == -1) {
	FREE(temp);
      } else {
	temp->next = mobile_paths;
	mobile_paths = temp;
      }
      temp = latemp;
    }
  }
}

void check_path(void)
{
  extern struct time_info_data time_info;
  extern struct path_q *mobile_paths;

/* someday, something better than temp will strike me.. */

  struct path_info *temp;
  struct path_perform *perf;
  struct path_q *temp_path;
  struct char_data *ch, *next_ch;
  int tohrmin, today, toweek, tomonth, enqueue, ccount;

  /* Lets erase the current mobile_paths? Sure, we'll see what happens. */
  temp_path = mobile_paths;
  while (temp_path) {
    mobile_paths = temp_path->next;
    FREE(temp_path);
    temp_path = mobile_paths;
  }
  tohrmin = (time_info.minutes + (time_info.hours * 60));
  today = time_info.day + 1;
  /* this looks ugly. i'm sure it works though, so deal. */
  toweek = (((((time_info.month * 28) + time_info.day + 1) % 28) + 1) % 5) + 1;
  tomonth = time_info.month + 1;

  for (ch = character_list; ch; ch = next_ch) {
    next_ch = ch->next;
    if (IS_NPC(ch) && GET_PATH(ch))
      temp = GET_PATH(ch);
    else
      temp = NULL;

    ccount = 0;
    while (temp) {
      enqueue = 0;
      perf = temp->performance;

      if ((perf->day == today || perf->day == -1) &&
	  ((tohrmin < perf->hrmin) || perf->hrmin == -1) && perf->week == -1) {
	enqueue = 1;
      }
      if (enqueue) {
	CREATE(temp_path, struct path_q, 1);
	temp_path->pnum = ccount;
	temp_path->mob = ch;
	temp_path->next = mobile_paths;
	mobile_paths = temp_path;
      }
      enqueue = 0;
      temp = temp->next;
      ccount++;
    }
  }
}



    /*
     * Read a number from a file.
     */
int fread_number(FILE * fp)
{
  int number;
  bool sign;
  char c;

  do {
    c = getc(fp);
  } while (isspace(c));

  number = 0;

  sign = FALSE;
  if (c == '+') {
    c = getc(fp);
  } else if (c == '-') {
    sign = TRUE;
    c = getc(fp);
  }
  if (!isdigit(c)) {
    log("Fread_number: bad format.");
    raise(SIGSEGV);
  }
  while (isdigit(c)) {
    number = number * 10 + c - '0';
    c = getc(fp);
  }

  if (sign)
    number = 0 - number;

  if (c == '|')
    number += fread_number(fp);
  else if (c != ' ')
    ungetc(c, fp);

  return number;
}

      /*
       * Read to end of line (for comments).
       */
void fread_to_eol(FILE * fp)
{
  char c;

  do {
    c = getc(fp);
  } while (c != '\n' && c != '\r');

  do {
    c = getc(fp);
  } while (c == '\n' || c == '\r');

  ungetc(c, fp);
  return;
}


	/*
	 * Read one word (into static buffer).
	 */
char *fread_word(FILE * fp)
{
  static char word[MAX_INPUT_LENGTH];
  char *pword;
  char cEnd;

  do {
    cEnd = getc(fp);
  }
  while (isspace(cEnd));

  if (cEnd == '\'' || cEnd == '"') {
    pword = word;
  } else {
    word[0] = cEnd;
    pword = word + 1;
    cEnd = ' ';
  }

  for (; pword < word + MAX_INPUT_LENGTH; pword++) {
    *pword = getc(fp);
    if (cEnd == ' ' ? isspace(*pword) || *pword == '~' : *pword == cEnd) {
      if (cEnd == ' ' || cEnd == '~')
	ungetc(*pword, fp);
      *pword = '\0';
      return word;
    }
  }

  log("SYSERR: Fread_word: word too long.");
  raise(SIGSEGV);
  return NULL;
}



struct index_data *get_obj_index(int vnum)
{
  int nr;
  for (nr = 0; nr <= top_of_objt; nr++) {
    if (obj_index[nr].virtual == vnum)
      return &obj_index[nr];
  }
  return NULL;
}

struct index_data *get_mob_index(int vnum)
{
  int nr;
  for (nr = 0; nr <= top_of_mobt; nr++) {
    if (mob_index[nr].virtual == vnum)
      return &mob_index[nr];
  }
  return NULL;
}


/* returns the real number of the monster with given virtual number */
int real_mobile(int virtual)
{
  int bot, top, mid;

  bot = 0;
  top = top_of_mobt;

  /* perform binary search on mob-table */
  for (;;) {
    mid = (bot + top) / 2;

    if ((mob_index + mid)->virtual == virtual)
      return (mid);
    if (bot >= top)
      return (-1);
    if ((mob_index + mid)->virtual > virtual)
      top = mid - 1;
    else
      bot = mid + 1;
  }
}

/* returns the real number of the object with given virtual number */
int real_object(int virtual)
{
  int bot, top, mid;

  bot = 0;
  top = top_of_objt;

  /* perform binary search on obj-table */
  for (;;) {
    mid = (bot + top) / 2;

    if ((obj_index + mid)->virtual == virtual)
      return (mid);
    if (bot >= top)
      return (-1);
    if ((obj_index + mid)->virtual > virtual)
      top = mid - 1;
    else
      bot = mid + 1;
  }
}
void write_mud_date_to_file(void)
{
  FILE *f;
  struct time_write date;

  f = fopen("etc/date_record", "w");
  date.year = time_info.year;
  date.month = time_info.month;
  date.day = time_info.day;
  fwrite(&date, sizeof(struct time_write), 1, f);
  fclose(f);
}

void read_mud_date_from_file(void)
{
  FILE *f;
  struct time_write read_date;

  f = fopen("etc/date_record", "r");
  if (!f) {
    log("SYSERR: File etc/date_record not found, mud date will be reset to default!");
    return;
  }
  fread(&read_date, sizeof(struct time_write), 1, f);
/*  time_info.year = read_date.year; */
  time_info.year = 575;
  time_info.month = read_date.month;
  time_info.day = read_date.day;
  fclose(f);
}

/* these will aid us in upping max_existance */
ACMD(do_existance)
{
  int cmd_no, zone, n_vnum, n_max, n_what;
  int counter = 0, changed = FALSE, amount = 0;
  char vnum[MAX_STRING_LENGTH], max[MAX_STRING_LENGTH], what[MAX_STRING_LENGTH];
  char *usage;

  *vnum = '\0';
  *max = '\0';
  *what = '\0';

  usage = str_dup("Usage: existance [mob|obj] [vnum] [max]\r\n  If max is set to -1, the max will be set sequentially.\r\n");
  argument = two_arguments(argument, what, vnum);
  one_argument(argument, max);


  if (!*what || !*vnum) {
    send_to_char(usage, ch);
    return;
  }
  if (!is_number(vnum)) {
    send_to_char(usage, ch);
    return;
  }
  if (!str_cmp(what, "obj"))
    n_what = 1;
  else if (!str_cmp(what, "mob"))
    n_what = 2;
  else {
    send_to_char(usage, ch);
    return;
  }

  n_vnum = atoi(vnum);
  if (*max && is_number(max))
    n_max = atoi(max);
  else if (*max)
    n_max = -1;
  else
    n_max = 0;

  for (zone = 0; zone <= top_of_zone_table; zone++) {
    changed = FALSE;
    for (cmd_no = 0; ZCMD.command != 'S'; cmd_no++) {
      switch (ZCMD.command) {
      case 'M':
	if (n_what == 2 && n_vnum == ZCMD.arg1) {
	  counter++;
	  if (n_max == -1) {
	    ZCMD.arg2 = counter;
	    changed = TRUE;
	  } else if (n_max > 0) {
	    ZCMD.arg2 = n_max;
	    changed = TRUE;
	  } else {
	    amount = ZCMD.arg2;
	    changed = FALSE;
	  }
	}
	break;
      case 'O':
      case 'G':
      case 'P':
      case 'E':
	if (n_what == 1 && n_vnum == ZCMD.arg1) {
	  counter++;
	  if (n_max == -1) {
	    ZCMD.arg2 = counter;
	    changed = TRUE;
	  } else if (n_max > 0) {
	    ZCMD.arg2 = n_max;
	    changed = TRUE;
	  } else {
	    amount = ZCMD.arg2;
	    changed = FALSE;
	  }
	}
	break;
      }
    }
    if (changed && n_what == 1)
      olc_add_to_save_list(zone, OLC_SAVE_OBJ);
    else if (changed && n_what == 2)
      olc_add_to_save_list(zone, OLC_SAVE_MOB);
  }
  if (n_max == 0)		/* We are just checking the max, not changing */
    sprintf(buf, "The highest setting for existance is %d, with %d occurances.\r\n", amount, counter);
  else
    sprintf(buf, "Ok, %d items changed to have a max_existance of %d.\r\n", counter, n_max);
  send_to_char(buf, ch);
}
