

#define BFS_ERROR           -1
#define BFS_ALREADY_THERE   -2
#define BFS_NO_PATH         -3

#define BFS_MAX_ROOMS      8192U /* limits how many rooms are searched for a target */


/*
 * Utility macros 
 */
#define MARK(room) (SET_BIT(world[room].room_flags, ROOM_BFS_MARK))
#define UNMARK(room) (REMOVE_BIT(world[room].room_flags, ROOM_BFS_MARK))
#define IS_MARKED(room) (IS_SET(world[room].room_flags, ROOM_BFS_MARK))
#define TOROOM(x, y) (world[(x)].dir_option[(y)]->to_room)
#define IS_CLOSED(x, y) (IS_SET(world[(x)].dir_option[(y)]->exit_info, EX_CLOSED))
#define IS_SECRET(x, y) (IS_SET(world[(x)].dir_option[(y)]->exit_info, EX_SECRET))
#define IS_WALLED(x, y) (IS_SET(world[(x)].dir_option[(y)]->exit_info, EX_WALLED))
#define IS_LOCKED(x, y) (IS_CLOSED(x, y) && \
                         IS_SET(world[(x)].dir_option[(y)]->exit_info, EX_LOCKED))
#define IS_HIDDEN(x, y) (IS_SECRET(x, y) || IS_BLOCKED(x, y))
#define IS_BLOCKED(x, y) (IS_CLOSED(x, y) && \
                         IS_SET(world[(x)].dir_option[(y)]->exit_info, \
                                EX_BLOCKED))
#define TO_OCEAN(x, y) (world[TOROOM(x, y)].sector_type == SECT_WATER_NOSWIM)
#define TO_ROAD(x, y) (world[(x)].dir_option[(y)] && \
                       (TOROOM(x, y) != NOWHERE) &&  \
                       (world[TOROOM(x, y)].sector_type == SECT_CITY))
#define ON_ROAD(x) (world[x].sector_type == SECT_CITY)
#define TO_NOSWIM(x, y)(world[TOROOM(x, y)].sector_type == \
                        SECT_WATER_NOSWIM)
#define NEEDS_FLY(x, y) (TO_OCEAN(x,y) || TO_NOSWIM(x,y))
#define TO_NOGROUND(x, y)(world[TOROOM(x, y)].sector_type == \
                          SECT_NO_GROUND)
#define VALID_EDGE(x, y) (world[(x)].dir_option[(y)] && \
			  (TOROOM(x, y) != NOWHERE) &&	\
                         /* (!IS_SECRET(x, y)) && */        \
                          (!IS_WALLED(x, y)) &&         \
                          (!IS_LOCKED(x, y)) &&         \
                          (!IS_BLOCKED(x, y)) &&        \
                          (!TO_OCEAN(x, y)) &&          \
                          (zone_table[world[x].zone].kingdom == \
                           zone_table[world[TOROOM(x,y)].zone].kingdom) && \
			  (!IS_MARKED(TOROOM(x, y))))

#define VALID_WAGON_EDGE(x, y) (world[(x)].dir_option[(y)] && \
			  (TOROOM(x, y) != NOWHERE) &&	\
			  (TO_ROAD(x, y)) &&		\
                          (!IS_SECRET(x, y)) &&         \
                          (!IS_WALLED(x, y)) &&         \
                          (!IS_LOCKED(x, y)) &&         \
                          (!IS_BLOCKED(x, y)) &&        \
                          (!TO_OCEAN(x, y)) &&          \
			  (!IS_MARKED(TOROOM(x, y))))

#define VALID_SHIP_EDGE(x, y) (world[(x)].dir_option[(y)] && \
                               (TOROOM(x, y) != NOWHERE) &&	\
                               (!IS_SECRET(x, y)) &&         \
                               (!IS_LOCKED(x, y)) &&         \
                               (!IS_BLOCKED(x, y)) &&        \
                               (TO_OCEAN(x, y) \
				|| TO_NOSWIM(x, y)) && \
                               (!IS_MARKED(TOROOM(x, y))))

#define VALID_DIST_EDGE(x, y) (world[(x)].dir_option[(y)] && \
                               (TOROOM(x, y) != NOWHERE) &&	\
                               (!IS_MARKED(TOROOM(x, y))))

#define HITCHABLE(o) (GET_ITEM_TYPE(o) == ITEM_VEHICLE || \
	(GET_ITEM_TYPE(o) == ITEM_CONTAINER && \
	(isname (o->name, "cart") || isname (o->name, "wagon") || \
	isname (o->name, "carriage") || isname (o->name, "chariot"))))


