
/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "db.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "spells.h"
#include "olc.h"
#include "events.h"
#include "map.h"

/* external structs */
extern struct char_data *character_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct room_data *world;
extern struct str_app_type str_app[];
extern struct char_data *mob_proto;
extern int is_empty(int zone_nr);
extern int find_first_step(int src, int target);
extern char *dirs[];
extern int rev_dir[];
extern char *race_types[];
int CoordsInRangeForInnerRoomTravel(struct char_data *ch, int dir);
extern byte SameSectorType(struct char_data *ch, int door);
byte DetermineDirectionTraveled(struct char_data *mob, int dir);
extern void hunt_victim(struct char_data *ch);
void move_closer(struct char_data *ch, struct char_data *vict, struct obj_data *obj);
extern int CantEnterCoord(struct char_data *ch, int dir);

struct mob_movement_event {
  struct char_data *ch;
  struct char_data *vict;
  struct obj_data *obj;
  room_vnum starting_room;
  room_vnum ending_room;
  sh_int priority_dir;
};

EVENTFUNC(mob_movement_event)
{
  struct mob_movement_event *mme = (struct mob_movement_event *) event_obj;
  struct char_data *ch, *vict;
  struct obj_data *obj;
  room_vnum starting_room;
  room_vnum ending_room;
  int dir = -1, priority_dir, speed = 10;

  ch = mme->ch;
  vict = mme->vict;
  obj = mme->obj;
  starting_room = mme->starting_room;
  ending_room = mme->ending_room;
  priority_dir = mme->priority_dir;

  /* first off, any/all reasons to cancel this move. Very important,
     since many things can loop this event. */
  if (FIGHTING(ch) || !AWAKE(ch)) {
    free(event_obj);
    GET_MOB_MOVE(ch) = NULL;
    return 0;
  }
  hunt_victim(ch);

  /* hunting mobs */
  if (TARGET_ROOM(ch) != -1 && real_room(TARGET_ROOM(ch)) > 0) {
    dir = find_first_step(ch->in_room, TARGET_ROOM(ch));
    if (dir == BFS_ALREADY_THERE) {
      TARGET_ROOM(ch) = -1;
      free(event_obj);
      GET_MOB_MOVE(ch) = NULL;
      return 0;
    } else if (dir == BFS_ERROR || dir == BFS_NO_PATH) {
      sprintf(buf, "Removing GOTO path from mob %d due to error.",
	      GET_MOB_VNUM(ch));
      log(buf);
      TARGET_ROOM(ch) = -1;
      free(event_obj);
      GET_MOB_MOVE(ch) = NULL;
      return 0;
    } else {			/* this is if everything worked! */
      if ((GET_POS(ch) == POS_STANDING) &&
	  ((dir < NUM_OF_DIRS) &&
      (world[EXIT(ch, dir)->to_room].zone == world[ch->in_room].zone))) {
	perform_move(ch, dir, 1);
	return number(1, 5);	/* should be mob speed */
      }
    }
  }
  /* priority mobs are those trying to get an object, eg */
  if (priority_dir > -1) {
    dir = priority_dir;
    speed = number(1, 5);	/* should be mob speed */
    perform_move(ch, dir, 1);
    move_closer(ch, vict, obj);	/* reset event with new info */
    free(event_obj);
    return 0;
  }
  /* normal mobs, not hunting anyone/anything */
  if (number(0, 2)) {
#if 0
    dir = number(0, NUM_OF_DIRS - 1);
#else
    dir = ch->mob_specials.last_direction;
#endif
    if (!MOB_FLAGGED(ch, MOB_SENTINEL) && !AFF_FLAGGED(ch, AFF_TAMED) && (GET_POS(ch) == POS_STANDING)) {
      dir = DetermineDirectionTraveled(ch, dir);
      if (!MOB_FLAGGED(ch, MOB_STAY_SECTOR) || SameSectorType(ch, dir)) {
	speed = number(5, 10);	/* should be mob speed */
	perform_move(ch, dir, 1);
	sprintf(buf, "(moving %s)\r\n", dirs[dir]);
	send_to_char(buf, ch);
	return speed;
      }
    }
  }
  free(event_obj);
  GET_MOB_MOVE(ch) = NULL;
  return 0;
}

void HandleMobMoves(struct char_data *ch)
{
  struct mob_movement_event *mme;

  if (GET_MOB_MOVE(ch))
    event_cancel(GET_MOB_MOVE(ch));

  if (!MOB_FLAGGED(ch, MOB_SLOW) || !number(0, 2)) {
    CREATE(mme, struct mob_movement_event, 1);
    mme->ch = ch;
    mme->vict = NULL;
    mme->obj = NULL;
    mme->starting_room = world[ch->in_room].number;
    mme->ending_room = -1;
    mme->priority_dir = -1;
    GET_MOB_MOVE(ch) = event_create(mob_movement_event, mme, number(1, 10));
  }
}


void move_closer(struct char_data *ch, struct char_data *vict, struct obj_data *obj)
{
  struct mob_movement_event *mme;
  int dir = -1;

  if (!ch)
    return;

  if (vict) {
    if (GET_X_CORD(ch) > GET_X_CORD(vict)) {
      if (GET_Y_CORD(ch) > GET_Y_CORD(vict))
	dir = SOUTHWEST;
      else if (GET_Y_CORD(ch) < GET_Y_CORD(vict))
	dir = SOUTHEAST;
      else
	dir = SOUTH;
    }
    if (GET_X_CORD(ch) < GET_X_CORD(vict)) {
      if (GET_Y_CORD(ch) > GET_Y_CORD(vict))
	dir = NORTHWEST;
      else if (GET_Y_CORD(ch) < GET_Y_CORD(vict))
	dir = NORTHEAST;
      else
	dir = NORTH;
    }
    if (GET_Y_CORD(ch) > GET_Y_CORD(vict))
      dir = WEST;
    if (GET_Y_CORD(ch) < GET_Y_CORD(vict))
      dir = EAST;
  } else if (obj) {
    if (GET_X_CORD(ch) > GET_OBJ_X_CORD(obj)) {
      if (GET_Y_CORD(ch) > GET_OBJ_Y_CORD(obj))
	dir = SOUTHWEST;
      else if (GET_Y_CORD(ch) < GET_OBJ_Y_CORD(obj))
	dir = SOUTHEAST;
      else
	dir = SOUTH;
    }
    if (GET_X_CORD(ch) < GET_OBJ_X_CORD(obj)) {
      if (GET_Y_CORD(ch) > GET_OBJ_Y_CORD(obj))
	dir = NORTHWEST;
      else if (GET_Y_CORD(ch) < GET_OBJ_Y_CORD(obj))
	dir = NORTHEAST;
      else
	dir = NORTH;
    }
    if (GET_Y_CORD(ch) > GET_OBJ_Y_CORD(obj))
      dir = WEST;
    if (GET_Y_CORD(ch) < GET_OBJ_Y_CORD(obj))
      dir = EAST;
  }
  if (CoordsInRangeForInnerRoomTravel(ch, dir) < 0)
    dir = number(0, 6);

  CREATE(mme, struct mob_movement_event, 1);
  mme->ch = ch;
  mme->vict = vict;
  mme->obj = obj;
  mme->starting_room = world[ch->in_room].number;
  mme->ending_room = -1;
  mme->priority_dir = dir;

  if (IS_NPC(ch)) {
// frequent crashes from this. Why does event_cancel cause so many headaches?
    //    if (GET_MOB_MOVE(ch))
    //      event_cancel(GET_MOB_MOVE(ch));
    GET_MOB_MOVE(ch) = NULL;
    GET_MOB_MOVE(ch) = event_create(mob_movement_event, mme, number(1, 5));
  }
  return;
}


/* decides if mob should move this way. New direction returned if not. */
byte DetermineDirectionTraveled(struct char_data * mob, int dir)
{
#if 0
  switch (mob->mob_specials.last_direction) {
  case NORTH:
    if (dir == SOUTH || dir == SOUTHWEST || dir == NORTHWEST)
      dir = NORTH;
  case EAST:
    if (dir == WEST || dir == NORTHWEST || dir == SOUTHWEST)
      dir = EAST;
  case SOUTH:
    if (dir == NORTH || dir == NORTHWEST || dir == NORTHEAST)
      dir = SOUTH;
  case WEST:
    if (dir == EAST || dir == NORTHEAST || dir == SOUTHEAST)
      dir = WEST;
  case UP:
    if (dir == DOWN)
      dir = UP;
  case DOWN:
    if (dir == UP)
      dir = DOWN;
  case NORTHEAST:
    if (dir == SOUTHWEST || dir == SOUTH || dir == WEST)
      dir = NORTHEAST;
  case NORTHWEST:
    if (dir == SOUTHEAST || dir == SOUTH || dir == EAST)
      dir = NORTHWEST;
  case SOUTHEAST:
    if (dir == NORTHWEST || dir == NORTH || dir == WEST)
      dir = SOUTHEAST;
  case SOUTHWEST:
    if (dir == NORTHEAST || dir == NORTH || dir == EAST)
      dir = SOUTHWEST;
  }
#else

#endif
  /* get out of doorways */
  if (GET_X_CORD(mob) <= GET_X_SIZE(mob->in_room) && GET_Y_CORD(mob) <= GET_Y_SIZE(mob->in_room) &&
      world[mob->in_room].map &&
   world[mob->in_room].map->map_grid[GET_X_CORD(mob)][GET_Y_CORD(mob)] &&
      world[mob->in_room].map->map_grid[GET_X_CORD(mob)][GET_Y_CORD(mob)] == MAP_OPEN_DOOR) {
    dir = number(0, 3);		/* the 4 cardinals */
    sprintf(buf, "DEBUG: Standing in a doorway, moving %s to get out.\r\n", dirs[dir]);
    send_to_char(buf, mob);
    mob->mob_specials.last_direction = dir;
  }
  if (CantEnterCoord(mob, dir)) {
    sprintf(buf, "DEBUG: I cant move %s, trying %s instead.\r\n", dirs[dir], dirs[rev_dir[dir]]);
    send_to_char(buf, mob);
    dir = rev_dir[dir];
    if (CantEnterCoord(mob, dir)) {
      dir = number(0, 3);
      sprintf(buf, "DEBUG: Thats no good either, lets try %s.\r\n", dirs[dir]);
      send_to_char(buf, mob);
    }
    mob->mob_specials.last_direction = dir;
  }
  return dir;
}
