/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"


#include "structs.h"
#include "utils.h"
#include "spells.h"
#include "handler.h"
#include "events.h"


/* Player point types for events */
#define REGEN_HIT      0
#define REGEN_VIT      1

#define PULSES_PER_MUD_HOUR     (SECS_PER_MUD_HOUR*PASSES_PER_SEC)


void RemoveLimb(struct char_data *ch, int location);


/* event object structure for point regen */
struct regen_event_obj {
  struct char_data *ch;		/* character regening */
  int type;			/* HIT only for now */
};


EVENTFUNC(points_event)
{
  struct regen_event_obj *regen = (struct regen_event_obj *) event_obj;
  struct char_data *ch;
  int type, gain;
  char logbuf[100];

  ch = regen->ch;
  type = regen->type;

  if (GET_STAT(ch) >= STAT_STUNNED && !IS_FIGHTING(ch)) {
    /* no help for the dying */

    /*
     * Increment type of points by one.
     * If not at max, reenqueue the event.
     */

    switch (type) {
    case REGEN_HIT:
      GET_HIT(ch) = MIN(GET_HIT(ch) + 1, GET_MAX_HIT(ch));
      GET_HIT_WA(ch) = MIN(GET_HIT_WA(ch) + 1, GET_MAX_HIT(ch) / 2);
      GET_HIT_WH(ch) = MIN(GET_HIT_WH(ch) + 1, GET_MAX_HIT(ch) / 4);
      GET_HIT_SA(ch) = MIN(GET_HIT_SA(ch) + 1, GET_MAX_HIT(ch) / 2);
      GET_HIT_SH(ch) = MIN(GET_HIT_SH(ch) + 1, GET_MAX_HIT(ch) / 4);
      GET_HIT_TO(ch) = MIN(GET_HIT_TO(ch) + 1, GET_MAX_HIT(ch));
      GET_HIT_LL(ch) = MIN(GET_HIT_LL(ch) + 1, GET_MAX_HIT(ch) / 2);
      GET_HIT_LF(ch) = MIN(GET_HIT_LF(ch) + 1, GET_MAX_HIT(ch) / 4);
      GET_HIT_RL(ch) = MIN(GET_HIT_RL(ch) + 1, GET_MAX_HIT(ch) / 2);
      GET_HIT_RF(ch) = MIN(GET_HIT_RF(ch) + 1, GET_MAX_HIT(ch) / 4);
      GET_HIT_HD(ch) = MIN(GET_HIT_HD(ch) + 1, GET_MAX_HIT(ch) / 4);

      if (GET_STAT(ch) <= STAT_STUNNED)
	update_pos(ch);

      gain = GET_MAX_HIT(ch);
      if ((GET_HIT(ch) < gain) || (GET_HIT_WA(ch) < gain / 2)
	  || (GET_HIT_WH(ch) < gain / 4) || (GET_HIT_SA(ch) < gain / 2)
	  || (GET_HIT_SH(ch) < gain / 4) || (GET_HIT_TO(ch) < gain)
	  || (GET_HIT_LL(ch) < gain / 2) || (GET_HIT_LF(ch) < gain / 4)
	  || (GET_HIT_RL(ch) < gain / 2) || (GET_HIT_RF(ch) < gain / 4)
	  || (GET_HIT_HD(ch) < gain / 4)) {
	/* reenqueue the event */
	gain = hit_gain(ch);
	return (PULSES_PER_MUD_HOUR / (gain ? gain : 1));
      }
      break;

    case REGEN_VIT:
      GET_VIT(ch) = MIN(GET_VIT(ch) + 1, GET_MAX_VIT(ch));

      if (GET_STAT(ch) <= STAT_STUNNED)
	update_pos(ch);

      if (GET_VIT(ch) < GET_MAX_VIT(ch)) {
	/* reenqueue the event */
	gain = vit_gain(ch);
	return (PULSES_PER_MUD_HOUR / (gain ? gain : 1));
      }
      break;
    default:
      sprintf(logbuf, "SYSERR:  Unknown points event type %d", type);
      log(logbuf);
      //FREE(event_obj);
      return 0;
    }
  }
  /* kill this event */
  GET_POINTS_EVENT(ch, type) = NULL;
  //FREE(event_obj);
  return 0;
}


/*
 * subtracts amount of hitpoints from ch's current and starts points event
 */
int alter_hit(struct char_data *ch, int amount, int location, char above_max)
{
  struct regen_event_obj *regen;
  long time;
  int gain, actual;
  int max = MAX(GET_HIT(ch), GET_MAX_HIT(ch));

  if (above_max)
    actual = amount;
  else
    actual = (amount > 0) ? amount : BOUNDED(GET_HIT(ch) - GET_MAX_HIT(ch), amount, 0);

  GET_HIT(ch) -= actual;

  //location damage
  switch (location) {
  case LOCATION_ALL:
    GET_HIT_WA(ch) = (GET_HIT_WA(ch) < 0) ? GET_HIT_WA(ch) :
	BOUNDED(0, GET_HIT_WA(ch) - amount / 2, max / 2);
    GET_HIT_WH(ch) = (GET_HIT_WH(ch) < 0) ? GET_HIT_WH(ch) :
	BOUNDED(0, GET_HIT_WH(ch) - amount / 4, max / 4);
    GET_HIT_SA(ch) = (GET_HIT_SA(ch) < 0) ? GET_HIT_SA(ch) :
	BOUNDED(0, GET_HIT_SA(ch) - amount / 2, max / 2);
    GET_HIT_SH(ch) = (GET_HIT_SH(ch) < 0) ? GET_HIT_SH(ch) :
	BOUNDED(0, GET_HIT_SH(ch) - amount / 4, max / 4);
    GET_HIT_TO(ch) = BOUNDED(HIT_DEAD - 1, GET_HIT_TO(ch) - amount, max);
    GET_HIT_LL(ch) = (GET_HIT_LL(ch) < 0) ? GET_HIT_LL(ch) :
	BOUNDED(0, GET_HIT_LL(ch) - amount / 2, max / 2);
    GET_HIT_LF(ch) = (GET_HIT_LF(ch) < 0) ? GET_HIT_LF(ch) :
	BOUNDED(0, GET_HIT_LF(ch) - amount / 4, max / 4);
    GET_HIT_RL(ch) = (GET_HIT_RL(ch) < 0) ? GET_HIT_RL(ch) :
	BOUNDED(0, GET_HIT_RL(ch) - amount / 2, max / 2);
    GET_HIT_RF(ch) = (GET_HIT_RF(ch) < 0) ? GET_HIT_RF(ch) :
	BOUNDED(0, GET_HIT_RF(ch) - amount / 4, max / 4);
    GET_HIT_HD(ch) = (GET_HIT_HD(ch) < 0) ? GET_HIT_HD(ch) :
	BOUNDED(0, GET_HIT_HD(ch) - amount / 4, max / 4);

    if ((amount > 0) && (GET_HIT_HD(ch) < GET_MAX_HIT(ch) / 16)) {
      SET_BIT(AFF_FLAGS(ch), AFF_BLIND);
      if (GET_HIT_HD(ch) < GET_MAX_HIT(ch) / 32)
	SET_BIT(AFF4_FLAGS(ch), AFF4_DEAF);
    }
    break;
  case LOCATION_FACE:
  case LOCATION_BRAIN:
    if (GET_HIT_HD(ch) >= 0) {
      GET_HIT_HD(ch) = BOUNDED(-10, GET_HIT_HD(ch) - amount, max / 4);
      if (GET_HIT_HD(ch) < 0)
	RemoveLimb(ch, location);
    }				//if

    if ((amount > 0) && (GET_HIT_HD(ch) < GET_MAX_HIT(ch) / 16))
      SET_BIT(AFF_FLAGS(ch), AFF_BLIND);
    break;
  case LOCATION_WEAPON_ARM:
    if (GET_HIT_WA(ch) >= 0) {
      GET_HIT_WA(ch) = BOUNDED(-1, GET_HIT_WA(ch) - amount, max / 2);
      if (GET_HIT_WA(ch) < 0)
	RemoveLimb(ch, location);
    }
//vlogf("DEBUG: GET_HIT_WA(ch) = %d, amount = %d, max = %d", GET_HIT_WA(ch), amount, max);
    break;
  case LOCATION_WEAPON_HAND:
    if (GET_HIT_WH(ch) >= 0) {
      GET_HIT_WH(ch) = BOUNDED(-1, GET_HIT_WH(ch) - amount, max / 4);
      if (GET_HIT_WH(ch) < 0)
	RemoveLimb(ch, location);
    }
    break;
  case LOCATION_SHIELD_ARM:
    if (GET_HIT_SA(ch) >= 0) {
      GET_HIT_SA(ch) = BOUNDED(-1, GET_HIT_SA(ch) - amount, max / 2);
      if (GET_HIT_SA(ch) < 0)
	RemoveLimb(ch, location);
    }				//if

    break;
  case LOCATION_SHIELD_HAND:
    if (GET_HIT_SH(ch) >= 0) {
      GET_HIT_SH(ch) = BOUNDED(-1, GET_HIT_SH(ch) - amount, max / 4);
      if (GET_HIT_SH(ch) < 0)
	RemoveLimb(ch, location);
    }
    break;
  case LOCATION_LEFT_LEG:
    if (GET_HIT_LL(ch) >= 0) {
      GET_HIT_LL(ch) = BOUNDED(-1, GET_HIT_LL(ch) - amount, max / 2);
      if (GET_HIT_LL(ch) < 0)
	RemoveLimb(ch, location);
    }				//if

    break;
  case LOCATION_RIGHT_LEG:
    if (GET_HIT_RL(ch) >= 0) {
      GET_HIT_RL(ch) = BOUNDED(-1, GET_HIT_RL(ch) - amount, max / 2);
      if (GET_HIT_RL(ch) < 0)
	RemoveLimb(ch, location);
    }				//if

    break;
  case LOCATION_LEFT_FOOT:
    if (GET_HIT_LF(ch) >= 0) {
      GET_HIT_LF(ch) = BOUNDED(-1, GET_HIT_LF(ch) - amount, max / 4);
      if (GET_HIT_LF(ch) < 0)
	RemoveLimb(ch, location);
    }				//if

    break;
  case LOCATION_RIGHT_FOOT:
    if (GET_HIT_RF(ch) >= 0) {
      GET_HIT_RF(ch) = BOUNDED(-1, GET_HIT_RF(ch) - amount, max / 4);
      if (GET_HIT_RF(ch) < 0)
	RemoveLimb(ch, location);
    }				//if

    break;
  case LOCATION_TORSO:
    GET_HIT_TO(ch) = BOUNDED(-10, GET_HIT_TO(ch) - amount, max);
    break;
  default:
    GET_HIT_TO(ch) = BOUNDED(-10, GET_HIT_TO(ch) - amount, max);
  }

  if (GET_HIT(ch) <= HIT_INCAP)
    return actual;

  if (GET_HIT(ch) < GET_MAX_HIT(ch) && !GET_POINTS_EVENT(ch, REGEN_HIT)) {
    CREATE(regen, struct regen_event_obj, 1);
    regen->ch = ch;
    regen->type = REGEN_HIT;
    gain = hit_gain(ch);
    time = PULSES_PER_MUD_HOUR / (gain ? gain : 1);
    GET_POINTS_EVENT(ch, REGEN_HIT) = event_create(points_event, regen, time);

    if (amount >= 0) {
      /* if the character gained hp, update position */
      update_pos(ch);
    }
  }
  return actual;
}


/* updates regen rates.  Use when big regen rate changes are made */
void check_regen_rates(struct char_data *ch)
{
  struct regen_event_obj *regen;
  int type, gain = 0;
  long time;

  if (GET_HIT(ch) <= HIT_INCAP)
    return;

  for (type = REGEN_HIT; type <= REGEN_VIT; type++) {	/* left in case we add */

    switch (type) {
    case REGEN_HIT:
      if (GET_HIT(ch) >= GET_MAX_HIT(ch))
	continue;
      gain = hit_gain(ch);
      break;
    case REGEN_VIT:
      if (GET_VIT(ch) >= GET_MAX_VIT(ch))
	continue;
      gain = vit_gain(ch);
      break;
    }

    time = PULSES_PER_MUD_HOUR / (gain ? gain : 1);

    if (!GET_POINTS_EVENT(ch, type) ||
	(time < event_time(GET_POINTS_EVENT(ch, type)))) {

      /* take off old event, create updated event */
      if (GET_POINTS_EVENT(ch, type))
	event_cancel(GET_POINTS_EVENT(ch, type));
      CREATE(regen, struct regen_event_obj, 1);
      regen->ch = ch;
      regen->type = type;
      GET_POINTS_EVENT(ch, type) = event_create(points_event, regen, time);
    }
  }
}
