/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <signal.h>

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"


/*   external vars  */
extern struct room_data *world;
extern struct char_data *character_list;
extern struct descriptor_data *descriptor_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct time_info_data time_info;
extern char *dirs[];
extern int rev_dir[];
extern char *activating_commands[];
extern char *coin_stringv(int amount);
extern void send_to_zone_indoor(int zone_rnum, char *messg);
extern void send_to_zone_outdoor(int zone_rnum, char *messg);
extern char perform_wear(struct char_data *ch, struct obj_data *obj, int where);
ACMD(do_hit);

/* specs.swords.c */
int CheckEgo(struct char_data *ch, struct obj_data *obj);
int CheckGetEgo(struct char_data *ch, struct obj_data *obj);
int CheckEgoGive(struct char_data *ch, struct char_data *vict, struct obj_data *obj);
int ItemEgoClash(struct char_data *ch, struct obj_data *obj, int bon);
int EgoBladeSave(struct char_data *ch);
int EvilBlade(struct char_data *ch, int cmd, char *arg, struct obj_data *tobj, int type);
int GoodBlade(struct char_data *ch, int cmd, char *arg, struct obj_data *tobj, int type);
int NeutralBlade(struct char_data *ch, int cmd, char *arg, struct obj_data *tobj, int type);


#define EGO

#ifdef EGO
int CheckEgo(struct char_data *ch, struct obj_data *obj)
{
  int j = 0;

  if (!obj || !ch) {
    log("!obj || !ch in CheckEgo, utility.c");
    return FALSE;
  }
  /*
     if the item is limited, check its ego.
     use some funky function to determine if pc's ego is higher than objs'
     ego.. if it is, proceed.. otherwise, deny.
   */
  j = ItemEgoClash(ch, obj, 0);
  if ((j < -5) && (GET_LEVEL(ch) < LVL_IMPL)) {
    act("$p almost seems to say 'You're much too puny to use me, twerp!'", 0,
	ch, obj, 0, TO_CHAR);

    if (obj->in_obj == NULL) {
      act("$p falls to the floor", 0, ch, obj, 0, TO_CHAR);
      act("$p removes itself from $n, and falls to the floor", 0, ch, obj, 0, TO_ROOM);
    }
    return FALSE;
  } else if ((j < 0) && (GET_LEVEL(ch) < LVL_IMPL)) {
    act("$p almost seems to say 'You're pretty puny.  I don't want to be seen with you!\n", 0, ch, obj, 0, TO_CHAR);

    if (obj->in_obj == NULL) {
      act("$p falls to the floor", 0, ch, obj, 0, TO_CHAR);
      act("$p removes itself from $n, and falls to the floor", 0, ch, obj, 0, TO_ROOM);
    }
    return FALSE;
  } else
    return TRUE;

  return FALSE;

}

int CheckGetEgo(struct char_data *ch, struct obj_data *obj)
{
  return TRUE;
}

int CheckEgoGive(struct char_data *ch, struct char_data *vict,
		 struct obj_data *obj)
{
  int j = 0;

  /*
     if the item is limited, check its ego.
     use some funky function to determine if pc's ego is higher than objs'
     ego.. if it is, proceed.. otherwise, deny.
   */
  j = ItemEgoClash(vict, obj, 0);
  if ((j < -5) && (GET_LEVEL(vict) < LVL_IMPL)) {
    if (AWAKE(vict)) {
      act("$p almost seems to say 'You're much too puny to use me, twerp!'", 0,
	  vict, obj, 0, TO_CHAR);
      act("$p refuses to leave $N's hands", 0, vict, obj, ch, TO_CHAR);
    }
    act("$p refuses to be given to $n by $N ", 0, vict, obj, ch, TO_ROOM);
    return FALSE;
  } else if ((j < 0) && (GET_LEVEL(vict) < LVL_IMPL)) {
    if (AWAKE(vict)) {
      act("$p almost seems to say 'You're pretty puny.  I don't want to be seen with you!\n", 0, vict, obj, 0, TO_CHAR);
      act("$p refuses to leave $N's hands", 0, vict, obj, ch, TO_CHAR);
    }
    act("$p refuses to be given to $n by $N.", 0, vict, obj, ch, TO_ROOM);
    return FALSE;
  } else
    return TRUE;

  return FALSE;
}


int ItemEgoClash(struct char_data *ch, struct obj_data *obj, int bon)
{
  int obj_ego, p_ego, tmp;

  obj_ego = GET_OBJ_COST(obj);

  if (strstr(obj->name, "scroll") || strstr(obj->name, "potion") ||
      strstr(obj->name, "bag") ||
      strstr(obj->name, "key")) {
    return (0);
  }
  if (obj_ego >= 14000 || obj_ego < 0) {

    if (obj_ego < 0)
      obj_ego = 50000;

    obj_ego /= 666;

/*    
   alignment stuff
 */

    if (IS_OBJ_STAT(obj, ITEM_ANTI_GOOD) ||
	(IS_OBJ_STAT(obj, ITEM_ANTI_EVIL))) {
      if (IS_NEUTRAL(ch))
	obj_ego += obj_ego / 4;
    }
    if (IS_PC(ch)) {
      p_ego = GET_LEVEL(ch) /*+ HowManyClasses(ch) */ ;

      if (p_ego > 40) {
	p_ego *= (p_ego - 39);
      } else if (p_ego > 20) {
	p_ego += (p_ego - 20);
      }
    } else {
      p_ego = 10000;
    }

    tmp = GET_INT(ch) + GET_WIS(ch) + GET_CHA(ch);
    tmp /= 3;


    tmp *= GET_HIT(ch);
    tmp /= GET_MAX_HIT(ch);


    return ((p_ego + tmp + bon + number(1, 6)) - (obj_ego + number(1, 6)));
  }
  return (1);
}


int EgoBladeSave(struct char_data *ch)
{
  int total;

  if (GET_LEVEL(ch) <= 10)
    return FALSE;
  total = (GET_LEVEL(ch) + GET_STR(ch) + GET_CON(ch));
  if (GET_HIT(ch) == 0)
    return FALSE;
  total = total - (GET_MAX_HIT(ch) / GET_HIT(ch));
  if (number(1, 101) > total) {
    return FALSE;
  } else
    return TRUE;
}


int EvilBlade(struct char_data *ch, int cmd, char *arg, struct obj_data *tobj, int type)
{
  extern struct str_app_type str_app[];
  struct obj_data *obj, *blade;
  struct char_data *joe, *holder;
  struct char_data *lowjoe = 0;
  char arg1[128], buf[250];

  if (IS_TRUSTED(ch))
    return FALSE;

  for (obj = world[ch->in_room].contents;
       obj; obj = obj->next_content) {
    if (obj == tobj) {
      /* I am on the floor */
      for (joe = world[ch->in_room].people; joe;
	   joe = joe->next_in_room) {
	if ((GET_ALIGNMENT(joe) <= -400) && (!IS_TRUSTED(joe))) {
	  if (lowjoe) {
	    if (GET_ALIGNMENT(joe) < GET_ALIGNMENT(lowjoe)) {
	      lowjoe = joe;
	    }
	  } else
	    lowjoe = joe;
	}
      }
      if (lowjoe) {
	if (CAN_GET_OBJ(lowjoe, obj)) {
	  obj_from_room(obj);
	  obj_to_char(obj, lowjoe);

	  sprintf(buf, "%s leaps into your hands!\n\r", obj->short_description);
	  send_to_char(buf, lowjoe);

	  sprintf(buf, "%s jumps from the floor and leaps into %s's hands!\n\r",
		  obj->short_description, GET_NAME(lowjoe));
	  act(buf, FALSE, lowjoe, 0, 0, TO_ROOM);

	  if (!EgoBladeSave(lowjoe)) {
	    if (!lowjoe->equipment[WEAR_WIELD]) {
	      sprintf(buf, "%s forces you to wield it!\n\r", obj->short_description);
	      send_to_char(buf, lowjoe);
	      perform_wear(lowjoe, obj, 12);
	      return FALSE;
	    } else {
	      sprintf(buf, "You can feel %s attept to make you wield it.\n\r",
		      obj->short_description);
	      send_to_char(buf, lowjoe);
	      return FALSE;
	    }
	  }
	}
      }
    }
  }
  for (holder = world[ch->in_room].people; holder;
       holder = holder->next_in_room) {
    for (obj = holder->carrying; obj; obj = obj->next_content) {
      if (GET_OBJ_TYPE(obj) != ITEM_BOARD) {	/* same as checking that its not a board proc */
	/*held */
	if (holder->equipment[WEAR_WIELD]) {
	  if ((!EgoBladeSave(holder)) && (!EgoBladeSave(holder))) {
	    sprintf(buf, "%s gets pissed off that you are wielding another weapon!\n\r",
		    obj->short_description);
	    send_to_char(buf, holder);
	    sprintf(buf, "%s knocks %s out of your hands!!\n\r", obj->short_description,
		    holder->equipment[WEAR_WIELD]->short_description);
	    send_to_char(buf, holder);
	    blade = unequip_char(holder, WEAR_WIELD);
	    if (blade)
	      obj_to_room(blade, holder->in_room);
	    if (!holder->equipment[WEAR_WIELD]) {
	      sprintf(buf, "%s forces you yto wield it!\n\r",
		      obj->short_description);
	      send_to_char(buf, holder);
	      perform_wear(holder, obj, 12);
	      return FALSE;
	    }
	  }
	}
	if (!EgoBladeSave(holder)) {
	  if (!EgoBladeSave(holder)) {
	    if (!holder->equipment[WEAR_WIELD]) {
	      sprintf(buf, "%s forces you yto wield it!\n\r",
		      obj->short_description);
	      send_to_char(buf, holder);
	      perform_wear(holder, obj, 12);
	      return FALSE;
	    }
	  }
	}
	if (affected_by_spell(holder, SPELL_CHARM_PERSON)) {
	  affect_from_char(holder, SPELL_CHARM_PERSON);
	  sprintf(buf, "Due to the %s, you feel less enthused about your master.\n\r",
		  obj->short_description);
	  send_to_char(buf, holder);
	}
      }
    }
    if (holder->equipment[WEAR_WIELD]) {
      if (GET_OBJ_TYPE(holder->equipment[WEAR_WIELD]) != ITEM_BOARD) {	/* same as checking that its not a board proc */
	/*YES! I am being held! */
	obj = holder->equipment[WEAR_WIELD];
	if (affected_by_spell(holder, SPELL_CHARM_PERSON)) {
	  affect_from_char(holder, SPELL_CHARM_PERSON);
	  sprintf(buf, "Due to the %s, you feel less enthused about your master.\n\r",
		  obj->short_description);
	  send_to_char(buf, holder);
	}
	if (FIGHTING(holder)) {
	  sprintf(buf, "%s almost sings in your hand!\n\r",
		  obj->short_description);
	  send_to_char(buf, holder);
	  sprintf(buf, "You can hear $n's %s almost sing with joy!", obj->short_description);
	  act(buf, FALSE, holder, 0, 0, TO_ROOM);
	  if ((holder == ch) && (cmd == 151)) {
	    if (EgoBladeSave(ch) && EgoBladeSave(ch)) {
	      sprintf(buf, "You can feel %s attempt to stay in the fight!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      return FALSE;
	    } else {
	      sprintf(buf, "%s laughs at your attempt to flee from a fight!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      sprintf(buf, "%s gives you a little warning...\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      sprintf(buf, "%s twists around and smacks you!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      sprintf(buf, "Wow! $n's %s just whipped around and smacked $m one!",
		      obj->short_description);
	      act(buf, FALSE, ch, 0, 0, TO_ROOM);
	      GET_HIT(ch) -= 25;
	      if (GET_HIT(ch) < 0) {
		GET_HIT(ch) = 0;
		SET_POS(ch, POS_PRONE + STAT_RESTING);
	      }
	      return TRUE;
	    }
	  }
	}
	if ((cmd == 66) && (holder == ch)) {
	  one_argument(arg, arg1);
	  if (strcmp(arg1, "all") == 0) {
	    if (!EgoBladeSave(ch)) {

	      sprintf(buf, "%s laughs at your attempt remove it!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      sprintf(buf, "%s gives you a little warning...\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      sprintf(buf, "%s twists around and smacks you!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      sprintf(buf, "Wow! $n's %s just whipped around and smacked $m one!",
		      obj->short_description);
	      act(buf, FALSE, ch, 0, 0, TO_ROOM);

	      GET_HIT(ch) -= 25;
	      if (GET_HIT(ch) < 0) {
		GET_HIT(ch) = 0;
		SET_POS(ch, POS_PRONE + STAT_RESTING);
	      }
	      return TRUE;
	    } else {
	      sprintf(buf, "You can feel %s attempt to stay wielded!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      return FALSE;
	    }
	  } else {
	    if (isname(arg1, obj->name)) {
	      if (!EgoBladeSave(ch)) {

		sprintf(buf, "%s laughs at your attempt to remove it!\n\r",
			obj->short_description);
		send_to_char(buf, ch);
		sprintf(buf, "%s gives you a little warning...\n\r",
			obj->short_description);
		send_to_char(buf, ch);
		sprintf(buf, "%s twists around and smacks you!\n\r",
			obj->short_description);
		send_to_char(buf, ch);
		sprintf(buf, "Wow! $n's %s just whipped around and smacked $m one!",
			obj->short_description);
		act(buf, FALSE, ch, 0, 0, TO_ROOM);

		GET_HIT(ch) -= 25;
		if (GET_HIT(ch) < 0) {
		  GET_HIT(ch) = 0;
		  SET_POS(ch, POS_PRONE + STAT_RESTING);
		}
		return TRUE;
	      } else {
		sprintf(buf, "You can feel %s attempt to stay wielded!\n\r", obj->short_description);
		send_to_char(buf, ch);
		return FALSE;
	      }
	    }
	  }
	}
	for (joe = world[holder->in_room].people; joe;
	     joe = joe->next_in_room) {
	  if ((GET_ALIGNMENT(joe) >= 500) &&
	    (IS_MOB(joe)) && (CAN_SEE(holder, joe)) && (holder != joe)) {
	    if (lowjoe) {
	      if (GET_ALIGNMENT(joe) > GET_ALIGNMENT(lowjoe)) {
		lowjoe = joe;
	      }
	    } else
	      lowjoe = joe;
	  }
	}
	if (lowjoe) {
	  if (!EgoBladeSave(holder)) {
	    if (GET_POS(holder) != POS_STANDING) {
	      sprintf(buf, "%s yanks you yo your feet!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      SET_POS(holder, POS_STANDING + STAT_NORMAL);
	    }
	    sprintf(buf, "%s leaps out of control!!\n\r",
		    obj->short_description);
	    send_to_char(buf, holder);
	    sprintf(buf, "%s jumps for $n's neck!", obj->short_description);
	    act(buf, FALSE, lowjoe, 0, 0, TO_ROOM);
	    do_hit(holder, lowjoe->player.name, 0, 0);
	    return TRUE;
	  } else {
	    return FALSE;
	  }
	}
	if ((cmd == 70) && (holder == ch)) {
	  sprintf(buf, "%s almost sings in your hands!!\n\r",
		  obj->short_description);
	  send_to_char(buf, ch);
	  sprintf(buf, "You can hear $n's %s almost sing with joy!", obj->short_description);

	  act(buf, FALSE, ch, 0, 0, TO_ROOM);
	  return FALSE;
	}
      }
    }
  }
  return FALSE;
}

int GoodBlade(struct char_data *ch, int cmd, char *arg, struct obj_data *tobj, int type)
{
  extern struct str_app_type str_app[];
  struct obj_data *obj, *blade;
  struct char_data *joe, *holder;
  struct char_data *lowjoe = 0;
  char arg1[128], buf[250];

  if (IS_TRUSTED(ch))
    return FALSE;

  for (obj = world[ch->in_room].contents;
       obj; obj = obj->next_content) {
    if (obj == tobj) {
      /* I am on the floor */
      for (joe = world[ch->in_room].people; joe;
	   joe = joe->next_in_room) {
	if ((GET_ALIGNMENT(joe) >= 350) && (!IS_TRUSTED(joe))) {
	  if (lowjoe) {
	    if (GET_ALIGNMENT(joe) > GET_ALIGNMENT(lowjoe)) {
	      lowjoe = joe;
	    }
	  } else
	    lowjoe = joe;
	}
      }
      if (lowjoe) {
	if (CAN_GET_OBJ(lowjoe, obj)) {
	  obj_from_room(obj);
	  obj_to_char(obj, lowjoe);

	  sprintf(buf, "%s leaps into your hands!\n\r", obj->short_description);
	  send_to_char(buf, lowjoe);

	  sprintf(buf, "%s jumps from the floor and leaps into %s's hands!\n\r",
		  obj->short_description, GET_NAME(lowjoe));
	  act(buf, FALSE, lowjoe, 0, 0, TO_ROOM);

	  if (!EgoBladeSave(lowjoe)) {
	    if (!lowjoe->equipment[WEAR_WIELD]) {
	      sprintf(buf, "%s forces you to wield it!\n\r", obj->short_description);
	      send_to_char(buf, lowjoe);
	      perform_wear(lowjoe, obj, 12);
	      return FALSE;
	    } else {
	      sprintf(buf, "You can feel %s attept to make you wield it.\n\r",
		      obj->short_description);
	      send_to_char(buf, lowjoe);
	      return FALSE;
	    }
	  }
	}
      }
    }
  }
  for (holder = world[ch->in_room].people; holder;
       holder = holder->next_in_room) {
    for (obj = holder->carrying; obj; obj = obj->next_content) {
      if (GET_OBJ_TYPE(obj) != ITEM_BOARD) {	/* same as checking that its not a board proc */
	/*held */
	if (holder->equipment[WEAR_WIELD]) {
	  if ((!EgoBladeSave(holder)) && (!EgoBladeSave(holder))) {
	    sprintf(buf, "%s gets pissed off that you are wielding another weapon!\n\r",
		    obj->short_description);
	    send_to_char(buf, holder);
	    sprintf(buf, "%s knocks %s out of your hands!!\n\r", obj->short_description,
		    holder->equipment[WEAR_WIELD]->short_description);
	    send_to_char(buf, holder);
	    blade = unequip_char(holder, WEAR_WIELD);
	    if (blade)
	      obj_to_room(blade, holder->in_room);
	    if (!holder->equipment[WEAR_WIELD]) {
	      sprintf(buf, "%s forces you to wield it!\n\r",
		      obj->short_description);
	      send_to_char(buf, holder);
	      perform_wear(holder, obj, 12);
	      return FALSE;
	    }
	  }
	}
	if (!EgoBladeSave(holder)) {
	  if (!EgoBladeSave(holder)) {
	    if (!holder->equipment[WEAR_WIELD]) {
	      sprintf(buf, "%s forces you yto wield it!\n\r",
		      obj->short_description);
	      send_to_char(buf, holder);
	      perform_wear(holder, obj, 12);
	      return FALSE;
	    }
	  }
	}
	if (affected_by_spell(holder, SPELL_CHARM_PERSON)) {
	  affect_from_char(holder, SPELL_CHARM_PERSON);
	  sprintf(buf, "Due to the effects of %s, you feel less enthused about your master.\n\r",
		  obj->short_description);
	  send_to_char(buf, holder);
	}
      }
    }
    if (holder->equipment[WEAR_WIELD]) {
      if (GET_OBJ_TYPE(holder->equipment[WEAR_WIELD]) != ITEM_BOARD) {	/* same as checking that its not a board proc */
	/*YES! I am being held! */
	obj = holder->equipment[WEAR_WIELD];

	/* remove charm */
	if (affected_by_spell(holder, SPELL_CHARM_PERSON)) {
	  affect_from_char(holder, SPELL_CHARM_PERSON);
	  sprintf(buf, "Due to the effects of %s, you feel less enthused about your master.\n\r",
		  obj->short_description);
	  send_to_char(buf, holder);
	}
	/* remove blindness */
	if (affected_by_spell(holder, SPELL_BLINDNESS)) {
	  affect_from_char(holder, SPELL_BLINDNESS);
	  sprintf(buf, "%s hums in your hands, you can see!\n\r",
		  obj->short_description);
	  send_to_char(buf, holder);
	}
	/* remove chill touch */
	if (affected_by_spell(holder, SPELL_CHILL_TOUCH)) {
	  affect_from_char(holder, SPELL_CHILL_TOUCH);
	  sprintf(buf, "%s hums in your hands, you feel warm again!\n\r",
		  obj->short_description);
	  send_to_char(buf, holder);
	}
	/* remove slow */
	if (affected_by_spell(holder, SPELL_SLOW)) {
	  affect_from_char(holder, SPELL_SLOW);
	  sprintf(buf, "%s hums in your hands, you feel yourself speed back up!\n\r",
		  obj->short_description);
	  send_to_char(buf, holder);
	}
	/* remove poison */
	if (affected_by_spell(holder, SPELL_POISON)) {
	  affect_from_char(holder, SPELL_POISON);
	  sprintf(buf, "%s hums in your hands, the sick feeling fades!\n\r",
		  obj->short_description);
	  send_to_char(buf, holder);
	}
	/* wielder is hurt, heal them */
	if ((number(1, 101) > 90) && GET_HIT(holder) < GET_MAX_HIT(holder) / 2) {
	  act("You get a gentle warm pulse from $p, you feel MUCH better!", FALSE, holder, obj, 0, TO_CHAR);
	  act("$n smiles as $p pulses in $s hands!", FALSE, holder, obj, 0, TO_ROOM);
	  GET_HIT(holder) = (GET_MAX_HIT(holder) - number(1, 10));
	  return FALSE;
	}
	if (FIGHTING(holder)) {
	  sprintf(buf, "%s almost sings in your hand!\n\r",
		  obj->short_description);
	  send_to_char(buf, holder);
	  sprintf(buf, "You can hear %s almost sing with joy in $n's hands!", obj->short_description);
	  act(buf, FALSE, holder, 0, 0, TO_ROOM);
	  if ((holder == ch) && (cmd == 151)) {
	    if (EgoBladeSave(ch) && EgoBladeSave(ch)) {
	      sprintf(buf, "You can feel %s attempt to stay in the fight!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      return FALSE;
	    } else {
	      sprintf(buf, "%s laughs at your attempt to flee from a fight!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      sprintf(buf, "%s gives you a little warning...\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      sprintf(buf, "%s twists around and smacks you!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      sprintf(buf, "Wow! $n's %s just whipped around and smacked $m one!",
		      obj->short_description);
	      act(buf, FALSE, ch, 0, 0, TO_ROOM);
	      GET_HIT(ch) -= 10;
	      if (GET_HIT(ch) < 0) {
		GET_HIT(ch) = 0;
		SET_POS(ch, POS_PRONE + STAT_RESTING);
	      }
	      return TRUE;
	    }
	  }
	}
	if ((cmd == 66) && (holder == ch)) {
	  one_argument(arg, arg1);
	  if (strcmp(arg1, "all") == 0) {
	    if (!EgoBladeSave(ch)) {

	      sprintf(buf, "%s laughs at your attempt remove it!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      sprintf(buf, "%s gives you a little warning...\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      sprintf(buf, "%s twists around and smacks you!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      sprintf(buf, "Wow! $n's %s just whipped around and smacked $m one!",
		      obj->short_description);
	      act(buf, FALSE, ch, 0, 0, TO_ROOM);

	      GET_HIT(ch) -= 10;
	      if (GET_HIT(ch) < 0) {
		GET_HIT(ch) = 0;
		SET_POS(ch, POS_PRONE + STAT_RESTING);
	      }
	      return TRUE;
	    } else {
	      sprintf(buf, "You can feel %s attempt to stay wielded!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      return FALSE;
	    }
	  } else {
	    if (isname(arg1, obj->name)) {
	      if (!EgoBladeSave(ch)) {

		sprintf(buf, "%s laughs at your attempt to remove it!\n\r",
			obj->short_description);
		send_to_char(buf, ch);
		sprintf(buf, "%s gives you a little warning...\n\r",
			obj->short_description);
		send_to_char(buf, ch);
		sprintf(buf, "%s twists around and smacks you!\n\r",
			obj->short_description);
		send_to_char(buf, ch);
		sprintf(buf, "Wow! $n's %s just whipped around and smacked $m one!",
			obj->short_description);
		act(buf, FALSE, ch, 0, 0, TO_ROOM);

		GET_HIT(ch) -= 10;
		if (GET_HIT(ch) < 0) {
		  GET_HIT(ch) = 0;
		  SET_POS(ch, POS_PRONE + STAT_RESTING);
		}
		return TRUE;
	      } else {
		sprintf(buf, "You can feel %s attempt to stay wielded!\n\r", obj->short_description);
		send_to_char(buf, ch);
		return FALSE;
	      }
	    }
	  }
	}
	for (joe = world[holder->in_room].people; joe;
	     joe = joe->next_in_room) {
	  if ((GET_ALIGNMENT(joe) <= -350) &&
	    (IS_MOB(joe)) && (CAN_SEE(holder, joe)) && (holder != joe)) {
	    if (lowjoe) {
	      if (GET_ALIGNMENT(joe) < GET_ALIGNMENT(lowjoe)) {
		lowjoe = joe;
	      }
	    } else
	      lowjoe = joe;
	  }
	}
	if (lowjoe) {
	  if (!EgoBladeSave(holder)) {
	    if (GET_POS(holder) != POS_STANDING) {
	      sprintf(buf, "%s yanks you yo your feet!\n\r",
		      obj->short_description);
	      send_to_char(buf, ch);
	      SET_POS(holder, POS_STANDING + STAT_NORMAL);
	    }
	    sprintf(buf, "%s leaps out of control!!\n\r",
		    obj->short_description);
	    send_to_char(buf, holder);
	    sprintf(buf, "%s howls out for $n's neck!", obj->short_description);
	    act(buf, FALSE, lowjoe, 0, 0, TO_ROOM);
	    do_hit(holder, lowjoe->player.name, 0, 0);
	    return TRUE;
	  } else {
	    return FALSE;
	  }
	}
	if ((cmd == 70) && (holder == ch)) {
	  sprintf(buf, "%s almost sings in your hands!!\n\r",
		  obj->short_description);
	  send_to_char(buf, ch);
	  sprintf(buf, "You can hear $n's %s almost sing with joy!", obj->short_description);

	  act(buf, FALSE, ch, 0, 0, TO_ROOM);
	  return FALSE;
	}
      }
    }
  }

  return FALSE;
}

int NeutralBlade(struct char_data *ch, int cmd, char *arg, struct obj_data *tobj, int type)
{
  return FALSE;
}

#endif
