/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"
#include "db.h"
#include "comm.h"
#include "handler.h"
#include "events.h"
#include "spells.h"
#include "interpreter.h"
#include "skills.h"
#include "olc.h"

#define MAGICBOOKoff		// books written in tongue_magic
#define SCHOOLS_ONoff		// check school/sphere of player
#define SPELL_DEBUGoff		// additional info, tells whats going on

extern struct spell_info_type spell_info[TOP_SPELL_DEFINE + 1];
extern struct room_data *world;
extern const char *spells[];
extern char *class_types[];
struct obj_data *FindSpellBookWithSpell(struct char_data *ch, int spl, int mode);
void display_mob_spells(struct char_data *target, struct char_data *ch);


struct memorizing_event {
  struct char_data *ch;
};

struct scribing_event {
  struct char_data *ch;
  struct scribing_data_type *s;
};

EVENTFUNC(memorizing_event);
EVENTFUNC(scribing_event);
ACMD(do_scribe);
ACMD(do_memorize);
ACMD(do_teach);

struct spell_data {
  sh_int spell;
  int ticks;
  char pray;
  struct spell_data *next;
};

bool
IsMemmingClass(struct char_data *ch)
{

  if (GET_CLASS(ch) == CLASS_MAGIC_USER
      || GET_CLASS(ch) == CLASS_MULTI_C_M
      || GET_CLASS(ch) == CLASS_MULTI_C_I
      || GET_CLASS(ch) == CLASS_MULTI_C_F_M
      || GET_CLASS(ch) == CLASS_MULTI_I_T
      || GET_CLASS(ch) == CLASS_MULTI_F_M
      || GET_CLASS(ch) == CLASS_MULTI_M_T
      || GET_CLASS(ch) == CLASS_MULTI_F_M_T
      || GET_CLASS(ch) == CLASS_ILLUSIONIST
      || GET_CLASS(ch) == CLASS_MULTI_F_I
      || IS_TRUSTED(ch))
    return TRUE;
  return FALSE;
}


bool
IsPrayingClass(struct char_data * ch)
{

  if (GET_CLASS(ch) == CLASS_CLERIC
      || GET_CLASS(ch) == CLASS_MULTI_C_M
      || GET_CLASS(ch) == CLASS_MULTI_C_I
      || GET_CLASS(ch) == CLASS_MULTI_C_F_M
      || GET_CLASS(ch) == CLASS_MULTI_C_R
      || GET_CLASS(ch) == CLASS_RANGER
      || GET_CLASS(ch) == CLASS_DRUID
      || GET_CLASS(ch) == CLASS_PALADIN
      || GET_CLASS(ch) == CLASS_MULTI_C_F
      || GET_CLASS(ch) == CLASS_MULTI_C_T
      || IS_TRUSTED(ch))
    return TRUE;
  return FALSE;
}

/*
   ok, the spell system is one _REAL_ kludge, but in advanced way.
   reasoning:
   keyword     = just way to see it's genuine spellbook data entry
   description = real mem_hogger thing - it is string of size
   (MAX_SPELLS+1) with entry for each spell.
   1=no spell, 2 = spell, 0 = end of book
 */

struct extra_descr_data *
 find_spell_description(struct obj_data *obj)
{
  struct extra_descr_data *tmp;

  for (tmp = obj->ex_description; tmp; tmp = tmp->next)
    if (tmp->keyword && tmp->description)
      if ((tmp->keyword[0] == 3) && (tmp->keyword[1] == 1) &&
	  (tmp->keyword[2] == 3))
	return tmp;
  return NULL;
}

bool SpellInThisSpellBook_p(struct extra_descr_data * tmp, int spl)
{
  if (IS_SET(tmp->description[spl / 8], 1 << (spl % 8)))
    return TRUE;
  return FALSE;
}

bool SpellInThisSpellBook(struct extra_descr_data * tmp, int spl)
{
  if (SpellInThisSpellBook_p(tmp, spl))
    return TRUE;
  return FALSE;
}

bool SpellInSpellBook(struct char_data * ch, int spl, int mode)
{
  if (!(FindSpellBookWithSpell(ch, spl, mode)))
    return FALSE;
  return TRUE;
}

struct obj_data *SpellBookAtHand(struct char_data *ch)
{
  struct obj_data *obj1, *obj2;

  obj1 = ch->equipment[WEAR_HOLD] ? ch->equipment[WEAR_HOLD] : NULL;
  obj2 = ch->equipment[WEAR_HOLD2] ? ch->equipment[WEAR_HOLD2] : NULL;
  if (obj1 && (GET_OBJ_TYPE(obj1) == ITEM_BOOK || GET_OBJ_TYPE(obj1) == ITEM_SCROLL) && CAN_SEE_OBJ(ch, obj1))
    return obj1;
  if (obj2 && (GET_OBJ_TYPE(obj2) == ITEM_BOOK || GET_OBJ_TYPE(obj1) == ITEM_SCROLL) && CAN_SEE_OBJ(ch, obj2))
    return obj2;

  return NULL;
}

bool
CanMemorizeSpell(struct char_data * ch, int spellnum)
{
  if (spellnum > 0 && spellnum <= LAST_SPELL) {
    if (IsMemmingClass(ch) && !SpellInSpellBook(ch, spellnum, SBOOK_MODE_AT_HAND + SBOOK_MODE_IN_INV + SBOOK_MODE_NO_SCROLL))
      return FALSE;
    if (GET_SPELL(ch, spellnum) >= MEMINFO_KNOWN)
      return TRUE;
  }
  return FALSE;
}

struct obj_data *Find_process_entry(struct char_data *ch, struct obj_data *obj, int spl)
{
  struct extra_descr_data *desc;
  int i, b, c;

  if (GET_OBJ_TYPE(obj) == ITEM_BOOK) {
    if ((desc = find_spell_description(obj))) {
      b = GET_OBJ_VAL(obj, 0);
      c = GET_OBJ_VAL(obj, 1);
      if ((c == 0) || (c >= NUM_CLASSES))
	return NULL;
#ifdef MAGICBOOK
      if ((b <= TONGUE_USING) || (b > TONGUE_LASTHEARD))
	return NULL;
#endif
      if (SpellInThisSpellBook(desc, spl))
	return obj;
    }
  } else if (GET_OBJ_TYPE(obj) == ITEM_SCROLL) {
#ifdef SPELL_DEBUG
    send_to_char("&+CDEBUG:&n Searching scroll for the spell now.\r\n", ch);
#endif
    for (i = 1; i < 8; i++)
      if ((GET_OBJ_VAL(obj, i) >= 0) && (GET_OBJ_VAL(obj, i) == spl))
	return obj;
  }
  return NULL;
}

struct obj_data *FindSpellBookWithSpell(struct char_data *ch, int spl, int mode)
{
  struct obj_data *obj, *obj2;

  /*
     current format: 
     value[0] = language spell is in 
     value[1] = class the spellbook is for 
     value[2] = number of pages in spellbook 
     value[3] = number of pages _used_
     extra desc with keyword '\03\01\03\00' contains all the neccessary spells.
   */

  if (IS_SET(mode, SBOOK_MODE_AT_HAND))
    if (!IS_SET(mode, SBOOK_MODE_NO_BOOK))
      if ((obj2 = SpellBookAtHand(ch)))
	if ((obj = Find_process_entry(ch, obj2, spl)))
	  return obj;
  if (IS_SET(mode, SBOOK_MODE_IN_INV)) {
    for (obj = ch->carrying; obj; obj = obj->next_content) {
      if ((GET_OBJ_TYPE(obj) == ITEM_BOOK) && CAN_SEE_OBJ(ch, obj)) {
	if (!IS_SET(mode, SBOOK_MODE_NO_BOOK)) {
	  if ((obj2 = Find_process_entry(ch, obj, spl)))
	    return obj2;
	}
      } else if ((GET_OBJ_TYPE(obj) == ITEM_SCROLL) && CAN_SEE_OBJ(ch, obj) && !IS_SET(mode, SBOOK_MODE_NO_SCROLL))
	if ((obj2 = Find_process_entry(ch, obj, spl)))
	  return obj2;
    }
  }
  return NULL;
}

byte ListSpellsInSpellBook(struct char_data * ch, struct obj_data * obj)
{
  int counter, columns = 0, i;
  struct extra_descr_data *tmp;
  char tempbuf[MAX_INPUT_LENGTH];

  *tempbuf = '\0';
  *buf = '\0';

  if (GET_OBJ_TYPE(obj) == ITEM_SCROLL) {
    strcpy(buf, "This scroll appears to contain the spells of:\r\n");
    for (i = 1; i < 8; i++)
      if ((GET_OBJ_VAL(obj, i) > 0))
	sprintf(buf + strlen(buf), "%s\r\n", spells[GET_OBJ_VAL(obj, i)]);
    if (buf)
      send_to_char(buf, ch);
    else
      send_to_char("Hmm, the scroll is empty!\r\n", ch);
    return 1;
  }
  /* on to true books */
  if (!(tmp = find_spell_description(obj)))
    return 0;

  get_char_cols(ch);

  for (counter = 1; counter < NUM_SPELLS; counter++) {
    if (SpellInThisSpellBook(tmp, counter)) {
      sprintf(buf + strlen(buf), "%s%2d%s) %s%-20.20s %s", grn, ++columns,
	    nrm, blu, spells[counter], !(columns % 3) ? "\r\n&n" : "&n");
    }
  }
  send_to_char(buf, ch);
  return 1;
}

int AddSpellToSpellBook(struct char_data *ch, struct obj_data *obj, int spl)
{
  struct extra_descr_data *tmp;

  if (!(tmp = find_spell_description(obj))) {
    /* creation blues */
    CREATE(tmp, struct extra_descr_data, 1);

    tmp->next = obj->ex_description;
    obj->ex_description = tmp;
    sprintf(buf, "%c%c%c", (char) 3, (char) 1, (char) 3);
    tmp->keyword = str_dup(buf);
    CREATE(tmp->description, char, (MAX_SPELLS + 1) / 8 + 1);
  }
  if (SpellInThisSpellBook(tmp, spl))
    return FALSE;
  if (!GET_OBJ_VAL(obj, 1) && ch) {
    if (IS_MAGE_SPELL(spl) && IS_MAGE(ch))
      GET_OBJ_VAL(obj, 1) = CLASS_MAGIC_USER;
    else
      GET_OBJ_VAL(obj, 1) = GET_CLASS(ch);
    GET_OBJ_OWNER(obj) = GET_IDNUM(ch);
  } else if (!GET_OBJ_VAL(obj, 1))
    GET_OBJ_VAL(obj, 1) = CLASS_MAGIC_USER;

#ifdef MAGICBOOK
  if (!GET_OBJ_VAL(obj, 0))
    GET_OBJ_VAL(obj, 0) = TONGUE_MAGIC;
#endif
  /* not there, we've gotta add it */
  SET_BIT(tmp->description[spl / 8], 1 << (spl % 8));

  /* ok, we done, we happy campers.  */
  return TRUE;
}

int RemoveSpellFromSpellBook(struct char_data *ch, struct obj_data *obj, int spl)
{
  struct extra_descr_data *tmp;

  if (!(tmp = find_spell_description(obj))) {
    send_to_char("No spells in this book.\r\n", ch);
    return 0;
  }
  if (!SpellInThisSpellBook(tmp, spl))
    return FALSE;

#ifdef MAGICBOOK
  if (!GET_OBJ_VAL(obj, 0))
    GET_OBJ_VAL(obj, 0) = TONGUE_MAGIC;
#endif

  /* there, we've gotta remove it */
  REMOVE_BIT(tmp->description[spl / 8], 1 << (spl % 8));

  /* ok, we done, we happy campers.  */
  return TRUE;
}

/***** Memorizing functions *****/

const int MageSpellsPerLevel[34][9] =
{
  {1, 0, 0, 0, 0, 0, 0, 0, 0},	/* level 1 */
  {2, 0, 0, 0, 0, 0, 0, 0, 0},	/* level 2 */
  {2, 1, 0, 0, 0, 0, 0, 0, 0},	/* level 3 */
  {3, 2, 0, 0, 0, 0, 0, 0, 0},	/* level 4 */
  {4, 2, 1, 0, 0, 0, 0, 0, 0},	/* level 5 */
  {4, 2, 2, 0, 0, 0, 0, 0, 0},	/* level 6 */
  {4, 3, 2, 1, 0, 0, 0, 0, 0},	/* level 7 */
  {4, 3, 3, 2, 0, 0, 0, 0, 0},	/* level 8 */
  {4, 3, 3, 2, 1, 0, 0, 0, 0},	/* level 9 */
  {4, 4, 3, 2, 2, 0, 0, 0, 0},	/* level 10 */
  {4, 4, 4, 3, 3, 0, 0, 0, 0},	/* level 11 */
  {4, 4, 4, 4, 4, 1, 0, 0, 0},	/* level 12 */
  {5, 5, 5, 4, 4, 2, 0, 0, 0},	/* level 13 */
  {5, 5, 5, 4, 4, 2, 1, 0, 0},	/* level 14 */
  {5, 5, 5, 5, 5, 2, 1, 0, 0},	/* level 15 */
  {5, 5, 5, 5, 5, 3, 2, 1, 0},	/* level 16 */
  {5, 5, 5, 5, 5, 3, 3, 2, 0},	/* level 17 */
  {5, 5, 5, 5, 5, 3, 3, 2, 1},	/* level 18 */
  {5, 5, 5, 5, 5, 3, 3, 3, 1},	/* level 19 */
  {5, 5, 5, 5, 5, 4, 3, 3, 2},	/* level 20 */
  {5, 5, 5, 5, 5, 4, 4, 4, 2},	/* level 21 */
  {5, 5, 5, 5, 5, 5, 4, 4, 3},	/* level 22 */
  {5, 5, 5, 5, 5, 5, 5, 5, 3},	/* level 23 */
  {5, 5, 5, 5, 5, 5, 5, 5, 4},	/* level 24 */
  {5, 5, 5, 5, 5, 5, 5, 5, 5},	/* level 25 */
  {6, 6, 6, 6, 5, 5, 5, 5, 5},	/* level 26 */
  {6, 6, 6, 6, 6, 6, 6, 5, 5},	/* level 27 */
  {6, 6, 6, 6, 6, 6, 6, 6, 6},	/* level 28 */
  {7, 7, 7, 7, 6, 6, 6, 6, 6},	/* level 29 */
  {7, 7, 7, 7, 7, 7, 7, 6, 6},	/* level 30 */
  {7, 7, 7, 7, 7, 7, 7, 7, 7},	/* level 31 */
  {7, 7, 7, 7, 7, 7, 7, 7, 7},	/* level 32 */
  {7, 7, 7, 7, 7, 7, 7, 7, 7},	/* level 33 */
  {7, 7, 7, 7, 7, 7, 7, 7, 7},	/* level 34 */
};

const int ClericSpellsPerLevel[34][7] =
{
  {1, 0, 0, 0, 0, 0, 0},	/* level 1 */
  {2, 0, 0, 0, 0, 0, 0},	/* level 2 */
  {2, 1, 0, 0, 0, 0, 0},	/* level 3 */
  {3, 2, 0, 0, 0, 0, 0},	/* level 4 */
  {3, 3, 1, 0, 0, 0, 0},	/* level 5 */
  {3, 3, 2, 0, 0, 0, 0},	/* level 6 */
  {3, 3, 2, 1, 0, 0, 0},	/* level 7 */
  {3, 3, 3, 2, 0, 0, 0},	/* level 8 */
  {4, 4, 3, 2, 1, 0, 0},	/* level 9 */
  {4, 4, 3, 3, 2, 0, 0},	/* level 10 */
  {5, 4, 4, 3, 2, 1, 0},	/* level 11 */
  {6, 5, 5, 3, 2, 2, 0},	/* level 12 */
  {6, 6, 6, 4, 2, 2, 0},	/* level 13 */
  {6, 6, 6, 5, 3, 2, 0},	/* level 14 */
  {7, 7, 7, 5, 4, 2, 0},	/* level 15 */
  {7, 7, 7, 6, 5, 3, 1},	/* level 16 */
  {8, 8, 8, 6, 5, 3, 1},	/* level 17 */
  {8, 8, 8, 7, 6, 4, 1},	/* level 18 */
  {9, 9, 9, 7, 6, 4, 2},	/* level 19 */
  {9, 9, 9, 8, 7, 5, 2},	/* level 20 */
  {9, 9, 9, 9, 8, 6, 2},	/* level 21 */
  {9, 9, 9, 9, 9, 6, 3},	/* level 22 */
  {9, 9, 9, 9, 9, 7, 3},	/* level 23 */
  {9, 9, 9, 9, 9, 8, 3},	/* level 24 */
  {9, 9, 9, 9, 9, 9, 4},	/* level 25 */
  {9, 9, 9, 9, 9, 9, 4},	/* level 26 */
  {9, 9, 9, 9, 9, 9, 5},	/* level 27 */
  {9, 9, 9, 9, 9, 9, 6},	/* level 28 */
  {9, 9, 9, 9, 9, 9, 7},	/* level 29 */
  {9, 9, 9, 9, 9, 9, 8},	/* level 30 */
  {9, 9, 9, 9, 9, 9, 9},	/* level 31 */
  {9, 9, 9, 9, 9, 9, 9},	/* level 32 */
  {9, 9, 9, 9, 9, 9, 9},	/* level 33 */
  {9, 9, 9, 9, 9, 9, 9},	/* level 34 */
};

char GetMaxSphere(struct char_data *ch)
{
  int i = 0, j = 0;

  if (IS_TRUSTED(ch))
    i = 7;
  else {
    switch (GET_CLASS(ch)) {
    case CLASS_MULTI_C_M:
    case CLASS_MULTI_C_F_M:
    case CLASS_MULTI_C_I:
    case CLASS_CLERIC:
    case CLASS_DRUID:
    case CLASS_MULTI_C_R:
    case CLASS_MULTI_C_T:
      for (i = 0; i < 7; i++) {
	j = ClericSpellsPerLevel[MIN(33, GET_LEVEL(ch) - 1)][i];
	if (j == 0)
	  break;
      }
      break;
    case CLASS_PALADIN:
      for (i = 0; i < 4; i++) {
	j = ClericSpellsPerLevel[BOUNDED(0, GET_LEVEL(ch) - 8, 9)][i];
	if (j == 0)
	  break;
      }
      break;
    case CLASS_RANGER:
      for (i = 0; i < 3; i++) {
	j = ClericSpellsPerLevel[BOUNDED(0, GET_LEVEL(ch) - 7, 9)][i];
	if (j == 0)
	  break;
      }
      break;
    default:
      break;
    }
  }

  return i;
}

char GetMaxPossibleSphere(struct char_data *ch)
{
  int i = 0;

  if (IS_TRUSTED(ch))
    i = 7;
  else {
    switch (GET_CLASS(ch)) {
    case CLASS_MULTI_C_M:
    case CLASS_MULTI_C_F_M:
    case CLASS_MULTI_C_I:
    case CLASS_CLERIC:
    case CLASS_DRUID:
    case CLASS_MULTI_C_R:
    case CLASS_MULTI_C_T:
      i = 7;
      break;
    case CLASS_PALADIN:
      i = 4;
      break;
    case CLASS_RANGER:
      i = 3;
      break;
    default:
      break;
    }
  }

  return i;
}

/* returns highest spell level allowed */
int GetMaxSchool(struct char_data *ch)
{
  int i = 0, j = 0;

  if (IS_TRUSTED(ch))
    i = 9;
  else {
    switch (GET_CLASS(ch)) {
    case CLASS_MAGIC_USER:
    case CLASS_ILLUSIONIST:
    case CLASS_MULTI_C_M:
    case CLASS_MULTI_F_M:
    case CLASS_MULTI_M_T:
    case CLASS_MULTI_F_M_T:
    case CLASS_MULTI_C_F_M:
    case CLASS_MULTI_C_I:
    case CLASS_MULTI_F_I:
      for (i = 0; i < 9; i++) {
	j = MageSpellsPerLevel[MIN(GET_LEVEL(ch) - 1, 33)][i];
	if (j == 0)
	  break;
      }
      break;
    default:
      break;
    }
  }
  return i;
}

int GetMaxPossibleSchool(struct char_data *ch)
{
  int i = 0;

  if (IS_TRUSTED(ch))
    i = 9;
  else {
    switch (GET_CLASS(ch)) {
    case CLASS_MAGIC_USER:
    case CLASS_ILLUSIONIST:
    case CLASS_MULTI_C_M:
    case CLASS_MULTI_F_M:
    case CLASS_MULTI_M_T:
    case CLASS_MULTI_F_M_T:
    case CLASS_MULTI_C_F_M:
    case CLASS_MULTI_C_I:
    case CLASS_MULTI_F_I:
      i = 9;
      break;
    default:
      break;
    }
  }
  return i;
}

byte GetMaxCircle(struct char_data * ch, byte pray)
{
  return ((pray) ? GetMaxSphere(ch) : GetMaxSchool(ch));
}

byte GetMaxPossibleCircle(struct char_data * ch, byte pray)
{
  return ((pray) ? GetMaxPossibleSphere(ch) : GetMaxPossibleSchool(ch));
}

/* returns how many they can retain in this circle */
int MaxSpellsPerCircle(struct char_data *ch, int circle, byte sphere)
{
  int count = 0, level;

  if (!ch)
    return 0;

  level = (GET_LEVEL(ch) >= LVL_GOD ? 30 : GET_LEVEL(ch) - 1);
  level = MAX(0, level);	/* to avoid 0 mobs listing at -1 */
  circle--;			/* we send in actual circle, this needs 0 circle */

  if (!sphere) {
    switch (GET_CLASS(ch)) {
    case CLASS_MAGIC_USER:
    case CLASS_ILLUSIONIST:
    case CLASS_MULTI_C_M:
    case CLASS_MULTI_F_M:
    case CLASS_MULTI_M_T:
    case CLASS_MULTI_F_M_T:
    case CLASS_MULTI_C_F_M:
      count = MageSpellsPerLevel[level][circle];
      /* add in int bonus here */
      break;
    default:
      break;
    }
  } else {
    switch (GET_CLASS(ch)) {
    case CLASS_CLERIC:
    case CLASS_DRUID:
    case CLASS_MULTI_C_R:
    case CLASS_MULTI_C_F_M:
    case CLASS_MULTI_C_M:
      count = ClericSpellsPerLevel[level][circle];
      /* add in wis bonus here */
      break;
    case CLASS_PALADIN:
      count = MIN(ClericSpellsPerLevel[BOUNDED(0, level - 8, 9)][MIN(circle, 3)],
		  3);
      break;
    case CLASS_RANGER:
      count = MIN(ClericSpellsPerLevel[BOUNDED(0, level - 7, 9)][MIN(circle, 2)],
		  3);
      break;
    default:
      break;
    }
  }
  return count;
}

/* returns what circle this spell is */
int GetSpellCircle(struct char_data *ch, int spellnum)
{
  int c, m, i = LVL_IMPL + 1;

  if (spellnum < 1)
    return 0;

  if (ch == NULL) {
    if (spell_info[spellnum].sphere)
      i = spell_info[spellnum].min_level[CLASS_CLERIC];
    else
      i = spell_info[spellnum].min_level[CLASS_MAGIC_USER];
  } else {
    c = spell_info[spellnum].min_level[CLASS_CLERIC];
    m = spell_info[spellnum].min_level[CLASS_MAGIC_USER];
    if ((c <= MAX_CIRCLE) && (m <= MAX_CIRCLE)) {
      if (IS_MAGE(ch) && IS_PRIEST(ch))
	i = MAX(c, m);
      else if (IS_PRIEST(ch))
	i = c;
      else if (IS_MAGE(ch))
	i = m;
    } else if (c <= MAX_CIRCLE) {
      if (IS_TRUSTED(ch) || IS_PRIEST(ch))
	i = c;
    } else {
      if (IS_TRUSTED(ch) || IS_MAGE(ch))
	i = m;
    }
  }

  return i;
}

/* checks if list has one of the required spheres */
char HasRequiredSphere(ulong list, ulong spheres)
{
  int i;
  ulong checkbit;

  for (i = 0; i < MAX_SPHERE; i++) {
    checkbit = spheres & (1 << i);
    /*
       if(checkbit && IS_SET(list, (i+1)))
       break;
     */
    if (checkbit && IS_SET((list >> i), 1))
      break;
  }

  return (i == MAX_SPHERE) ? 0 : 1;
}

char HasRequiredSchool(ulong list, ulong school)
{
  int i;
  ulong checkbit;

  for (i = 0; i < MAX_SCHOOL; i++) {
    checkbit = school & (1 << i);
    /*
       if(checkbit && IS_SET(list, (i+1)))
       break;
     */
    if (checkbit && IS_SET((list >> i), 1))
      break;
  }

  return (i == MAX_SCHOOL) ? 0 : 1;
}

/* returns how many they've already memmed in this circle */
int CurrentSpellsMemmed(struct char_data *ch, int circle, char pray)
{
  int i, count = 0;
  struct spell_data *ptr = NULL;

  for (i = 1; i <= LAST_SPELL; i++) {
    if ((pray) ? (!IS_PRIEST_SPELL(i)) : (!IS_MAGE_SPELL(i)))
      continue;
    if ((GET_SPELL(ch, i) == MEMINFO_READY)
	&& (GetSpellCircle(ch, i) == circle))
      count++;
    else if (GET_SPELL(ch, i) > MEMINFO_READY &&
	     (GetSpellCircle(ch, i) == circle))
      count += GET_SPELL(ch, i) - MEMINFO_READY;
  }
  for (ptr = ch->char_specials.spell_mem_list; ptr; ptr = ptr->next) {
    if ((pray) ? (GET_SPELL_SPHERE(ptr->spell) == 0) :
	(GET_SPELL_SCHOOL(ptr->spell) == 0))
      continue;

    if (GetSpellCircle(ch, ptr->spell) == circle)
      count++;
  }
  return count;
}

/*Checks if a player can learn a spell. 
   priest: TRUE-player is a priest
   FALSE-player is not a priest (implies a mage)
   cantlearn: TRUE-check MEMINFO_CANTLEARN
   FALSE-ignore MEMINFO_CANTLEARN
   maxpossible: TRUE-get max circle possible for the class
   FALSE-get max circle possible for player's current level
 */
bool can_know_spell(struct char_data * ch, int spellnum, bool priest,
		    bool cantlearn, bool maxpossible)
{
  ulong sphere = GET_SPELL_SPHERE(spellnum);
  ulong school = GET_SPELL_SCHOOL(spellnum);
  int circle = GetSpellCircle(ch, spellnum);
  bool result = TRUE;

  if (IS_TRUSTED(ch))
    return TRUE;

  if (cantlearn && GET_SPELL(ch, spellnum) == MEMINFO_CANTLEARN) {
    result = FALSE;
  }
  if (circle > ((maxpossible) ? GetMaxPossibleCircle(ch, priest) :
		GetMaxCircle(ch, priest))) {
    result = FALSE;
  }
  if (priest) {
    if (circle > 3) {
      if (!HasRequiredSphere(GET_PRIEST_SPHERE_MAJOR(ch), sphere)) {
	result = FALSE;
      }
    } else if (!HasRequiredSphere(GET_PRIEST_SPHERE_MAJOR(ch), sphere) &&
	       !HasRequiredSphere(GET_PRIEST_SPHERE_MINOR(ch), sphere)) {
      result = FALSE;
    }
  } else if (!HasRequiredSchool(GET_MAGE_SCHOOL(ch), school)) {
    result = FALSE;
  }
  return result;
}

struct spell_data *add_new_memorize_spell(struct char_data *ch, int spell,
					  char pray)
{
  struct spell_data *ptr, *temp;
  int level;

  level = GetSpellCircle(ch, spell);

  CREATE(ptr, struct spell_data, 1);

  ptr->next = NULL;
  ptr->spell = spell;
  ptr->pray = pray;
  ptr->ticks = PASSES_PER_SEC * SECS_PER_ROUND * level;
  /* around 20 seconds per circle */

  if (!ch->char_specials.spell_mem_list)
    ch->char_specials.spell_mem_list = ptr;
  else {
    for (temp = ch->char_specials.spell_mem_list; temp->next; temp = temp->next);
    temp->next = ptr;
  }

  return ptr;
}

void delete_memorizing_spell(struct char_data *ch, struct spell_data *ptr)
{
  struct spell_data *temp;

  if (!ch->char_specials.spell_mem_list || !ptr)
    return;

  if (ptr == ch->char_specials.spell_mem_list) {
    ch->char_specials.spell_mem_list = ptr->next;
  } else {
    for (temp = ch->char_specials.spell_mem_list; temp && (temp->next != ptr); temp = temp->next);
    if (temp)
      temp->next = ptr->next;
  }

  FREE((char *) ptr);
  ptr = NULL;
}

void stop_memorizing(struct char_data *ch)
{
  if (IS_PC(ch) && IS_AFFECTED(ch, AFF_MEMORIZING)) {
    if (IsMemmingClass(ch)) {
      send_to_char("You abandon your studies.\r\n", ch);
      act("$n, looking very frustrated, packs up $s book.", FALSE, ch, 0, 0, PROXIMITY);
    } else {
      send_to_char("You abandon your praying.\r\n", ch);
      act("$n, looking very frustrated, packs up $s holy symbol.", FALSE, ch, 0, 0, PROXIMITY);
    }
    REMOVE_BIT(AFF_FLAGS(ch), AFF_MEMORIZING);

    if (GET_ACTION(ch))
      event_cancel(GET_ACTION(ch));

    GET_ACTION(ch) = NULL;
  }
}

ACMD(do_memorize)
{
  int i, j, intel = 0, circle, duh = 0, a, spellnum = 0;
  struct spell_data *ptr;
  struct memorizing_event *me = NULL;
  char Gbuf1[MAX_STRING_LENGTH];
  char Gbuf2[MAX_STRING_LENGTH];
  char *s, *t;
  long sphere, school;
  char pray = 0;

  if (!ch || !ch->desc)
    return;

  if ((subcmd == SCMD_PRAY))
    pray = 1;

  if ((subcmd == SCMD_PRAY) && !IsPrayingClass(ch)) {
    send_to_char("The gods ignore you.\r\n", ch);
    return;
  }
  if ((subcmd == SCMD_MEM) && !IsMemmingClass(ch)) {
    send_to_char("You aren't trained in magic.\r\n", ch);
    return;
  }
  if ((subcmd == SCMD_MEM) && (IS_AFFECTED(ch, AFF_CAMPING))) {
    send_to_char("You can't memorize while setting up camp!\n", ch);
    return;
  }
  if ((subcmd == SCMD_PRAY) && (IS_AFFECTED(ch, AFF_CAMPING))) {
    send_to_char("You can't pray while setting up camp!\n", ch);
    return;
  }
  /* If there are no chars in argument */
  if (!argument || !*argument) {
    Gbuf1[0] = 0;
    for (j = GetMaxCircle(ch, pray); j > 0; j--) {
      duh = 0;
      for (i = 1; i <= LAST_SPELL; i++) {
	if ((pray) ? (!IS_PRIEST_SPELL(i)) : (!IS_MAGE_SPELL(i)))
	  continue;

	if (GET_SPELL(ch, i) >= MEMINFO_READY && (GetSpellCircle(ch, i) == j)) {
	  if (!*Gbuf1)
	    strcpy(Gbuf1, "You have memorized the following spells:\r\n");

	  if (!duh) {
	    duh = TRUE;
	    sprintf(Gbuf1 + strlen(Gbuf1), "(%2d%s circle) - (%2d) %s\r\n",
		 j, j == 1 ? "st" : j == 2 ? "nd" : j == 3 ? "rd" : "th",
		    GET_SPELL(ch, i) + 1 - MEMINFO_READY, spells[i]);
	  } else
	    sprintf(Gbuf1 + strlen(Gbuf1), "               - (%d) %s\r\n",
		    GET_SPELL(ch, i) + 1 - MEMINFO_READY, spells[i]);
	}
      }
    }
    if (ch->char_specials.spell_mem_list) {
      if (*Gbuf1)
	strcat(Gbuf1, "\r\nAnd you ");
      else
	strcat(Gbuf1, "You ");

      if (!pray)
	strcat(Gbuf1, "are currently memorizing the following spells:\r\n");
      else
	strcat(Gbuf1, "are currently praying for the following spells:\r\n");

      for (ptr = ch->char_specials.spell_mem_list; ptr; ptr = ptr->next) {
/*
   if ((ptr == ch->char_specials.spell_mem_list) && IS_AFFECTED(ch, AFF_MEMORIZING))
 */
	if ((pray) ? (!IS_PRIEST_SPELL(ptr->spell)) : (!IS_MAGE_SPELL(ptr->spell)))
	  continue;

	intel = ptr->ticks / 10;
	j = GetSpellCircle(ch, ptr->spell);

	if (j > 9)		//gods praying or memming

	  j = 10;

	sprintf(Gbuf1 + strlen(Gbuf1), "%5d seconds:  (%2d%s) %s\r\n",
		intel, j, (j == 1) ? "st" : (j == 2) ? "nd" : (j == 3) ? "rd" : "th", spells[ptr->spell]);
      }
    }
    if (!pray)
      strcat(Gbuf1, "\r\nYou can memorize ");
    else
      strcat(Gbuf1, "\r\nYou can pray ");
    i = 0;
    for (a = 1; a <= GetMaxCircle(ch, pray); a++)
      if (MaxSpellsPerCircle(ch, a, pray))
	if (MaxSpellsPerCircle(ch, a, pray) > (intel = (CurrentSpellsMemmed(ch, a, pray)))) {
	  if (i) {
	    strcat(Gbuf1, Gbuf2);
	    strcat(Gbuf1, ", ");
	  }
	  i++;
	  sprintf(Gbuf2, "%d %d%s", MaxSpellsPerCircle(ch, a, pray) - CurrentSpellsMemmed(ch, a, pray), a,
	     (a == 1) ? "st" : (a == 2) ? "nd" : (a == 3) ? "rd" : "th");
	}
    if (!i)
      strcat(Gbuf1, "no more spells.\r\n");
    else {
      if (i > 1)
	strcat(Gbuf1, "and ");
      strcat(Gbuf1, Gbuf2);
      strcat(Gbuf1, " circle spell(s).\r\n");
    }
    page_string(ch->desc, Gbuf1, 1);

#ifdef SPELL_DEBUG
    sprintf(Gbuf2, "&+CDEBUG:&n Current spells memmed: ");
    for (i = 0; i < 9; i++) {
      sprintf(Gbuf1, "(%d) %d ", i + 1, CurrentSpellsMemmed(ch, i + 1, pray));
      strcat(Gbuf2, Gbuf1);
    }
    strcat(Gbuf2, "\r\n");
    send_to_char(Gbuf2, ch);
#endif

    if (ch->char_specials.spell_mem_list && !IS_AFFECTED(ch, AFF_MEMORIZING) &&
	GET_COND(ch, FULL) && GET_COND(ch, THIRST) && !GET_ACTION(ch)) {
      if (!pray) {
	send_to_char("You continue your study.\r\n", ch);
	strcpy(Gbuf1, "$n opens an ancient tome and begins studying it intently.");
      } else {
	send_to_char("You continue your praying.\r\n", ch);
	sprintf(Gbuf1, "$n takes out $s %sholy symbol and begins praying intently.", IS_EVIL(ch) ? "un" : "");
      }
      act(Gbuf1, TRUE, ch, 0, 0, PROXIMITY);

      CREATE(me, struct memorizing_event, 1);
      me->ch = ch;
      GET_ACTION(ch) = event_create(memorizing_event, me, ch->char_specials.spell_mem_list->ticks);

      SET_BIT(AFF_FLAGS(ch), AFF_MEMORIZING);
    }
    return;
  }
#if 1
  // I doubt we need this here.
  /* get: blank, spell name, target name */
  s = strtok(argument, "'");
  s = strtok(NULL, "'");
  if (s == NULL) {
    send_to_char("Spell names must be enclosed in the Holy Magic Symbols: '\r\n", ch);
    return;
  }
  t = strtok(NULL, "\0");

  spellnum = find_spells_num(s);
#else
  skip_spaces(&argument);
  spellnum = find_spells_num(argument);
#endif

#ifdef SPELL_DEBUG
  sprintf(buf, "&+CDEBUG:&N Spellnum is %d.\r\n", spellnum);
  send_to_char(buf, ch);
#endif

  if ((spellnum < 1) || (spellnum > LAST_SPELL)) {
    send_to_char("Um.. what spell are you trying to memorize?\r\n", ch);
    return;
  }
  if (!pray && !IS_TRUSTED(ch))
    duh = SpellInSpellBook(ch, spellnum, SBOOK_MODE_IN_INV + SBOOK_MODE_AT_HAND);

  circle = GetSpellCircle(ch, spellnum);

#ifdef SPELL_DEBUG
  sprintf(buf, "&+CDEBUG:&N circle is %d\r\n", circle);
  send_to_char(buf, ch);
#endif

  if (circle > GetMaxCircle(ch, pray)) {
    if (pray || duh) {
      send_to_char("That is too powerful an enchantment for you to master.. yet, anyway.\r\n", ch);
      return;
    } else if (!duh) {
      send_to_char("Yes, you _HAVE_ heard of rumors about that spell.. but you think it beyond\r\nyour powers anyway.\r\n", ch);
      return;
    }
  } else if (!duh && !pray && !IS_TRUSTED(ch)) {
    send_to_char("Sorry, but you haven't got that spell in your spellbook!\r\n", ch);
    return;
  }
#ifdef SCHOOLS_ON
  sphere = GET_SPELL_SPHERE(spellnum);
  school = GET_SPELL_SCHOOL(spellnum);
  if ((GET_SPELL(ch, spellnum) == MEMINFO_CANTLEARN) ||
      (((circle > 3) ? !HasRequiredSphere(GET_PRIEST_SPHERE_MAJOR(ch), sphere) :
	!(HasRequiredSphere(GET_PRIEST_SPHERE_MAJOR(ch), sphere) ||
	  HasRequiredSphere(GET_PRIEST_SPHERE_MINOR(ch), sphere)))
       && !HasRequiredSchool(GET_MAGE_SCHOOL(ch), school))) {
    send_to_char("Sorry, that spell is simply too difficult for you to ever learn.\r\n", ch);
    return;
  }
#endif
  if (CurrentSpellsMemmed(ch, circle, pray) >= MaxSpellsPerCircle(ch, circle, pray)) {
    send_to_char("You can't hold any more spells in your thought.\r\n", ch);
    return;
  }
  if (GET_STAT(ch) != STAT_RESTING) {
    send_to_char("One must rest before attempting to learn new magiks.\r\n", ch);
    stop_memorizing(ch);
    return;
  }
  if (!(GET_COND(ch, FULL)) || !(GET_COND(ch, THIRST))) {
    send_to_char("You can't concentrate with ", ch);
    if (!GET_COND(ch, FULL))
      if (!GET_COND(ch, THIRST))
	send_to_char("your stomach aching and your throat dry.\r\n", ch);
      else
	send_to_char("your stomach rumbling.\r\n", ch);
    else
      send_to_char("your throat this parched.\r\n", ch);
    stop_memorizing(ch);
    return;
  }
  ptr = add_new_memorize_spell(ch, spellnum, pray);

  /* create and initialize the event 
     CREATE(me, struct memorizing_event, 1);
     me->ch = ch;
   */

  if (!IS_AFFECTED(ch, AFF_MEMORIZING) && !GET_ACTION(ch)) {
    if (pray)
      sprintf(Gbuf1, "$n takes out $s %sholy symbol and begins praying intently.", IS_EVIL(ch) ? "un" : "");
    else
      strcpy(Gbuf1, "$n opens an ancient tome and begins studying it intently.");
    act(Gbuf1, TRUE, ch, 0, 0, PROXIMITY);

    CREATE(me, struct memorizing_event, 1);
    me->ch = ch;
    GET_ACTION(ch) = event_create(memorizing_event, me, ch->char_specials.spell_mem_list->ticks);
    SET_BIT(AFF_FLAGS(ch), AFF_MEMORIZING);
  }
  if (pray)
    sprintf(Gbuf1, "You are praying for %s, which will take about %d seconds.\r\n",
	    spells[spellnum], ptr->ticks / 10);
  else
    sprintf(Gbuf1, "You are memorizing %s, which will take about %d seconds.\r\n",
	    spells[spellnum], ptr->ticks / 10);
  send_to_char(Gbuf1, ch);
}

EVENTFUNC(memorizing_event)
{
  struct spell_data *ptr;
  struct memorizing_event *me = (struct memorizing_event *) event_obj;
  struct char_data *ch;
  int a = 0;
  char pray;

  ch = me->ch;

  ptr = ch->char_specials.spell_mem_list;
  pray = ptr->pray;

  if (!ptr) {
    GET_ACTION(ch) = NULL;
    return 0;
  }
  if (!IS_AFFECTED(ch, AFF_MEMORIZING)) {
    GET_ACTION(ch) = NULL;
    return 0;
  }
  if (GET_STAT(ch) != STAT_RESTING) {
    send_to_char("One must rest before attempting to learn new magiks.\r\n", ch);
    stop_memorizing(ch);
    return 0;
  }
  if (!(GET_COND(ch, FULL)) || !(GET_COND(ch, THIRST))) {
    send_to_char("You can't concentrate with ", ch);
    if (!GET_COND(ch, FULL))
      if (!GET_COND(ch, THIRST))
	send_to_char("your stomach aching and your throat dry.\r\n", ch);
      else
	send_to_char("your stomach rumbling.\r\n", ch);
    else
      send_to_char("your throat this parched.\r\n", ch);
    stop_memorizing(ch);
    return 0;
  }
  if (!pray && !IS_TRUSTED(ch) &&
      !(SpellInSpellBook(ch, ptr->spell, SBOOK_MODE_IN_INV + SBOOK_MODE_AT_HAND))) {
    send_to_char("Uh huh, you managed to misplace your spellbook during memorization?\r\n", ch);
    stop_memorizing(ch);
    return 0;
  }
/*  for (temp = ch->char_specials.spell_mem_list; temp->next; temp = temp->next) {
   ptr = temp;
   if (ptr->ticks >= 1)
   continue;
   }
 */
  if (pray)
    sprintf(buf, "You have finished praying for %s.\r\n", spells[ptr->spell]);
  else
    sprintf(buf, "You have finished memorizing %s.\r\n", spells[ptr->spell]);

  send_to_char(buf, ch);

/*
   if (!ch->char_specials.spell_mem_list) {
   if (!IsMemmingClass(ch)) {
   send_to_char("Your prayers are complete.\r\n", ch);
   sprintf(buf, "$n raises $s %sholy symbol and smiles %s.",
   IS_EVIL(ch) ? "un" : "", IS_EVIL(ch) ? "malevolently" : "broadly");
   } else {
   send_to_char("Your studies are complete.\r\n", ch);
   sprintf(buf, "$n closes $s book and %s.", IS_EVIL(ch) ? "grins malevolently" : "smiles broadly");
   }
   act(buf, TRUE, ch, 0, 0, PROXIMITY);
   REMOVE_BIT(AFF_FLAGS(ch), AFF_MEMORIZING);
   }
 */

  a = GetSpellCircle(ch, ptr->spell);

  if (a <= 0) {
    send_to_char("You cannot memorize this spell.\r\n", ch);
  } else {
    if (IS_TRUSTED(ch) ||
	(MaxSpellsPerCircle(ch, a, pray) >= CurrentSpellsMemmed(ch, a, pray))) {
      if (GET_SPELL(ch, ptr->spell) < MEMINFO_READY) {
	SET_SPELL(ch, ptr->spell, MEMINFO_READY);
      } else if (GET_SPELL(ch, ptr->spell) >= MEMINFO_READY) {
	SET_SPELL(ch, ptr->spell, (GET_SPELL(ch, ptr->spell) + 1));
      }
    } else {
      send_to_char("You cannot get any more spells at this circle.\r\n", ch);
    }
  }
#ifdef SPELL_DEBUG
  sprintf(buf, "&+CDEBUG:&n Adding the spell %s to memory now. (%d)\r\n",
	  spells[ptr->spell], GET_SPELL(ch, ptr->spell));
  send_to_char(buf, ch);
#endif

  delete_memorizing_spell(ch, ptr);
  if (!ch->char_specials.spell_mem_list) {
    a = 0;
    if (pray) {
      send_to_char("Your prayers are complete.\r\n", ch);
      sprintf(buf, "$n raises $s %sholy symbol and smiles %s.",
      IS_EVIL(ch) ? "un" : "", IS_EVIL(ch) ? "malevolently" : "broadly");
    } else {
      send_to_char("Your studies are complete.\r\n", ch);
      sprintf(buf, "$n closes $s book and %s.", IS_EVIL(ch) ? "grins malevolently" : "broadly");
    }
    act(buf, TRUE, ch, 0, 0, PROXIMITY);
    REMOVE_BIT(AFF_FLAGS(ch), AFF_MEMORIZING);
  } else
    a = ch->char_specials.spell_mem_list->ticks;

  if (a == 0)
    GET_ACTION(ch) = NULL;

  return a;
}

int GetPagesInBook(struct obj_data *obj)
{
  return GET_OBJ_VAL(obj, 3);
}

int GetTotalPagesInBook(struct obj_data *obj)
{
  return GET_OBJ_VAL(obj, 2);
}


int GetSpellPages(int class, int spl)
{
  return (spell_info[spl].min_level[class]);
}

struct scribing_data_type *scribing_base, *scribing_free_base = NULL;

int ScribeJobs(void)
{
  struct scribing_data_type *foo;
  int i = 0;

  for (foo = scribing_base; foo; foo = foo->next)
    i++;
  return i;
}

int ScribeJobsFree(void)
{
  struct scribing_data_type *foo;
  int i = 0;

  for (foo = scribing_free_base; foo; foo = foo->next)
    i++;
  return i;
}

void slay_scribe_data(struct scribing_data_type *tmp)
{
  struct scribing_data_type *t;

  /* update linked list */
  /* now it _SHOULD_ work. we shall see if it does.  */

  for (t = scribing_base; (t->next && (t->next != tmp)); t = t->next);
  if (t->next)
    t->next = tmp->next;
  if (tmp == scribing_base)
    scribing_base = tmp->next;

  tmp->next = scribing_free_base;
  scribing_free_base = tmp;
}

void scribe_job_nuke_subfunc(struct scribing_data_type *tmp)
{
  struct obj_data *obj;

  if (IS_AFFECTED(tmp->ch, AFF_SCRIBING)) {
    send_to_char("So much for that scribing effort!\r\n", tmp->ch);
    REMOVE_BIT(AFF_FLAGS(tmp->ch), AFF_SCRIBING);
    GET_ACTION(tmp->ch) = NULL;
  }
  switch (tmp->flag) {
  case 0:
    if (!tmp->source.teacher)
      break;
    if (!IS_AFFECTED(tmp->source.teacher, AFF_SCRIBING))
      break;
    send_to_char("So much for that teaching effort!\r\n", tmp->source.teacher);
    REMOVE_BIT(AFF_FLAGS(tmp->source.teacher), AFF_SCRIBING);
    break;
  case 1:			/* nothing happens when you stop scribing from spellbook */
    break;
  case 2:
    if (number(0, 4))
      break;
    obj = FindSpellBookWithSpell(tmp->ch, tmp->spell, SBOOK_MODE_IN_INV + SBOOK_MODE_NO_BOOK);
    if (!obj)
      break;
    send_to_char("An ancient parchment comes apart in your hands..\r\n", tmp->ch);
    tmp->source.obj = NULL;
    extract_obj(obj);
    break;
  }
  slay_scribe_data(tmp);
  return;
}

struct scribing_data_type *
 find_scribe_job_char(struct char_data *ch)
{
  struct scribing_data_type *tmp;

  for (tmp = scribing_base; tmp; tmp = tmp->next)
    if (tmp->ch == ch || (tmp->flag == 0 && tmp->source.teacher == ch))
      return tmp;
  send_to_char("Something truly deranged just happened. please report this to a coder ASAP.\r\n", ch);
  return NULL;
}

struct scribing_data_type *
 find_scribe_job_obj(struct obj_data *obj)
{
  struct scribing_data_type *tmp;

  for (tmp = scribing_base; tmp; tmp = tmp->next)
    if ((tmp->flag && (tmp->source.obj == obj)) || (tmp->book == obj))
      return tmp;
  return NULL;
}

void scribe_job_nuke(struct char_data *ch)
{
  struct scribing_data_type *tmp;

  if ((tmp = find_scribe_job_char(ch))) {
    if (GET_ACTION(ch))
      event_cancel(GET_ACTION(ch));
    GET_ACTION(ch) = NULL;
    scribe_job_nuke_subfunc(tmp);
    return;
  }
  /*
     something real weird just happened ! - this proc needs a life, like doing
     of cores or something. debugging now, thus no core, only report.
   */
  REMOVE_BIT(AFF_FLAGS(ch), AFF_SCRIBING);
}

void check_for_scribe_nukage_object(struct obj_data *obj)
{
  struct scribing_data_type *tmp;

  if ((tmp = find_scribe_job_obj(obj))) {
    scribe_job_nuke_subfunc(tmp);
    return;
  }
}

void scribe_job_done(struct char_data *ch)
{
  struct scribing_data_type *tmp;
  int i, j;

  tmp = find_scribe_job_char(ch);

  REMOVE_BIT(AFF_FLAGS(ch), AFF_SCRIBING);
  GET_ACTION(ch) = NULL;
  if (!tmp) {
    /* cringe, something screwed.  */
    send_to_char("Something in your scribing attempt went awry. please report this to a coder.\r\n", ch);
    return;
  }
  sprintf(buf, "You finish scribing the spell %s to $p.", spells[tmp->spell]);
  act(buf, TRUE, ch, tmp->book, 0, TO_CHAR);
  if (!tmp->flag && tmp->source.teacher) {
    sprintf(buf, "You finish teaching spell %s to %s.\r\n", spells[tmp->spell], GET_NAME(ch));
    send_to_char(buf, tmp->source.teacher);
    REMOVE_BIT(AFF_FLAGS(tmp->source.teacher), AFF_SCRIBING);
  }
  if (tmp->flag == 2) {
#ifdef SPELL_DEBUG
    send_to_char("&+CDEBUG:&n Scribing from a scroll. Deciding if its used up now.\r\n", ch);
#endif
    j = 0;
    for (i = 1; i < 4; i++)
      if (GET_OBJ_VAL(tmp->source.obj, i) >= 1)
	j++;
    if (j == 1) {
      send_to_char("The ancient parchment comes apart in your hands..\r\n", tmp->ch);
      extract_obj(tmp->source.obj);
    } else if (j == 0) {
      send_to_char("Error detected - please report this to a coder imm. [0 spells in scroll.\r\n", ch);
      slay_scribe_data(tmp);
      return;
    } else {
      j = 0;
      for (i = 1; ((i < 4) && !j); i++)
	if (GET_OBJ_VAL(tmp->source.obj, i) == tmp->spell) {
	  j = 1;
	  GET_OBJ_VAL(tmp->source.obj, i) = 0;
	  send_to_char("You mess up the scroll so badly that part of it becomes unreadable.\r\n", ch);
	}
    }
  } else if (tmp->flag == 3) {
    if (GET_OBJ_OWNER(tmp->source.obj) != GET_IDNUM(tmp->ch)) {
      RemoveSpellFromSpellBook(tmp->ch, tmp->source.obj, tmp->spell);
      send_to_char("Your scribing erases the spell from the book.\r\n", ch);
    }
  }
  /* ok, those few funny things done.. let's do something else: */
  if (!AddSpellToSpellBook(tmp->ch, tmp->book, tmp->spell)) {
    send_to_char("Hmm, you have that spell in your spellbook already???\r\n", ch);
    slay_scribe_data(tmp);
    return;
  }
  if (GET_OBJ_VAL(tmp->book, 2) - GET_OBJ_VAL(tmp->book, 3)) {
    sprintf(buf, "The spell uses %d pages in $p, leaving %d more pages free.",
	    tmp->pagetime,
	    GET_OBJ_VAL(tmp->book, 2) - GET_OBJ_VAL(tmp->book, 3));
    act(buf, TRUE, ch, tmp->book, 0, TO_CHAR);
  } else
    send_to_char("The spell used the rest of the empty pages in the book.\r\n", ch);
  slay_scribe_data(tmp);
}

void AddScribingAffect(struct char_data *ch)
{
  WAIT_STATE(ch, 2 * PULSE_VIOLENCE);
  SET_BIT(AFF_FLAGS(ch), AFF_SCRIBING);
}

void add_scribe_data(int spl, struct char_data *ch, struct obj_data *book, int flag, struct obj_data *obj, struct char_data *teacher)
{
  struct scribing_data_type *tmp;
  struct scribing_event *se = NULL;

  if (scribing_free_base) {
    tmp = scribing_free_base;
    scribing_free_base = tmp->next;
  } else
    CREATE(tmp, struct scribing_data_type, 1);

  bzero(tmp, sizeof(*tmp));
  /* ok, now we've gotta new array entry which is empty.  */
  tmp->next = scribing_base;
  scribing_base = tmp;

  /* ok, struct added to top of the 'thing' */
  /* let's fill it up, shall we not? :) */
  tmp->ch = ch;
  tmp->book = book;
  tmp->flag = flag;
  if (flag)
    tmp->source.obj = obj;
  else
    tmp->source.teacher = teacher;
  tmp->spell = spl;
  /* ok, now doing some heavy calculation pagetime's unit is 1/4 second */
  tmp->pagetime = GetSpellPages(CLASS_MAGIC_USER, spl) + dice(1, 6);
/*
   tmp->pagetime =
   MAX(2, ((38 - STAT_INDEX(GET_C_DEX(ch))) / 3)) *
   MAX(1, ((38 - STAT_INDEX(GET_C_INT(ch))) / 2)) *
   GetSpellCircle(ch, spl) * 4 / MAX(5, GET_LEVEL(ch)) *
   (200 - GET_CHAR_SKILL(ch, SKILL_SCRIBE)) / 100;
 */
  AddScribingAffect(ch);
  if (!flag)
    if (teacher)
      AddScribingAffect(teacher);
  CREATE(se, struct scribing_event, 1);
  se->ch = ch;
  se->s = tmp;

  GET_ACTION(ch) = event_create(scribing_event, se, PASSES_PER_SEC * SECS_PER_ROUND);
}

/* flag has diff modes: 0 = teach, 1 = scribe */
/* obj empty if teach mode, contains spellbook/scroll otherwise */
/* teacher = fellow who's teaching char something */

void add_scribing(struct char_data *ch, int spl, struct obj_data *book, int flag, struct obj_data *obj, struct char_data *teacher)
{
  if (flag || !teacher)
    send_to_char("You start to copy down the spell..\r\n", ch);
  else {
    act("You start to teach $N..", TRUE, teacher, 0, ch, TO_CHAR);
    sprintf(buf, "$n starts to teach you %s..", spells[spl]);
    act(buf, TRUE, teacher, 0, ch, TO_VICT);
  }
  add_scribe_data(spl, ch, book, flag, obj, teacher);
  WAIT_STATE(ch, 3 * PULSE_VIOLENCE);
  if (!flag)
    WAIT_STATE(ch, 3 * PULSE_VIOLENCE);
}

int ScriberSillyChecks(struct char_data *ch, int spl)
{
  struct obj_data *o1, *o2, *o3;
  struct extra_descr_data *d;

  if (IS_AFFECTED(ch, AFF_MEMORIZING) || GET_ACTION(ch)) {
    send_to_char("You're busy enough as it is!\r\n", ch);
    return FALSE;
  }
  if (IS_AFFECTED(ch, AFF_SCRIBING)) {
    send_to_char("Sorry, you cannot speed up your scribing by doing multiple jobs at same time, \r\nnor teach/scribe at same time.\r\nYou are not UNIX compatible.\r\n*bonk*\r\n\n", ch);
    return FALSE;
  }
  if ((GET_STAT(ch) != STAT_RESTING) ||
      ((GET_POS(ch) != POS_SITTING) && (GET_POS(ch) != POS_KNEELING))) {
    send_to_char("You don't feel comfortable enough to start to scribe.\r\n", ch);
    return FALSE;
  }
  if (!(o1 = ch->equipment[WEAR_HOLD])) {
    send_to_char("You must have something to scribe to and from!\r\n", ch);
    return FALSE;
  }
  if (!(o2 = ch->equipment[WEAR_HOLD2])) {
    send_to_char("You must have something to scribe to and from!\r\n", ch);
    return FALSE;
  }
  if (GET_OBJ_TYPE(o1) == ITEM_SCROLL) {
    o3 = o1;
    o1 = o2;
    o2 = o3;
  }
  if (GET_OBJ_TYPE(o1) != ITEM_SCROLL && GET_OBJ_TYPE(o2) != ITEM_SCROLL) {
    send_to_char("You need a scroll with which to scribe from! [held]\r\n", ch);
    return FALSE;
  }
  if (GET_OBJ_TYPE(o1) != ITEM_BOOK && GET_OBJ_TYPE(o2) != ITEM_BOOK) {
    send_to_char("You need a spellbook to write to! [held]\r\n", ch);
    return FALSE;
  }
#ifdef MAGICBOOK
  if (o1->value[0] && (GET_LANGUAGE(ch, o1->value[0]) < 70)) {
    send_to_char("You need to be more fluent in the language before adding anything to\r\nthe book at hand.\r\n", ch);
    return FALSE;
  }
#endif
  /* enough checking for gods */
  if (IS_TRUSTED(ch))
    return TRUE;
/* piss ant stuff
   if (GET_OBJ_VAL(o1, 1) > -1 && (GET_CLASS(ch) != GET_OBJ_VAL(o1, 1))) {
   send_to_char("The spellbook you are holding is for the wrong class so you can't add to it..\r\n", ch);
   return FALSE;
   }
 */
/* and why should we care if they are only scribing it? They still cant cast it!
   if (GET_SPELL(ch, spl) == MEMINFO_CANTLEARN) {
   send_to_char("Sorry, that spell is simply to difficult for you to ever learn.\r\n", ch);
   return FALSE;
   }
 */
  if (GET_SPELL(ch, spl) >= MEMINFO_KNOWN) {
    send_to_char("You already know that spell.\r\n", ch);
    return FALSE;
  }
  d = find_spell_description(o1);
  if (d && SpellInThisSpellBook(d, spl)) {
    send_to_char("You have the spell already in your spellbook, boggle?\r\n", ch);
    return FALSE;
  }
  if ((GetSpellPages(GET_CLASS(ch), spl) + GET_OBJ_VAL(o1, 3)) > GET_OBJ_VAL(o1, 2)) {
#ifdef SPELL_DEBUG
    sprintf(buf, "&+CDEBUG:&n Spell requires %d pages, and you have %d left (spl: %d require: %d 1: %d 2: %d 3: %d).\r\n", (GetSpellPages(GET_CLASS(ch), spl) + GET_OBJ_VAL(o1, 3)), GET_OBJ_VAL(o1, 2), spl, GetSpellPages(GET_CLASS(ch), spl), GET_OBJ_VAL(o1, 1), GET_OBJ_VAL(o1, 2), GET_OBJ_VAL(o1, 3));
    send_to_char(buf, ch);
#endif
    send_to_char("Sorry, the spellbook at hand has no room for more spells.\r\n", ch);
    return FALSE;
  }
  return TRUE;
}

ACMD(do_teach)
{
  struct char_data *target;
  struct obj_data *o3;
  int spl;
  extern struct int_app_type int_app[];

  if (!*argument || !argument) {
    if (ch) {
      if (IS_AFFECTED(ch, AFF_SCRIBING))
	do_scribe(ch, argument, cmd, subcmd);
      else
	send_to_char("Usage: teach char spellname\r\n", ch);
    }
    return;
  }
  if (ch && !(IsMemmingClass(ch) || IS_TRUSTED(ch))) {
    send_to_char("Teaching others spells just isn't in your field of work!\r\n", ch);
    return;
  }
  /*
     hurm. the next things that need to be done: - spell exists - that teacher
     _has_ skill to teach to target, if teacher exists - target is ready to be
     teached - some misc checks - addition of the scribing_array with the usual

     params
   */
  argument = one_argument(argument, buf);
  if (!(target = get_char_room(buf, ch ? ch->in_room : cmd))) {
    if (ch)
      send_to_char("Teach who?\r\n", ch);
    return;
  }
  if (ch && !CAN_SEE(ch, target)) {
    send_to_char("You cannot see anyone with such name here!\r\n", ch);
    return;
  }
  if (ch == target) {
    send_to_char("You believe you know, or don't know, that spell already!\r\n", ch);
    return;
  }
  skip_spaces(&argument);
  spl = find_spells_num(argument);
  if (spl == -1 || GetSpellCircle(target, spl) > MAX_CIRCLE) {
    if (ch)
      send_to_char("Teach _WHAT_ spell?? That your target cannot learn, at least.\r\n", ch);
    return;
  }
  if (ch && (GetSpellCircle(ch, spl) > GetMaxSchool(ch) / 2)) {
    send_to_char("That spell is too difficult for you to teach.\r\n", ch);
    return;
  }
  if (ch && !(o3 = FindSpellBookWithSpell(ch, spl, SBOOK_MODE_IN_INV + SBOOK_MODE_AT_HAND)) &&
      !IS_TRUSTED(ch)) {
    send_to_char("You don't have such spell in your _own_ spellbooks!\r\n", ch);
    return;
  }
  /* ok, we've gotta valid spell _and_ target */

  if (!ScriberSillyChecks(target, spl)) {
    if (ch)
      send_to_char("Your student isn't ready to start yet!\r\n", ch);
    return;
  }
  if (number(1, 100) > int_app[GET_INT(target)].chance_learn) {
    send_to_char("You try to teach them the spell, but something \
has gone wrong.\r\n", ch);
    send_to_char("You don't think you will ever understand \
that spell.\r\n", target);
    sprintf(buf, "Learn Chance: %d\r\n", int_app[GET_INT(target)].chance_learn);
    send_to_char(buf, target);
    GET_SPELL(target, spl) = MEMINFO_CANTLEARN;
    return;
  } else
    GET_SPELL(target, spl) = MEMINFO_KNOWN;
  add_scribing(target, spl, SpellBookAtHand(target), 0, 0, ch);
}

ACMD(do_scribe)
{
  int spl = 0, flag = 0;
  struct obj_data *o1, *o2, *o3;
  struct scribing_data_type *tmp_s;

  if (!GET_SKILL(ch, SKILL_READING_WRITING) && !IS_TRUSTED(ch)) {
    send_to_char("You don't know how to scribe!\r\n", ch);
    return;
  }
  if (!*argument || !argument) {
    if (!IS_AFFECTED(ch, AFF_SCRIBING)) {
      send_to_char("Scribe what?\r\n", ch);
      return;
    }
    tmp_s = find_scribe_job_char(ch);
    if (tmp_s->ch != ch) {
      sprintf(buf, "You're teaching spell %s to $N.", spells[tmp_s->spell]);
      act(buf, TRUE, ch, 0, tmp_s->ch, TO_CHAR);
    } else {
      sprintf(buf, "You're %s spell %s from %s.\r\n", tmp_s->flag == 0 ? "learning" : "scribing to book", spells[spl], tmp_s->flag == 0 ? tmp_s->source.teacher ? "$N" : "guildmaster" : "$p");
      act(buf, TRUE, ch, tmp_s->source.obj, tmp_s->source.teacher, TO_CHAR);
    }
    return;
  }
  skip_spaces(&argument);
  spl = find_spells_num(argument);

#ifdef SPELL_DEBUG
  sprintf(buf, "&+CDEBUG:&N Spellnum is %d.\r\n", spl);
  send_to_char(buf, ch);
#endif

  if ((spl < 1) || (spl > LAST_SPELL)) {
    send_to_char("Um.. what spell are you trying to scribe?\r\n", ch);
    return;
  }
  if (!ScriberSillyChecks(ch, spl))
    return;

  o1 = ch->equipment[WEAR_HOLD];
  o2 = ch->equipment[WEAR_HOLD2];

  if (GET_OBJ_TYPE(o1) == ITEM_SCROLL) {
    o3 = o1;
    o1 = o2;
    o2 = o3;
  }
  if (!(o3 = FindSpellBookWithSpell(ch, spl, SBOOK_MODE_IN_INV + SBOOK_MODE_AT_HAND))) {
    send_to_char("You don't have that spell in any of your books or scrolls in learnable form!\r\n", ch);
    return;
  }
#ifdef SPELL_DEBUG
  send_to_char("&+CDEBUG:&N Adding scribe data now.\r\n", ch);
#endif
  if (GET_OBJ_TYPE(o3) == ITEM_SCROLL)
    flag = 2;
  else if (GET_OBJ_TYPE(o3) == ITEM_BOOK)
    flag = 3;
  else
    flag = 1;
  add_scribing(ch, spl, o1, flag, o3, NULL);
}

EVENTFUNC(scribing_event)
{
  struct scribing_event *se = (struct scribing_event *) event_obj;
  struct char_data *ch;
  struct scribing_data_type *s;

  ch = se->ch;
  s = se->s;

  /*
     what this proc does? simple: - checks if in proper state (resting,
     spellbook in one, quill in other hand). - if not so, it does some nasty
     things (aborts the scribing prematurely and informs the pc) - else, it
     just adds one more page.. and queries a new event.
   */
  if ((GET_STAT(ch) != STAT_RESTING) ||
      ((GET_POS(ch) != POS_SITTING) && (GET_POS(ch) != POS_KNEELING)) ||
      (!ch->equipment[WEAR_HOLD] || !ch->equipment[WEAR_HOLD2]) ||
      (GET_OBJ_TYPE(ch->equipment[WEAR_HOLD]) != ITEM_BOOK &&
       GET_OBJ_TYPE(ch->equipment[WEAR_HOLD2]) != ITEM_BOOK) ||
      (GET_OBJ_TYPE(ch->equipment[WEAR_HOLD]) != ITEM_SCROLL &&
       GET_OBJ_TYPE(ch->equipment[WEAR_HOLD2]) != ITEM_SCROLL) ||
      (!s || !(s->book))) {
    scribe_job_nuke(ch);
#ifdef SPELL_DEBUG
    send_to_char("&+CDEBUG:&n Something is improper, nuking scribe job from within the EVENTFUNC().\r\n", ch);
#endif
    return 0;
  }
  /* ok, we did it this far..  */

  s->page++;
  GET_OBJ_VAL(s->book, 3)++;

#ifdef SPELL_DEBUG
  sprintf(buf, "&+CDEBUG:&n s->page = %d\r\n", s->page);
  send_to_char(buf, ch);
#endif
  //if (IS_TRUSTED(ch) || s->page >= GetSpellPages(GET_CLASS(ch), s->spell))
  if (s->page >= s->pagetime)
    /* scribe job done! cackle */
    scribe_job_done(ch);
  else {
    send_to_char("You complete another page of the spell into your spellbook..\r\n", ch);
    /* ok, let's schedule next event: */
    return PASSES_PER_SEC * SECS_PER_ROUND;
  }
  return 0;
}


static int spell_cmp(const struct spl_list *, const struct spl_list *);

int spell_cmp(const struct spl_list *a, const struct spl_list *b)
{
  if (a->circle < b->circle)
    return -1;

  if (a->circle > b->circle)
    return 1;

  return (str_cmp(spells[a->spell], spells[b->spell]));
}

#define ORIGSPELLS 1
ACMD(do_spells)
{
#ifdef ORIGSPELLS
  int counter, columns = 0;
#endif
  int spell, spl, circle, i;
  struct char_data *target;
  struct spl_list spell_list[NUM_SPELLS];

  *buf = '\0';
  *buf1 = '\0';

  if (IS_NPC(ch)) {
    display_mob_spells(ch, ch);
    return;
  }
  get_char_cols(ch);

  if (IS_TRUSTED(ch) && !*argument) {
#ifdef ORIGSPELLS
    for (counter = 1; counter < NUM_SPELLS; counter++) {
      sprintf(buf + strlen(buf), "%s%2d%s) %s%-20.20s %s", grn, counter,
	  nrm, blu, spells[counter], !(++columns % 3) ? "\r\n&n" : "&n");
    }
    //  page_string(ch->desc, buf, TRUE);
    strcat(buf1, buf);
    page_string(ch->desc, buf1, 1);
    return;
#else
    target = ch;
    for (spell = 1; spell < NUM_SPELLS; spell++) {
      circle = GetSpellCircle(target, spell);
      if ((circle < 1) || (circle > MAX_CIRCLE))	//ignore these spells

	continue;
      if (can_know_spell(target, spell, TRUE, FALSE, TRUE)
	  || can_know_spell(target, spell, FALSE, FALSE, TRUE)) {
	spell_list[i].circle = circle;
	spell_list[i++].spell = spell;
      }
    }
#endif
  } else {
    if (IS_TRUSTED(ch) && *argument) {
      if (!(target = get_char_vis(ch, argument))) {
	send_to_char(NOPERSON, ch);
	return;
      }
    } else
      target = ch;

    /* first, build a list of all the spells this person can ever
       have. */
    i = 0;
    for (spell = 1; spell < NUM_SPELLS; spell++) {
      circle = GetSpellCircle(target, spell);
      /*
         if ((circle < MAX_CIRCLE + 1) && (circle > 0)) {
         spell_list[i].circle = circle;
         spell_list[i++].spell = spell;
         }
       */
      if ((IS_PRIEST(target) ?
	   can_know_spell(target, spell, TRUE, TRUE, TRUE) : FALSE)
	  || (IS_MAGE(target) ?
	    can_know_spell(target, spell, FALSE, FALSE, TRUE) : FALSE)) {
	spell_list[i].circle = circle;
	spell_list[i++].spell = spell;
      }
    }
  }
  /* then sort the list... */
  qsort(spell_list, i, sizeof(struct spl_list), spell_cmp);


  /* finally, show it */

  for (spl = 0; spl < i; spl++) {
    int spell = spell_list[spl].spell;
    circle = spell_list[spl].circle;

    if (!spl || (circle != spell_list[spl - 1].circle)) {
      sprintf(buf, "\r\n&+B%d%s CIRCLE:&N\r\n", circle,
	      circle == 1 ? "st" : circle == 2 ? "nd" : circle == 3
	      ? "rd" : "th");
      strcat(buf1, buf);
    }
    sprintf(buf, "%s%-25s", (circle > GetMaxCircle(target, 0)) ? "&+L" : "",
	    spells[spell]);

    if (IsMemmingClass(target)) {
      strcat(buf, (SpellInSpellBook(target, spell,
			     (SBOOK_MODE_IN_INV + SBOOK_MODE_AT_HAND))) ?
	     (circle > GetMaxCircle(target, 0)) ?
	     " [in spellbook, but too high level]" :
	     " [in spellbook]" : "");
      if (GET_SPELL(target, spell) == MEMINFO_CANTLEARN)
	strcat(buf, " [unlearnable]");
    }
    strcat(buf, "\r\n");
    strcat(buf1, buf);

  }
  if (!buf1[0]) {
    if (IS_PRIEST(target)) {
      strcpy(buf1, "You must worship a deity before you can get spells.\r\n");
    } else {
      strcpy(buf1, "Duh! You aren't of the casting type!\r\n");
    }
  }
  strcat(buf1, "\r\n");
  page_string(ch->desc, buf1, 1);
}


void display_mob_spells(struct char_data *target, struct char_data *ch)
{
  int spell, spl, circle, i;
  struct spl_list spell_list[NUM_SPELLS];

  *buf = '\0';
  *buf1 = '\0';

  get_char_cols(ch);

  /* first, build a list of all the spells this person can ever have. */
  i = 0;
  for (spell = 1; spell < NUM_SPELLS; spell++) {
    circle = GetSpellCircle(target, spell);
    if (GET_SPELL(target, spell) >= MEMINFO_READY &&
	(circle < MAX_CIRCLE + 1) && (circle > 0)) {
      spell_list[i].circle = circle;
      spell_list[i++].spell = spell;
    }
  }

  /* then sort the list... */
  qsort(spell_list, i, sizeof(struct spl_list), spell_cmp);


  /* finally, show it */

  for (spl = 0; spl < i; spl++) {
    int spell = spell_list[spl].spell;
    circle = spell_list[spl].circle;

    if (!spl || (circle != spell_list[spl - 1].circle)) {
      sprintf(buf, "&+B%d%s CIRCLE: (&n%d max&+B)&N\r\n", circle,
	      circle == 1 ? "st" : circle == 2 ? "nd" : circle == 3
	      ? "rd" : "th", MaxSpellsPerCircle(target, circle, IS_PRIEST(target)));
      strcat(buf1, buf);
    }
    sprintf(buf, "%s%-20s (%d)", (circle > GetMaxCircle(target, IS_PRIEST(target))) ? "&+L" : "",
	    spells[spell], GET_SPELL(target, spell) + 1 - MEMINFO_READY);

    strcat(buf, "\r\n");
    strcat(buf1, buf);

  }
  if (!buf1[0]) {
    return;
  }
  strcat(buf1, "\r\n");
  send_to_char(buf1, ch);
}
