/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#define PWIPE
#define NEW_BUFFERSoff
#define NEWMAP on
#include "ships.h"
#include "defines.h"


typedef signed char sbyte;
typedef unsigned char ubyte;
typedef signed short int sh_int;
typedef unsigned short int ush_int;
typedef char bool;
typedef enum { FTP_NORMAL, FTP_PUSH, FTP_PUSH_WAIT } ftp_mode;

#ifndef CIRCLE_WINDOWS
typedef char byte;
#endif

typedef int room_vnum;		/* A room's vnum type */
typedef int obj_vnum;		/* An object's vnum type */
typedef int mob_vnum;		/* A mob's vnum type */

typedef int room_rnum;		/* A room's real (internel) number type */
typedef int obj_rnum;		/* An object's real (internal) num type */
typedef int mob_rnum;		/* A mobile's real (internal) num type */

/* IH 05-Oct-98 Moved queue definition to queue.h */
struct q_element {
  void *data;
  long key;
  struct q_element *prev, *next;
};

struct event {
  EVENTFUNC(*func);
  void *event_obj;
  struct q_element *q_el;
};

/* Fighting event struct used in fighting engine */
struct fight_event_data {
  struct char_data *ch, *vict;
};

/* Water current event, sweeps them away */
struct water_current_event_obj {
  struct char_data *ch;		/* The character being swept away */
  struct obj_data *obj;		/* If its a boat/ship */
};

/* Falling of course */
struct falling_event_obj {
  struct char_data *ch;
  struct obj_data *obj;
  int speed;
  bool fatal;
};

/* blink event*/
struct blink_event_data {
  struct char_data *ch;
};

/* Extra description: used in objects, mobiles, and rooms */
struct extra_descr_data {
  char *keyword;		/* Keyword in look/examine          */
  char *description;		/* What to see                      */
  struct extra_descr_data *next;	/* Next in list                     */
};


/* object-related structures ***************************************** */


/* object flags; used in obj_data */
struct obj_flag_data {
  int value[8];			/* Values of the item (see list)    */
  byte type_flag;		/* Type of item                     */
  ulong wear_flags;		/* Where you can wear it            */
  ulong extra_flags;		/* If it hums, glows, etc.          */
  ulong anti_flags;		/* Specific anti-class and race     */
  ulong extra2_flags;		/* Range weapon-related flags       */
  byte size;			/* size of object (for race)        */
  byte space;			/* size of object (volume)          */
  sh_int race;
  sh_int condition;		/* items condition or level         */
  sh_int max_cond;		/* Items maximum condition	    */
  sh_int status;		/* brittle, melted, etc.            */
  sh_int material;		/* what its made of                 */
  sh_int craftsmanship;		/* how well made item is            */

  float weight;			/* Weight what else                 */
  int cost;			/* Value when sold (gp.)            */
  int orig_vnum;		/* Original prototypes vnum	    */
  int timer;			/* Timer for object                 */
  struct event *action;         /* event for action which takes time*/
  ulong bitvector;		/* To set chars bits                */
  ulong bitvector2;		/* 2nd 32 bits                      */
  ulong bitvector3;
  ulong bitvector4;
  ulong protection;
  ulong detecting;
  ulong controlling;

  int trap_eff;			/* trap effect type                 */
  int trap_dam;			/* trap damage type                 */
  sh_int trap_charge;		/* trap charges                     */
  sh_int trap_level;

  sh_int z_cord;
  sh_int x_cord;
  sh_int y_cord;
};


/* Used in obj_file_elem *DO*NOT*CHANGE* */
struct obj_affected_type {
  byte location;		/* Which ability to change (APPLY_XXX) */
  sbyte modifier;		/* How much it changes by              */
};


/* ================== Memory Structure for Objects ================== */
struct obj_data {
  obj_vnum item_number;		/* Where in data-base                   */
  long id;			/* Unique identifier                    */
  room_rnum in_room;		/* In what room -1 when conta/carr      */

  struct obj_flag_data obj_flags;	/* Object information               */
  struct obj_affected_type affected[MAX_OBJ_AFFECT];	/* affects */
  struct affected_type *spell_affects;  /* Temporary spell affects*/

  char *name;			/* Title of object :get etc.        */
  char *description;		/* When in room                     */
  char *short_description;	/* when worn/carry/in cont.         */
  char *action_description;	/* What to write when used          */
  struct extra_descr_data *ex_description;	/* extra descriptions     */
  struct char_data *carried_by;	/* Carried by :NULL in room/conta   */
  struct char_data *people;     /* List of NPC / PC in object       */
  struct char_data *worn_by;	/* Worn by?                         */
  sh_int worn_on;		/* Worn where?                      */
  
  struct char_data *orig_char;	/* polymorph object spell	    */
  long idnum_owner;

  struct obj_data *in_obj;	/* In what object NULL when none    */
  struct obj_data *contains;	/* Contains objects                 */

  struct trig_proto_list *proto_script; /* list of default triggers  */
  struct script_data *script;    /* script info for the object       */

  struct obj_data *next_content;	/* For 'contains' lists             */
  struct obj_data *next;	/* For the object list              */
};

/* ======================================================================= */


/* ====================== File Element for Objects ======================= */
/*                 BEWARE: Changing it will ruin rent files                */
struct obj_file_elem {
  obj_vnum item_number;

  sh_int locate;		/* that's the (1+)wear-location (when equipped) or */
  /* (20+)index in obj file (if it's in a container) BK */
  char name[128];
  char long_desc[512];
  char short_desc[128];
  char action_desc[256];
  int value[8];
  int wear;
  int type;
  ulong extra_flags;
  ulong extra2_flags;
  ulong anti_flags;

  byte size;			/* size of object (for race)        */
  byte space;			/* size of object (volume)          */
  sh_int race;

  sh_int condition;		/* items condition or level         */
  sh_int max_cond;		/* maximum condition of object	    */
  sh_int status;		/* brittle, melted, etc.            */
  sh_int material;		/* what its made of                 */
  sh_int craftsmanship;		/* how well made item is            */

  long idnum_owner;

  float weight;
  int cost;
  int timer;
  ulong bitvector;
  ulong bitvector2;
  ulong bitvector3;
  ulong bitvector4;
  struct obj_affected_type affected[MAX_OBJ_AFFECT];
};


/* header block for rent files.  BEWARE: Changing it will ruin rent files  */
struct rent_info {
  int time;
  int rentcode;
  int net_cost_per_diem;
  int nitems;
};

/* ======================================================================= */


/* room-related structures *********************************************** */


struct room_direction_data {
  char *general_description;	/* When look DIR.                   */

  char *keyword;		/* for open/close                       */

  int exit_info;		/* Exit info                            */
  obj_vnum key;			/* Key's number (-1 for no key)         */
  room_rnum to_room;		/* Where direction leads (NOWHERE)      */
  sh_int hitpoints;		/* For damaging them			*/
  sh_int x_cord;		/* Heighth of door from floor           */
  sh_int y_cord;
  sh_int z_cord;
};

/* this is done so I can allocate/deallocate on the fly */
struct map_data {
#ifdef NEWMAP
  sh_int **map_grid;
#else
  sh_int map_grid[51][51];
#endif
};

/* ================== Memory Structure for room ======================= */
struct room_data {
  room_vnum number;		/* Rooms number (vnum)                */
  long id;			/* Unique identifier                  */
  sh_int zone;			/* Room zone (for resetting)          */
  int sector_type;		/* sector type (move/hide)            */
  char *name;			/* Rooms name 'You are ...'           */
  char *description;		/* Shown when entered                 */
  struct extra_descr_data *ex_description;	/* for examine/look       */
  struct room_direction_data *dir_option[NUM_OF_DIRS];	/* Directions */
  int room_flags;		/* DEATH,DARK ... etc                 */
  int resources;
  int shape;			/* circular, square, etc              */
  sh_int x_size;		/* N/S measurement                    */
  sh_int y_size;		/* E/W measurement                    */
  sh_int z_size;
  sh_int kingdom_num;
  sh_int kingdom_type;
  sh_int current_speed;
  sh_int current_direction;
  sh_int chance_fall;
  struct ship_data *ship;       /* Holder for ship data */
  struct map_data *map;		/* holder for the grid			*/
  int map_specials;		/* display items like columns, rivers */
  byte light;			/* Number of lightsources in room     */
   SPECIAL(*func);

  struct trig_proto_list *proto_script; /* list of default triggers  */
  struct script_data *script;  /* script info for the object         */ 

  struct obj_data *contents;	/* List of items in room              */
  struct char_data *people;	/* List of NPC / PC in room           */
  struct room_affected_type *spell_affects;	/*list of spell affects and locations */
};

/* ====================================================================== */


/* char-related structures *********************************************** */

struct skill_usage_struct {
  time_t time_of_first_use;
  byte times_used;
};

struct skill_usage_chart_struct {
  byte max_can_use;
  time_t time_period_per_use;
};

/* memory structure for characters */
struct memory_rec_struct {
  long id;
  char *name;
  struct memory_rec_struct *next;
};

typedef struct memory_rec_struct memory_rec;



/* This structure is purely intended to be an easy way to transfer */
/* and return information about time (real or mudwise).            */
struct time_info_data {
  int hours, day, month, minutes;
  sh_int year;
};


/* These data contain information about a players time data */
struct time_data {
  time_t birth;			/* This represents the characters age                */
  time_t logon;			/* Time of the last logon (used to calculate played) */
  int played;			/* This is the total accumulated time played in secs */
};

/* data structure for justice witness record as held in memory.  This
   struct ends up being a linked list.. */
struct witness_data {
  time_t time;                  /* When did it happen? */
  char *attacker;               /* who did it? */
  char *victim;                 /* who did they do it to? */
  ubyte crime;                  /* what did they do? */
  int room;                     /* Where did they do it?  (VIRTUAL!) */
  struct witness_data *next;    /* next record (or NULL if none) */
};

/* general player-related info, usually PC's and NPC's */
struct char_player_data {
  char passwd[MAX_PWD_LENGTH + 1];	/* character's password      */
  char *name;			/* PC / NPC s name (kill ...  )         */
  char *short_descr;		/* for NPC 'actions'                    */
  char *long_descr;		/* for 'look'                           */
  char *description;		/* Extra descriptions                   */
  char *title;			/* PC / NPC's title                     */
  byte sex;			/* PC / NPC's sex                       */
  byte class;			/* PC / NPC's class                     */
  sh_int race;			/* PC / NPC's race                      */
  byte level;			/* PC / NPC's level                     */
  byte size;			/* PC / NPC's size                      */
  sh_int hometown;			/* PC s Hometown (zone)                 */
  room_vnum birthplace;		/* birth room                           */
  struct time_data time;	/* PC's AGE in days                 */
  ubyte weight;			/* PC / NPC's weight                    */
  ubyte height;			/* PC / NPC's height                    */
};

struct player_disguise_data {
  bool active_pc;		/* is he disguise                       */
  bool active_npc;
  char *name;			/* PC name                              */
  int class;			/* class                                */
  sh_int race;			/* race                                 */
  ubyte level;			/* level                                */
  char *title;			/* title                                */
};

/* Char's abilities.  Used in char_file_u *DO*NOT*CHANGE* */
struct char_ability_data {
  sbyte str;
  sbyte str_add;		/* 000 - 100 if strength 18             */
  sbyte intel;
  sbyte wis;
  sbyte dex;
  sbyte con;
  sbyte cha;
};

/*body part hp*/
struct char_body_hitpoints {
  int head;
#ifdef PWIPE
  int head2;
  int wings;
  int tail;
#endif
  int shield_arm;
  int shield_hand;
  int weapon_hand;
  int weapon_arm;
  int torso;
  int right_leg;
  int left_leg;
  int right_foot;
  int left_foot;
};

/* Char's points.  Used in char_file_u *DO*NOT*CHANGE* */
struct char_point_data {
  sh_int hit;
  sh_int max_hit;		/* Max hit for PC/NPC                      */

  struct char_body_hitpoints body_points;

  sh_int vitality;
  sh_int max_vitality;		/*How long can you go without falling asleep */

  sh_int armor;			/* Internal -100..100, external -10..10 AC */
  sh_int gold;			/* temp holder of gold, changed to obj later */
  int exp;			/* The experience of the player            */

  sbyte hitroll;		/* Any bonus or penalty to the hit roll    */
  sbyte damroll;		/* Any bonus or penalty to the damage roll */
#ifdef PWIPE
  sh_int align_ge;		/* These two are alignment		*/
  sh_int align_lc;
#endif
};

/* 
 * char_special_data_saved: specials which both a PC and an NPC have in
 * common, but which must be saved to the playerfile for PC's.
 *
 * WARNING:  Do not change this structure.  Doing so will ruin the
 * playerfile.  If you want to add to the playerfile, use the spares
 * in player_special_data.
 */
struct char_special_data_saved {
#ifndef PWIPE
	/* remove me, use new method elsewhere */
  int alignment;		/*   */
#endif
#ifdef PWIPE
  struct witness_data *witnessed;           /* linked list of witness records */
  sh_int jail_time;
  struct char_data *arrest_by;
  struct char_data *arrest_link;
  struct char_data *witnessing;
  struct char_data *witness_vict;
  int witness_cmd;
#endif
  long idnum;			/* player's idnum; -1 for mobiles       */
  long act;			/* act flag for NPC's; player flag for PC's */
  byte spells[MAX_SPELLS + 1];	/* array of skills plus skill 0         */
  byte skills[MAX_SKILLS + 1];	/* Skills */
  ulong granted_one;		/* commands granted                     */
  ulong granted_two;		/* commands granted                     */
  ulong affected_by;		/* Bitvector for spells/skills affected by */
  ulong affected_by2;		/* Bitvector for spells/skills affected by */
  ulong affected_by3;
  ulong affected_by4;
#ifdef PWIPE
  ulong protection;		/* protection from gas, etc 		*/
  ulong detecting;		/* for all those detect spells		*/
  ulong controlling;		/* potion of human control, etc 	*/
#endif
  byte disease_type[5];
  byte disease_dura[5];
  byte weapon_prof;		/* weapon proficiency */
  byte non_weapon_prof;		/* non-weapon proficiency */
  byte high_level;
  int thief_skill;
  byte specialist;		/* if mage is specialist */
  ulong mage_school;		/* list of mage scholl available to player */
  ulong priest_sphere_major;	/* list of priest major sphere available to player */
  ulong priest_sphere_minor;	/* list of priest minor sphere available to player */
  int priest_deity;		/* priest deity */

  sh_int apply_saving_throw[5];	/* Saving throw (Bonuses)              */
#ifdef PWIPE
  struct skill_usage_struct skill_usage[MAX_SKILL_USAGE]; /* skills usable 1/day, etc */
#endif
};

/* Special playing constants shared by PCs and NPCs which aren't in pfile */
struct char_special_data {
  struct char_data *fighting;	/* Opponent                             */
  struct char_data *hunting;	/* Char hunted by this char             */
  struct event *blink_event;   	/* Event for blinking                   */
  struct char_data *riding;	/* Who are they riding? 		*/
  struct char_data *ridden_by;	/* Who is riding them? 			*/
  struct obj_data *vehicle;	/* cart, wagon			 */
  struct obj_data *furniture;   /* What they are sitting, sleeping on   */
  struct obj_data *polymorph;	/* Object they've polymorphed into	*/

  byte attacks_left;		/* How many more attacks to figure in
				   subsequent hit calls                 */
  sh_int position;			/* Standing, sleeping, etc.   */

  byte magic_resistance;	/* magic resiatnce */
  int carry_weight;		/* Carried weight                       */
  byte carry_items;		/* Number of items carried              */
  int timer;			/* Timer for update                     */
  int target_room;
  int target_location;  	/*where are we targetting? */
  sh_int style; 			/*aggressive, passive, normal*/
  byte s_o_t; 			/*slash or thrust :P*/
  struct spell_data *spell_mem_list;
  struct event *action;		/* event for action which takes time    */
  struct char_special_data_saved saved;		/* constants saved in plrfile  */

};


/*
 *  If you want to add new values to the playerfile, do it here.  DO NOT
 * ADD, DELETE OR MOVE ANY OF THE VARIABLES - doing so will change the
 * size of the structure and ruin the playerfile.  However, you can change
 * the names of the spares to something more meaningful, and then use them
 * in your new code.  They will automatically be transferred from the
 * playerfile into memory when players log in.
 */
struct player_special_data_saved {
  bool talks[MAX_TONGUE];	/* PC s Tongues 0 for NPC               */
  sh_int wimp_level;		/* Below this # of hit points, flee!    */
  byte freeze_level;		/* Level of god who froze char, if any  */
  sh_int invis_level;		/* level of invisibility                */
  room_vnum load_room;		/* Which room to place char in          */
  long pref;			/* preference flags for PC's.           */
  ubyte bad_pws;		/* number of bad password attemps       */
  sbyte conditions[3];		/* Drunk, full, thirsty                 */
  char poofin[POOF_LENGTH + 1];
  char poofout[POOF_LENGTH + 1];
  long int introd_list[MAX_INTRO];
  unsigned long int introd_times[MAX_INTRO];
  char *last_host;
  room_vnum known_rooms[25];

  /* spares below for future expansion.  You can change the names from
     'sparen' to something meaningful, but don't change the order.  */

  ubyte angle_type;
  ubyte facing;
  ubyte dominant_hand;
  ubyte spare3;
  ubyte spare4;
  ubyte spare5;
  int spare6;
  int olc_zone;
  int guildnum;			/* Index number of guild you belong to  */
  int guildrank;		/* Rank within guild                    */
  int multi_flags;		/* Class you've switched to             */
  int total_level;		/* Total levels gained, with above      */
  int prompt;
  int x_cord;
  int y_cord;
  int z_cord;
  int spare14;
  int spare15;
  int spare16;
  long law_flags;		/* wanted, citizen, etc			*/
  long spare18;
};

/*
 * Specials needed only by PCs, not NPCs.  Space for this structure is
 * not allocated in memory for NPCs, but it is for PCs and the portion
 * of it labelled 'saved' is saved in the playerfile.  This structure can
 * be changed freely; beware, though, that changing the contents of
 * player_special_data_saved will corrupt the playerfile.
 */
struct player_special_data {
  struct player_special_data_saved saved;

  char *poofin;			/* Description on arrival of a god.     */
  char *poofout;		/* Description upon a god's exit.       */
  struct alias *aliases;	/* Character's aliases                  */
  long last_tell;		/* idnum of last tell from              */
  void *last_olc_targ;		/* olc control                          */
  sh_int last_olc_mode;		/* olc control                          */
};

struct mob_attack_data {
  int min_damage;
  int max_damage;
  int attack_type;
  struct mob_attack_data *next;
};


/* Specials used by NPCs, not PCs */
struct mob_special_data {
  int attack_type;		/* The Attack Type Bitvector for NPC's     */
  byte default_pos;		/* Default position for NPC                */
  memory_rec *memory;		/* List of attackers to remember           */
  byte damnodice;		/* The number of damage dice's             */
  byte damsizedice;		/* The size of the damage dice's           */
  byte hp_die;			/* what die mob rolls for hit points       */
  byte hp_plus;			/* additional bonus to hit points          */
  int wait_state;		/* Wait state for bashed mobs              */
  int orig_vnum;		/* mob is a copy of vnum xx 		   */
  int special[2];               /* For special proc use only    	*/
  struct char_data *orig_char;	/* used instead of memory ptr to keep
				   track of who is controlling the mob  */
  struct mob_attack_data *mob_attack;
  byte last_direction;		/* last dir moved			*/
  struct event *movement;	/* event for movement which takes time  */
  byte thac0;
  sh_int treasure_type;
  int last_attack;
  int last_attack_ok;
  sh_int x_cord;		/* duped, cause players need them saved */
  sh_int y_cord;
  sh_int z_cord;
  sh_int facing;
  ubyte dominant_hand;
};

/* Affected Structure for rooms */
struct room_affected_type{
  sh_int level;			/*level of caster of spell on room*/
  long id;			/*who cast this*/
  sh_int type; 			/*The type of spell that caused this*/
  sh_int duration;		/*How long the spell effects will last*/
  struct location_data *loc;	/*The origin (location) of the spell*/
  ulong bitvector;		/*Which room flags it sets*/
  struct event *affect_event;	/*Any associated event for this spell*/
  struct room_affected_type *next;	/*next in list*/
}; 

/* An affect structure.  Used in char_file_u *DO*NOT*CHANGE* */
struct affected_type {
  sh_int level;			/* level of caster*/
  sh_int type;			/* The type of spell that caused this      */
  sh_int duration;		/* For how long its effects will last      */
  int modifier;			/* This is added to apropriate ability     */
  byte location;		/* Tells which ability to change(APPLY_XXX) */
  ulong bitvector;		/* Tells which bits to set (AFF_XXX)       */
  ulong bitvector2;		/* Tells which bits to set (AFF2_XXX)      */
  ulong bitvector3;
  ulong bitvector4;
  ulong protection;		/* protection from gas, etc 		*/
  ulong detecting;		/* for all those detect spells		*/
  ulong controlling;		/* potion of human control, etc 	*/
  struct event *affect_event;	/* The event for this affect         */

  struct affected_type *next;
};

/* Structure used for chars following other chars */
struct follow_type {
  struct char_data *follower;
  struct follow_type *next;
};

/* Structure used for grouping */
struct group_type{
  struct char_data *follower;
  struct group_type *next;
};

struct watch_data {
   struct char_data *ch;
   struct char_data *vict;
   struct event *event;
};

/* ================== Structure for player/non-player ===================== */
struct char_data {
  int pfilepos;			/* playerfile pos                */
  sh_int nr;			/* Mob's rnum                    */
  long id;			/* Unique identifier             */
  room_rnum in_room;		/* Location (real room number)   */
  room_rnum was_in_room;	/* location for linkdead people  */

  struct char_player_data player;	/* Normal data                   */
  struct char_ability_data real_abils;	/* Abilities without modifiers   */
  struct char_ability_data aff_abils;	/* Abils with spells/stones/etc  */
  struct char_point_data points;	/* Points                        */
  struct char_special_data char_specials;	/* PC/NPC specials        */
  struct player_special_data *player_specials;	/* PC specials            */
  struct mob_special_data mob_specials;		/* NPC specials           */

  struct affected_type *affected;	/* affected by what spells       */
  struct obj_data *equipment[NUM_WEARS];	/* Equipment array               */

  struct obj_data *carrying;	/* Head of list                  */

  struct descriptor_data *desc;	/* NULL for mobiles              */

  struct trig_proto_list *proto_script; /* list of default triggers      */
  struct script_data *script;         	/* script info for the object    */
  struct script_memory *memory;		/* for trigger memory		 */

  struct char_data *next_in_room;	/* For room->people - list        */
  struct char_data *next;		/* For either monster or ppl-list */
  struct char_data *next_in_furniture; 	/* For obj->people - list         */

  sh_int *completed_quests;	/* Quests completed               */
  int num_completed_quests;	/* Number completed               */
  int current_quest;		/* vnum of current quest          */

  struct player_disguise_data disguise;
  struct char_data *switched;

  struct follow_type *followers;	/* List of chars followers        */
   struct follow_type *watchers;
   struct group_type *group;
  struct char_data *leader;     /* Who is the characters group leader? */
  struct char_data *master;	/* Who is char following?         */
   struct watch_data *watching;
  struct event *points_event[3];	/* events for regening H/M/V      */
  int mpactnum;
};

/* ====================================================================== */


/* ==================== File Structure for Player ======================= */
/*             BEWARE: Changing it will ruin the playerfile               */
struct char_file_u {
  /* char_player_data */
  char name[MAX_NAME_LENGTH + 1];
  char description[EXDSCR_LENGTH];
  char short_descr[MAX_SHORTDESC_LENGTH + 1];
  char title[MAX_TITLE_LENGTH + 1];
  byte sex;
  byte class;
  sh_int race;
  byte level;
  byte size;
  sh_int hometown;
  time_t birth;			/* Time of birth of character     */
  int played;			/* Number of secs played in total */
  ubyte weight;
  ubyte height;

  char pwd[MAX_PWD_LENGTH + 1];	/* character's password */

  struct char_special_data_saved char_specials_saved;
  struct player_special_data_saved player_specials_saved;
  struct char_ability_data abilities;
  struct char_point_data points;
  struct affected_type affected[MAX_AFFECT];

  sh_int completed_quests[MAX_COMPLETED_QUESTS];
  sh_int current_quest;

  time_t last_logon;		/* Time (in secs) of last logon */
  char host[HOST_LENGTH + 1];	/* host of last logon */
};

/* ====================================================================== */


/* descriptor-related structures ***************************************** */


struct txt_block {
  char *text;
  int aliased;
  struct txt_block *next;
};


struct txt_q {
  struct txt_block *head;
  struct txt_block *tail;
};


struct descriptor_data {
  socket_t descriptor;		/* file descriptor for socket           */
  socket_t ident_sock;		/* socket used for ident process        */
  u_short  peer_port;		/* port of peer               		*/
  char host[HOST_LENGTH + 1];	/* hostname                             */
  byte close_me;		/* flag: this desc. should be closed    */
  byte bad_pws;			/* number of bad pw attemps this login  */
  byte idle_tics;		/* tics idle at password prompt         */
  int connected;		/* mode of 'connectedness'              */
  int wait;			/* wait for how many loops              */
  int desc_num;			/* unique num assigned to desc          */
  time_t login_time;		/* when the person connected            */
  char *showstr_head;		/* for keeping track of an internal str */
  char **showstr_vector;	/* for paging through texts             */
  int showstr_count;		/* number of pages to page through      */
  int showstr_page;		/* which page are we currently showing? */
  char **str;			/* for the modify-str system            */
  size_t max_str;		/*              -                       */
  char *backstr;		/* added for handling abort buffers     */
  long mail_to;			/* name for mail system                 */
  int has_prompt;		/* is the user at a prompt?             */
  char inbuf[MAX_RAW_INPUT_LENGTH];	/* buffer for raw input           */
  char last_input[MAX_INPUT_LENGTH];	/* the last input                 */
  char small_outbuf[SMALL_BUFSIZE];	/* standard output buffer         */
  char *output;			/* ptr to the current output buffer     */
  int bufptr;			/* ptr to end of current output         */
  int bufspace;			/* space left in the output buffer      */
#ifdef NEW_BUFFERS
  char *large_outbuf;		/* ptr to large buffer, if we need it   */
#else
  struct txt_block *large_outbuf; /* ptr to large buffer, if we need it */
#endif
  struct txt_q input;		/* q of unprocessed input               */
  struct char_data *character;	/* linked to char                       */
  struct char_data *original;	/* original char if switched            */
  struct descriptor_data *snooping;	/* Who is this char snooping       */
  struct descriptor_data *snoop_by;	/* And who is snooping this char   */
  struct descriptor_data *next;	/* link to next descriptor             */
  struct olc_data *olc;		/*. OLC info - defined in olc.h   . */
  struct abil_data *stat;	/*Order of stats */
  byte term_type;		/* terminal type */
   /* mudFTP control. */
   struct {
     char *filename;		/* Filename being written to */
     char *data;		/* Data being written */
     sh_int lines_left;		/* Lines left */
     sh_int lines_sent;		/* Lines sent so far */
     ftp_mode mode;		/* FTP_xxx */
   } ftp;
   byte view;			/* viewing distance, gods may adjust 	*/
   byte screen_length;		/* For those with larger monitors	*/
   char *username;
};


/* other miscellaneous structures ************************************** */

struct grantset_struct {
  char *cmd;
  char level;
  char pcnpc;
  char type;
  int bit;
};

struct msg_type {
  char *attacker_msg;		/* message to attacker */
  char *victim_msg;		/* message to victim   */
  char *room_msg;		/* message to room     */
};


struct message_type {
  struct msg_type die_msg;	/* messages when death                  */
  struct msg_type miss_msg;	/* messages when miss                   */
  struct msg_type hit_msg;	/* messages when hit                    */
  struct msg_type god_msg;	/* messages when hit on god             */
  struct message_type *next;	/* to next messages of this kind.       */
};


struct message_list {
  int a_type;			/* Attack type                          */
  int number_of_attacks;	/* How many attack messages to chose from. */
  struct message_type *msg;	/* List of messages.                    */
};

struct armor_apply_type {
  sh_int pd;
  sh_int dr;
};

struct body_armor_type {
  struct armor_apply_type torso;
  struct armor_apply_type arms;
  struct armor_apply_type legs;
};

struct weapon_app_type {
  sh_int mod;
  sh_int type;
  sh_int swing;
  sh_int thrust;
  sh_int range;
  sh_int parry;
  sh_int max_die;
};

struct dex_skill_type {
  sh_int p_pocket;
  sh_int p_locks;
  sh_int traps;
  sh_int sneak;
  sh_int hide;
};


struct dex_app_type {
  sh_int reaction;
  sh_int miss_att;
  sh_int defensive;
};


struct str_app_type {
  sh_int thrust;		/* To Hit (THAC0) Bonus/Penalty        */
  sh_int swing;			/* Damage Bonus/Penalty                */
  sh_int carry_w;		/* Maximum weight that can be carrried */
  sh_int wield_w;		/* Maximum weight that can be wielded  */
};


struct wis_app_type {
  byte def_adj;
  byte chance_fail;
};


struct int_app_type {
  byte max_lang;
  byte max_spell_lvl;
  byte chance_learn;
  byte max_nb_spell_lvl;
};


struct con_app_type {
  sh_int hitp;
  sh_int shock;
  sh_int special;
};


struct title_type {
  char *title_m;
  char *title_f;
  int exp;
};


/* element in monster and object index-tables   */
struct index_data {
  int virtual;			/* virtual number of this mob/obj           */
  int number;			/* number of existing units of this mob/obj */
  struct path_info *paths;
  char *farg;         /* string argument for special function     */
  struct trig_data *proto;     /* for triggers... the trigger     */
   SPECIAL(*func);
};

/* linked list for mob/object prototype trigger lists */
struct trig_proto_list {
  int vnum;                             /* vnum of the trigger   */
  struct trig_proto_list *next;         /* next trigger          */
};

struct path_perform {
  int hrmin;
  int day;
  int week;
  int month;
};

struct path_info {
  int type;			/* action, goto, etc.. */
  struct path_perform *performance;	/* daily, weekly, monthly */
  int target_room;		/* not necessarily used */
  char *arguments;		/* also, not necessarily used */
  struct path_info *next;
};

/* this is for the memory queue */
struct path_q {
  int pnum;
  struct char_data *mob;
  struct path_q *next;
};

/* used in the socials */
struct social_messg {
  int act_nr;
  char *command;		/* holds copy of activating command */
  char *sort_as;		/* holds a copy of a similar command or
				 * abbreviation to sort by for the parser */
  int hide;			/* ? */
  int min_victim_position;	/* Position of victim */
  int min_char_position;	/* Position of char */
  int min_level_char;		/* Minimum level of socialing char */

  /* No argument was supplied */
  char *char_no_arg;
  char *others_no_arg;

  /* An argument was there, and a victim was found */
  char *char_found;
  char *others_found;
  char *vict_found;

  /* An argument was there, as well as a body part, and a victim was found */
  char *char_body_found;
  char *others_body_found;
  char *vict_body_found;

  /* An argument was there, but no victim was found */
  char *not_found;

  /* The victim turned out to be the character */
  char *char_auto;
  char *others_auto;

  /* If the char cant be found search the char's inven and do these: */
  char *char_obj_found;
  char *others_obj_found;
};


struct location_data {
  int real_room;
  sh_int x_cord;
  sh_int y_cord;
  sh_int z_cord;
  sh_int angle;
};

struct deity_data {
  char *deity_name;
  char *deity_pantheon;
  ulong deity_major_sphere;
  ulong deity_minor_sphere;
  byte aligns[9];
};

struct scribing_data_type {
  struct char_data *ch;
  struct obj_data *book;
  union {
    struct obj_data *obj;
    struct char_data *teacher;
  }
  source;

  int spell;
  int pagetime;    /* in 1/4 seconds, RL / 1 pulses - time to
                      complete one page of the spell */
  int page;        /* page number last written - for event-wise
                      way we do these things nowadays. */
  int flag;        /* 0, 1, or 2. 0=teach, 1=spellbook, 2=scroll */
  struct scribing_data_type *next;
};


struct spl_list {
  int circle;
  int spell;
};


struct spell_event {
  struct char_data *ch;
  long target;
  byte in_loc;
  struct location_data location;
  char arg[MAX_INPUT_LENGTH];
  int spellnum;
};
